// ************************************************************************
// **   FILENAME:          D42LIB.CPP
// **
// **   CREATED:           4/8/96
// **   LAST MODIFIED:     8/20/99
// **
// **   VERSION:           4.0
// **   MODULE:            D42LIB.CPP
// **
// **   DESCRIPTION:       Dialogic D/42 Library Compatability Module   
// **
// **   Copyright (c) 1998 Dialogic Corp.
// **   RRM 3/6/98 Added new functions for DNA3
// **   WRK 8/20/99 Corrected function definitions for d42_brdstatus() and
// **               d42_openfeaturesession()
// ************************************************************************

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <sys\stat.h>
#include "srllib.h"
#include "dxxxlib.h"
#include "d42lib.h"

// prototypes for 'not implemented' functions
static long d42_long_notimp  (int);
static int  d42_int_notimp   (int);
static void d42_void_notimp  (int);
static char *d42_null_notimp (int);

#ifdef __cplusplus
extern "C" {				// C Plus Plus function bindings
#define extern
#endif

long d42_libinit (unsigned short);
long d42_GetDLLVersion (LPDWORD, LPDWORD);

#ifdef __cplusplus
}				// C Plus Plus function bindings
#undef extern
#endif


// typedefs and #defines
typedef void (*func_ptr)(void);

struct func_table
{
  char *fname;
  func_ptr fptr;
};

typedef enum _FUNCPTR_IDS
{
  ATD4_BDTYPE_ID = 0,
  ATD4_CHTYPE_ID,
  d42_brdstatus_ID,
  d42_display_ID,
  d42_flags_ID,
  d42_getparm_ID,
  d42_getver_ID,
  d42_gtcallid_ID,
  d42_indicators_ID,
  d42_lcdprompt_ID,
  d42_setparm_ID,
  d42_chnstatus_ID,
  d42_closefeaturesession_ID,
  d42_openfeaturesession_ID,
  d42_writetodisplay_ID,
  d42_longbeep_ID,
  d42_putpbxmsg_ID
} FUNCPTR_IDS;

typedef int       (*ptr_ATD4_BDTYPE)    (int);
typedef int       (*ptr_ATD4_CHTYPE)    (int);
typedef int       (*ptr_d42_brdstatus)  (int, char *, char *);
typedef int       (*ptr_d42_display)    (int, char *);
typedef int       (*ptr_d42_flags)      (int, char *);
typedef int       (*ptr_d42_getparm)    (int, int, void *);
typedef int       (*ptr_d42_getver)     (int, char *, int);
typedef int       (*ptr_d42_gtcallid)   (int, char *);
typedef int       (*ptr_d42_indicators) (int, char *);
typedef int       (*ptr_d42_lcdprompt)  (int, char *);
typedef int       (*ptr_d42_setparm)    (int, int, void *);
typedef int       (*ptr_d42_chnstatus)  (int, char*, char*);
typedef int       (*ptr_d42_closefeaturesession) (int);
typedef int       (*ptr_d42_openfeaturesession)  (int, char *, int *, int);
typedef int       (*ptr_d42_writetodisplay) (int, char*);
typedef int       (*ptr_d42_longbeep)       (int);
typedef int       (*ptr_d42_putpbxmsg)      (int, char*, char);

// Global variables
static struct func_table D42Func[] =
{
  {"ATD4_BDTYPE",   (func_ptr)d42_int_notimp},
  {"ATD4_CHTYPE",   (func_ptr)d42_int_notimp},
  {"d42_brdstatus", (func_ptr)d42_int_notimp},
  {"d42_display",   (func_ptr)d42_int_notimp},
  {"d42_flags",     (func_ptr)d42_int_notimp},
  {"d42_getparm",   (func_ptr)d42_int_notimp},
  {"d42_getver",    (func_ptr)d42_int_notimp},
  {"d42_gtcallid",  (func_ptr)d42_int_notimp},
  {"d42_indicators",(func_ptr)d42_int_notimp},
  {"d42_lcdprompt", (func_ptr)d42_int_notimp},
  {"d42_setparm",   (func_ptr)d42_int_notimp},
  {"d42_chnstatus", (func_ptr)d42_int_notimp},
  {"d42_closefeaturesession", (func_ptr)d42_int_notimp},
  {"d42_openfeaturesession",  (func_ptr)d42_int_notimp},
  {"d42_putpbxmsg",      (func_ptr)d42_int_notimp},
  {"d42_writetodisplay", (func_ptr)d42_int_notimp},
  {"d42_longbeep",       (func_ptr)d42_int_notimp}
};

char caMTLibName[] = "LIBD42MT.DLL";
char caSTLibName[] = "LIBD42.DLL";
char *pcLibName;

long d42_libinit (unsigned short usFlag)
{
  struct func_table *pD42 = D42Func;
  func_ptr pProcAddress;
  static HINSTANCE hLibrary = NULL;
  DWORD dwNumberOfFunctions;
  DWORD dwFileVer, dwProdVer;

  // Check if it's already open
  if (hLibrary != (HINSTANCE)NULL)
  {
    return -1;
  }

  // Determine which library to load
  if ((usFlag & DLGC_MT) == DLGC_MT)
  {
    pcLibName = caMTLibName;
  }
  else
  {
    pcLibName = caMTLibName;
    d42_GetDLLVersion (&dwFileVer, &dwProdVer);
    if (dwFileVer <= 0x0004000A)
    {
      pcLibName = caSTLibName;
    }
  }

  dwNumberOfFunctions = sizeof (D42Func) / sizeof (D42Func[0]);

  // Try to load the library
  hLibrary = LoadLibrary (pcLibName);
  if (hLibrary == (HINSTANCE)NULL)
  {
    return 0;
  }

  // Set up each entry to a default not implemented
  // function and then overwrite the true address if the 
  // function is actually present in the DLL
  while (dwNumberOfFunctions--)
  {
    pProcAddress = (func_ptr)GetProcAddress (hLibrary, pD42->fname);
    if (pProcAddress)
    {
      pD42->fptr = pProcAddress;
    }
    else
    {
      //sprintf (caTemp, "Function %s not implemented, error %X.", pFax->fname, GetLastError());
      //MessageBox (HWND_DESKTOP, caTemp, "CCLIB", MB_ICONINFORMATION | MB_OK);
    }
    pD42++;
  }

  return 0;
}

long d42_GetDLLVersion (LPDWORD pdwFileVer, LPDWORD pdwProdVer)
{
  return ( sr_GetVersion (pcLibName, pdwFileVer, pdwProdVer) );
}

int d42_set_lasterr (int iErr, long lError)
{
  dx_set_lasterr (iErr, lError);
  return 0;
}

long d42_long_notimp (int iErr)
{
  d42_set_lasterr (iErr, ED42_NOTIMP);
  return -1;
}

int d42_int_notimp (int iErr)
{
  d42_set_lasterr (iErr, ED42_NOTIMP);
  return -1;
}

void d42_void_notimp (int iErr)
{
  d42_set_lasterr (iErr, ED42_NOTIMP);
  return;
}

char *d42_null_notimp (int iErr)
{
  d42_set_lasterr (iErr, ED42_NOTIMP);
  return (char *) NULL;
}

int d42_dflt_errno (void)
{
  return errno;
}

int ATD4_BDTYPE (int iBoard)
{
  ptr_ATD4_BDTYPE p;

  p = (ptr_ATD4_BDTYPE) *D42Func[ATD4_BDTYPE_ID].fptr;
  return ( (*p) (iBoard));
}

int ATD4_CHTYPE (int iChannel)
{
  ptr_ATD4_CHTYPE p;

  p = (ptr_ATD4_CHTYPE) *D42Func[ATD4_CHTYPE_ID].fptr;
  return ( (*p) (iChannel));
}

int d42_brdstatus (int iBoard, char *pcStatus, char *pcBuffer)
{
  ptr_d42_brdstatus p;

  p = (ptr_d42_brdstatus) *D42Func[d42_brdstatus_ID].fptr;
  return ( (*p) (iBoard, pcStatus, pcBuffer));
}

int d42_display (int iChannel, char *pcBuffer)
{
  ptr_d42_display p;

  p = (ptr_d42_display) *D42Func[d42_display_ID].fptr;
  return ( (*p) (iChannel, pcBuffer));
}

int d42_flags (int iChannel, char *pcBuffer)
{
  ptr_d42_flags p;

  p = (ptr_d42_flags) *D42Func[d42_flags_ID].fptr;
  return ( (*p) (iChannel, pcBuffer));
}

int d42_getparm (int iChannel, int iParm, void *pValue)
{
  ptr_d42_getparm p;

  p = (ptr_d42_getparm) *D42Func[d42_getparm_ID].fptr;
  return ( (*p) (iChannel, iParm, pValue));
}

int d42_getver (int iChannel, char *pcBuffer, int iFlag)
{
  ptr_d42_getver p;

  p = (ptr_d42_getver) *D42Func[d42_getver_ID].fptr;
  return ( (*p) (iChannel, pcBuffer, iFlag));
}

int d42_gtcallid (int iChannel, char *pcBuffer)
{
  ptr_d42_gtcallid p;

  p = (ptr_d42_gtcallid) *D42Func[d42_gtcallid_ID].fptr;
  return ( (*p) (iChannel, pcBuffer));
}

int d42_indicators (int iChannel, char *pcBuffer)
{
  ptr_d42_indicators p;

  p = (ptr_d42_indicators) *D42Func[d42_indicators_ID].fptr;
  return ( (*p) (iChannel, pcBuffer));
}

int d42_lcdprompt (int iChannel, char *pcBuffer)
{
  ptr_d42_lcdprompt p;

  p = (ptr_d42_lcdprompt) *D42Func[d42_lcdprompt_ID].fptr;
  return ( (*p) (iChannel, pcBuffer));
}

int d42_setparm (int iChannel, int iParm, void *pValue)
{
  ptr_d42_setparm p;

  p = (ptr_d42_setparm) *D42Func[d42_setparm_ID].fptr;
  return ((*p) (iChannel, iParm, pValue));
}

int d42_chnstatus ( int SrlDevice, char *bufp, char *buffer )
{
  ptr_d42_chnstatus p;

  p = (ptr_d42_chnstatus) *D42Func[d42_chnstatus_ID].fptr;
  return ((*p) (SrlDevice, bufp, buffer));
}

int d42_closefeaturesession ( int SrlDevice )
{
  ptr_d42_closefeaturesession p;

  p = (ptr_d42_closefeaturesession) *D42Func[d42_closefeaturesession_ID].fptr;
  return ((*p) (SrlDevice));
}

int d42_openfeaturesession ( int SrlDevice, char *szDnNumber, int* piTerminalType, int iEvtMask )
{
  ptr_d42_openfeaturesession p;

  p = (ptr_d42_openfeaturesession) *D42Func[d42_openfeaturesession_ID].fptr;
  return ((*p) (SrlDevice, szDnNumber, piTerminalType, iEvtMask));
}

int d42_putpbxmsg ( int SrlDevice, char* bufp, char msglen )
{
  ptr_d42_putpbxmsg p;

  p = (ptr_d42_putpbxmsg) *D42Func[d42_putpbxmsg_ID].fptr;
  return ((*p) (SrlDevice, bufp, msglen));
}

int d42_writetodisplay ( int SrlDevice, char* bufp )
{
  ptr_d42_writetodisplay p;

  p = (ptr_d42_writetodisplay) *D42Func[d42_writetodisplay_ID].fptr;
  return ((*p) (SrlDevice, bufp));
}

int d42_longbeep ( int SrlDevice )
{
  ptr_d42_longbeep p;

  p = (ptr_d42_longbeep) *D42Func[d42_longbeep_ID].fptr;
  return ((*p) (SrlDevice));
}

// END OF FILE




