// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (C) 1998  Dialogic Corporation.  All Rights Reserved.
//
// A header for Dialogic-specific extensions.


#ifndef DIALOGIC_DEVSPEC_INCLUDED
#define DIALOGIC_DEVSPEC_INCLUDED	1

#include "devspec.h"

// Dialogic unique extension IDs for TSP v.4.0.9 and higher

#define DLG_EXTENSIONID0 0x426B0F40
#define DLG_EXTENSIONID1 0x1020D530
#define DLG_EXTENSIONID2 0x60009ABA
#define DLG_EXTENSIONID3 0x171FB008

#define DEVSPEC_DLG		0x00020000
#define PLAY_WAVE		(DEVSPEC_DLG + 1)
#define RECORD_WAVE		(DEVSPEC_DLG + 2)
#define PLAY_INDEXVOX	(DEVSPEC_DLG + 3)
#define DX_CHGFREQ		(DEVSPEC_DLG + 4)
#define DX_CHGDUR		(DEVSPEC_DLG + 5)
#define DX_CHGREPCNT	(DEVSPEC_DLG + 6)

// Device-specific play area
typedef struct _playentry
{
	WCHAR	wcFileName[MAX_PATH];		// contains file name 
	ULARGE_INTEGER	ldwOffsetSize;		// contains both offset(LowPart) & size(HighPart)
} PLAYENTRY;

typedef struct _playlist
{
	DWORD	dwRate;			// sampling rate specificator
	DWORD dwNumEntries;		// number of entries to play
	PLAYENTRY PlayEntry[1];	// extendable array of entries
} PLAYLIST; 

// Device-specific dx_chfreq, dx_chdur area
typedef struct _dxchfrqpl
{
	int nToneType;		// ToneID
	int nF1;			// Frequency 1
	int nDev1;			// Deviation of Frequency 1
	int nF2;			// Frequency 2
	int nDev2; 			// Deviation of Frequency 2
} DXCHFRQPL, *PDXCHFRQPL;

typedef struct _dxchdurpl
{	
	int nToneType;	   // ToneID
	int nOn;		   // Time on
	int nOnDev;		   // Time on deviation
	int nOff;		   // Time off
	int nOffDev;	   // Time off deviation
} DXCHDURPL, *PDXCHDURPL;

typedef struct _dxchreppl
{	
	int nToneType;	   // Tone ID
	int nRepCnt;	   // Repetition count
} DXCHREPPL, *PDXCHREPPL;

// Advanced Call progress Analysis area
#define	TSPI_ENABLE_PERFECTCALL		0x00000001	// Enables perfect call
#define	TSPI_ENABLERINGBACKSTATE	0x40000000	// Tells the TSP to report RINGBACK state instead of DISCONNECTED/NOANSWER 
#define	TSPI_FORCEINITCALLP			0x80000000	// Forces dx_initcallp
#define TSPI_DETECTLOOPCURRENT		0x20000000	// forces loop current detection v. 4.2.7 and higher ONLY

#define	TSPI_OPTEN			1	// DX_OPTEN Enable Operator Intercept with Connect
#define	TSPI_OPTDIS			2	// DX_OPTDIS Disable Operator Intercept 
#define TSPI_OPTNOCON		3	// DX_ OPTNOCON Enable Operator Intercept w/o Connect 
#define TSPI_PVDENABLE		4	// DX_PVDENABLE Enable PVD 
#define TSPI_PVDOPTEN		5	// DX_PVDOPTEN Enable PVD with OPTEN 
#define TSPI_PVDOPTNOCON	6	// DX_PVDOPTNOCON Enable PVD with OPTNOCON 
#define TSPI_PAMDENABLE		7	// DX_PAMDENABLE Enable PAMD 
#define TSPI_PAMDOPTEN		8	// DX_PAMDOPTEN Enable PAMD with OPTEN 

typedef struct _tspicap
{
	DWORD	dwEnablePerfCall;	// enable or disable Perfect Call:If enabled,dx_deltones and dx_initcallp are called in the beginning of each call.
	DWORD	dwCaps;				// select Perfect call features, maps directly to ca_intflg field of DX_CAP
	DWORD	dwNorbTimeout;		// noringback timeout, maps directly to ca_cnosig field of DX_CAP
	DWORD	dwRingbackCadence;	// ringback cadence to calculate number of rings
} TSPICAP, *PTSPICAP;

// advanced config structure area
typedef struct _BOARD_PARAMETERS {
	UINT	uFlashTime;			// number of milliseconds time on hook during a flash
	UINT	uPauseTime;			// Pause time-delay caused by a comma in the dialing string
	UINT	uDTMFDigitLength;	// Duration of DRMF digits for dialing
	UINT	uTTInterdigitDelay;	// DRMF Interdigit Delay time between digits in DTMF dialing
	UINT	uDTMFAntiTalkOff;	// Sets the minimum time for DRMF to be present during playback to be considered valid
	UINT	uPulseLengthMake;	// Puse Dial Make-Duration of pulse dial onhook interval
	UINT	uPulseLengthBreak;	// Pulse Dial Break - duration of pulse dial offhook interval
	UINT	uPulseLengthInterdigitDelay;	//??
	UINT	uDTMFDebounceTime;	// maximum length of time in which DTMF can be absent and then come back on again and still be considered the same DTMF tone
	UINT	uTermEmuMode;		// JZ
	WCHAR	uMsgAccessOn[16];	// JZ
	WCHAR	uMsgAccessOff[16];	// JZ
	UINT	uResetRingCnt;		// JZ
	UINT	uRingOn;			// JZ
	UINT	uRingOff;			// JZ
	UINT	uPDNKEY;			// JZ
	UINT	uXFERKEY;			// JZ
	UINT	uSENDKEY;			// JZ
	UINT	uCANCELKEY;			// JZ
	UINT	uDNKEY;				// JZ
	UINT	uRELKEY;		// JZ
	UINT	uSpeakerMode;		// JZ
} BOARD_PARAMETERS,*PBOARD_PARAMETERS;

typedef struct _CALL_PARAMETERS {
	UINT	uMinLCOFF;			// Minumum time before loop current drop message is sent
	UINT	uMinTimeRingDetect;	// Minimum time ring must be present to qualify as a ring
	UINT	uMinTimeEndRing;	// Minimum time for ring not to be present before qualifying as not ringing
	UINT	uNumRingsForAnswer;	// Number of rings before the 'no-answer' timeout is reported on the outgoing call
	UINT	uNumRingsForIncoming;	// Number of rings after which an incoming call is reported by the service provider
	UINT	uMaxWaitBetweenRings;	// Maximum time to wait for the next ring. Used to distinquish between incoming calls
	DWORD	dwEnablePerfCall;	// flag used to enable or disable Perfect Call
	DWORD	dwCaps;				// flag used to select Perfect call features
	DWORD	dwNorbTimeout;		// the value for the noringback timeout
} CALL_PARAMETERS,*PCALL_PARAMETERS;

typedef struct _TONE_PARAMETERS {
	UINT	uFreq1;				// The first frequency of the tone
	UINT	uFreq2;				// The second frequency of the tone
	UINT	uOnTime;			// Time duration when the signal is on
	UINT	uOffTime;			// Time duration when the signal is off
	UINT	uRepetitionCount;	// Cadence repetition count
	UINT	uFreq1Deviation;	// Deviation of the first frequency
	UINT	uFreq2Deviation;	// Deviation of the second frequency
	UINT	uOnTimeDeviation;	// Deviation of the 'on' period duration
	UINT	uOffTimeDeviation;	// Deviation of the 'off' period duration
} TONE_PARAMETERS,*PTONE_PARAMETERS;

typedef struct _BEEP_TONE_PARAMETERS {
	UINT	uFreq1;				// The first frequency of the beep tone
	UINT	uFreq2;				// The second frequency of the beep tone
	INT		nAmp1;				// First amplitude of the beep tone
	INT		nAmp2;				// Second amplitude of the beep tone
	INT		nDur;				// JZ the duration of a long beep
} BEEP_TONE_PARAMETERS,*PBEEP_TONE_PARAMETERS;

typedef struct _PBX_PARAMETERS {
	WCHAR	szPBXName[32];		// Name of PBX
	WCHAR	szForward[16];		// Dial string used to initialize call forwarding
	WCHAR	szUnForward[16];	// Dial string used to cancel call-forwarding
	WCHAR	szPark[16];			// Dial string used to park a call
	WCHAR	szUnPark[16];		// Dial string used to unpark a previously parked call
	WCHAR	szPickup[16];	// Dial string used to pickup a specific call
	WCHAR	szSecure[16];		// Dial string used to secure calls
	WCHAR	szHold[16];			// Dial string used to put active calls on hold
	WCHAR	szSwapHold[16];		// Dial string used to swap calls on hold
	WCHAR	szCompleteConf[16];	// Dial string used to merge a consultation call into a conference call
	WCHAR	szDropDialtoneConsult[16];	// Dial string used to drop consultation calls using 'Transfer and Abort' sequence while the consultation call is in the dialtone state
	WCHAR	szDropProceedingConsult[16];// Dial string used to drop consultation calls using 'Transfer and Abort' sequence while the consultation call in in the proceeding state
	WCHAR	szDropBusyConsult[16];	// Dial string used to drop consultation calls using 'Transfer and Abort' sequence while the consultation call is in the busy state
	WCHAR	szDropConnectedConsult[16];	// Dial string used to drop consultation calls using 'Transfer and Abort' sequence while the consultation call is in the connected state
	WCHAR	szDropDisconnectedConsult[16];	// Dial string used to drop consultation calls while the consultation call is in the disconnected state
	WCHAR	szConfirmFreq[16];	// Confirmation frequencies for the line forward operation
	WCHAR	szGroupPickup[16];	// Dial string used to pickup a group call
	WCHAR	szTransfer[16];		// Dial string used to transfer a call	
	WCHAR	szConference[16];	// Dial string used to form a conference
} PBX_PARAMETERS,*PPBX_PARAMETERS;

typedef struct _vtspconfig
{
	DWORD					dwCommand;
	DWORD					dwParmsPresent;
	WCHAR					szTsfFileName[MAX_PATH];
	BOARD_PARAMETERS		BoardParms;
	CALL_PARAMETERS			CallParms;
	TONE_PARAMETERS			ToneParms[3];	// disc1, disc2, fax
	BEEP_TONE_PARAMETERS	BeepParms;
	PBX_PARAMETERS			PbxParms;
} VTSPCONFIG,*PVTSPCONFIG;

// dwCommand defines
#define	TAKE_EFFECT_IMMEDIATELY		0x00000001	// re-initialize the channels
#define	SAVE_CONFIG_DATA			0x00000002	// save data in the Registry

// dwParmsPresent defines
#define	TSF_NAME_PRESENT			0x00000001
#define	BOARD_PARMS_PRESENT			0x00000002
#define	CALL_PARMS_PRESENT			0x00000004
#define	DISCTONE1_PARMS_PRESENT		0x00000008
#define	DISCTONE2_PARMS_PRESENT		0x00000010
#define	FAXTONE_PARMS_PRESENT		0x00000020
#define	BEEP_PARMS_PRESENT			0x00000040
#define	PBX_PARMS_PRESENT			0x00000080

//Digital Alarm defines
#define DTALARM_OOF     0x510 /* out of frame error, count saturation */
#define DTALARM_LOS     0x511 /* Initial loss of signal detection */
#define DTALARM_DPM     0x512 /* Driver performance monitor failure */
#define DTALARM_BPVS    0x513 /* Bipolar violation count saturation */
#define DTALARM_ECS     0x514 /* Error count saturation */
#define DTALARM_RYEL    0x515 /* Received yellow alarm */
#define DTALARM_RRA     DTALARM_RYEL            /* Received remote alarm */
#define DTALARM_RCL     0x516 /* Received carrier loss */
#define DTALARM_FERR    0x517 /* Frame bit error */
#define DTALARM_FSERR   DTALARM_FERR            /* Received frame sync error */
#define DTALARM_B8ZSD   0x518 /* Bipolar eight zero substitution detect */   
#define DTALARM_RBL     0x519 /* Received blue alarm */
#define DTALARM_RUA1    DTALARM_RBL             /* Received unframed all 1s */ 
#define DTALARM_RLOS    0x51A /* Received loss of sync */
#define DTALARM_RED     0x51B /* Got a read alarm condition */
#define DTALARM_MFSERR  0x51C /* Received multi frame sync error */
#define DTALARM_RSA1    0x51D /* Received signalling all 1s */
#define DTALARM_RDMA    0x51E /* Received distant multi-frame alarm */
#define DTALARM_CECS    0x51F /* CRC4 error count saturation */

/* -- recovered series -- */
#define DTALARM_OOFOK   0x520 /* out of frame error, count saturation recovered */
#define DTALARM_LOSOK   0x521 /* Initial loss of signal detection recovered */
#define DTALARM_DPMOK   0x522 /* Driver performance monitor failure recovered */
#define DTALARM_BPVSOK  0x523 /* Bipolar violation count saturation recovered */
#define DTALARM_ECSOK   0x524 /* Error count saturation recovered */
#define DTALARM_RYELOK  0x525 /* Received yellow alarm recovered */
#define DTALARM_RRAOK   DTALARM_RYELOK          /* Received remote alarm recovered */
#define DTALARM_RCLOK   0x526 /* Received carrier loss recovered */
#define DTALARM_FERROK  0x527 /* Frame bit error recovered */
#define DTALARM_FSERROK DTALARM_FERROK          /* Received frame sync error recovered */
#define DTALARM_B8ZSDOK 0x528 /* Bipolar eight zero substitution dtct recovered */  
#define DTALARM_RBLOK   0x529 /* Received blue alarm recovered */
#define DTALARM_RUA1OK  DTALARM_RBLOK           /* Received unframed all 1s recovered */ 
#define DTALARM_RLOSOK  0x52A /* Received loss of sync recovered */
#define DTALARM_REDOK   0x52B /* Got a read alarm condition recovered */
#define DTALARM_MFSERROK 0x52C   /* Received multi frame sync error recovered */
#define DTALARM_RSA1OK  0x52D /* Received signalling all 1s recovered */
#define DTALARM_RDMAOK  0x52E /* Received distant multi-frame alarm recovered */
#define DTALARM_CECSOK  0x52F /* CRC4 error count saturation recovered */

#endif
