
/************************************************************************
 **   FILENAME:          ipmlib.h
 **   CREATED:           1/1/2001
 **   LAST MODIFIED:     7/19/2001

 **   Copyright (c) 1999-2000 Dialogic Corp.
 **   All Rights Reserved

 **	
/************************************************************************/

#ifndef __IPMLIB_H__
#define __IPMLIB_H__

#ifdef _WIN32
#ifdef IPMEDIALIB_EXPORTS
#define IPMEDIALIB_API __declspec(dllexport)
#else
#define IPMEDIALIB_API __declspec(dllimport)
#endif
#else

#define IPMEDIALIB_API 
#endif

/* Constants */
#if ((!defined EV_ASYNC) && (!defined(EV_SYNC)))
#define EV_ASYNC              0x8000
#define EV_SYNC               0x0000
#endif

#define MAX_IPM_DIGITS		0x20 /*32*/
#define MAX_MEDIA_INFO		0x07 /*7*/
#define IP_ADDR_SIZE		0x20 /*32*/
#define MAX_QOS_SESSION		0x05 /*5*/
#define MAX_EVENT			0x19 /*25*/
#define MAX_ALARM			0x05 /*5*/
#define MAX_QOS_THRESHOLD	0x05 /*5*/

/*Event list*/
#define IPMEV_MASK					    0x9000
#define IPMEV_OPEN                   (IPMEV_MASK | 0x01)
#define IPMEV_SET_REMOTE_MEDIA_INFO  (IPMEV_MASK | 0x02)
#define IPMEV_STOPPED                (IPMEV_MASK | 0x03)
#define IPMEV_GET_LOCAL_MEDIA_INFO   (IPMEV_MASK | 0x04)
#define IPMEV_GET_SESSION_INFO       (IPMEV_MASK | 0x05)
#define IPMEV_LISTEN                 (IPMEV_MASK | 0x06)
#define IPMEV_UNLISTEN               (IPMEV_MASK | 0x07)
#define IPMEV_GET_XMITTS_INFO        (IPMEV_MASK | 0x08)
#define IPMEV_SEND_DIGITS            (IPMEV_MASK | 0x09)
#define IPMEV_RECEIVE_DIGITS         (IPMEV_MASK | 0x0a)
#define IPMEV_DIGITS_RECEIVED        (IPMEV_MASK | 0x0b)
#define IPMEV_EVENT_ENABLED          (IPMEV_MASK | 0x0c)
#define IPMEV_EVENT_DISABLED         (IPMEV_MASK | 0x0d)
#define IPMEV_GET_QOS_ALARM_STATUS   (IPMEV_MASK | 0x0e)
#define IPMEV_RESET_QOS_ALARM_STATUS (IPMEV_MASK | 0x0f)
#define IPMEV_SET_QOS_THRESHOLD_INFO (IPMEV_MASK | 0x10)
#define IPMEV_GET_QOS_THRESHOLD_INFO (IPMEV_MASK | 0x11)
#define IPMEV_GET_TIMESLOT_INFO      (IPMEV_MASK | 0x12)
#define IPMEV_SET_TIMESLOT_INFO      (IPMEV_MASK | 0x13)
#define IPMEV_QOS_ALARM					 (IPMEV_MASK | 0x14)
#define IPMEV_ERROR                  (IPMEV_MASK | 0x1e)

typedef enum
{
	QOSTYPE_UNKNOWN				=  0x00,
	QOSTYPE_DTMFDISCARDED		=	0x01,
	QOSTYPE_LOSTPACKETS			=  0x02,
	QOSTYPE_JITTER					=	0x03,
	QOSTYPE_MAXVSRP				=	0x04,
	QOSTYPE_ROUNDTRIPLATENCY	=	0x05
}eIPM_QOS_TYPE;

typedef enum
{
	EVT_UNKNOWN				=  0x00,
	EVT_DTMFDISCARDED		=	0x01,
	EVT_LOSTPACKETS		=  0x02,
	EVT_JITTER				=	0x03,
	EVT_MAXVSRP				=	0x04,
	EVT_ROUNDTRIPLATENCY	=	0x05
}eIPM_EVENT;

typedef struct ipm_open_info_tag
{
	int nRFU;
}IPM_OPEN_INFO, *PIPM_OPEN_INFO;

typedef struct ipm_close_info_tag
{
	int nRFU;
}IPM_CLOSE_INFO, *PIPM_CLOSE_INFO;

typedef enum
{
	ALARM_STATE_OFF	=	0x0,
	ALARM_STATE_ON		=	0x1
}eIPM_ALARM_STATE;

typedef struct ipm_qos_alarm_data_tag
{
   eIPM_QOS_TYPE		eQoSType;			/*The QoS parameter type */
   eIPM_ALARM_STATE	eAlarmState;       /*indicate if On/Off */
} IPM_QOS_ALARM_DATA, *PIPM_QOS_ALARM_DATA;

typedef struct ipm_qos_alarm_status_tag
{
   unsigned int unAlarmCount;
   IPM_QOS_ALARM_DATA QoSData[MAX_ALARM];
} IPM_QOS_ALARM_STATUS, *PIPM_QOS_ALARM_STATUS;

typedef struct ipm_qos_threshold_data_tag
{
    eIPM_QOS_TYPE	eQoSType;     
    unsigned int  unTimeInterval;
    unsigned int  unDebounceOn;
    unsigned int  unDebounceOff;
    unsigned int  unFaultThreshold;
    unsigned int  unPercentSuccessThreshold;
    unsigned int  unPercentFailThreshold;
} IPM_QOS_THRESHOLD_DATA, *PIPM_QOS_THRESHOLD_DATA;

typedef struct ipm_qos_threshold_info_tag
{
   unsigned int unCount;
   IPM_QOS_THRESHOLD_DATA pQoSThresholdData[MAX_QOS_THRESHOLD];
} IPM_QOS_THRESHOLD_INFO, *PIPM_QOS_THRESHOLD_INFO;

typedef struct ipm_event_info_tag
{
	unsigned int unCount;                /* number of following structures */
	unsigned int unEventData[MAX_EVENT]; /* Data associated with the event */
} IPM_EVENT_INFO, *PIPM_EVENT_INFO;

typedef struct ipm_qos_session_info_tag
{
	  eIPM_QOS_TYPE	eQoSType;
	  unsigned int		unData;	
} IPM_QOS_SESSION_INFO, *PIPM_QOS_SESSION_INFO;

typedef struct ipm_rtcp_session_info_tag
{
   unsigned int	unLocalSR_TimeStamp;
   unsigned int	unLocalSR_TxPackets;
   unsigned int	unLocalSR_TxOctets;
   unsigned int	unLocalSR_SendIndication;
   unsigned int	unLocalRR_FructionLost;
   unsigned int	unLocalRR_ComulativeLost;
   unsigned int	unLocalRR_SeqNumber;
   unsigned int	unLocalRR_ValidInfo;
   unsigned int	unRemoteSR_TimeStamp;
   unsigned int	unRemoteSR_TxPackets;
   unsigned int	unRemoteSR_TxOctets;
   unsigned int	unRemoteSR_SendIndication;
   unsigned int	unRemoteRR_FructionLost;
   unsigned int	unRemoteRR_ComulativeLost;
   unsigned int	unRemoteRR_SeqNumber;
   unsigned int	unRemoteRR_ValidInfo;

} IPM_RTCP_SESSION_INFO, *PIPM_RTCP_SESSION_INFO;

typedef struct ipm_session_info_tag
{
	IPM_RTCP_SESSION_INFO  RtcpInfo;
	unsigned int          unQoSInfoCount;
	IPM_QOS_SESSION_INFO   QoSInfo[MAX_QOS_SESSION];
} IPM_SESSION_INFO, *PIPM_SESSION_INFO;

typedef enum
{
	XMITTER_TDM		=	0x00,
	XMITTER_PACKET	=	0x01
}eIPM_TRANSMITTER_TYPE;

typedef enum
{
	ENCODING_MULAW		= 0x00,
	ENCODING_ALAW		= 0x01
}eENCODING_TYPE;

typedef enum
{
	TS_TRANSMIT		=	0x00,
	TS_RECEIVE		=  0x01
}eIP_TIMESLOT_TYPE;

typedef struct ipm_tdm_timeslot_info_tag
{
   eIP_TIMESLOT_TYPE	eTSType;      /* Transmit or Receive */
   unsigned int			unCount;      /* Number of Timeslots */
   unsigned long*		pulTimeslots; /* The TDM Bus timeslot number */
   eENCODING_TYPE		eEncodingType;/* The Bus Encoding format */

} IPM_TDM_TIMESLOT_INFO, *PIPM_TDM_TIMESLOT_INFO;

typedef struct ipm_timeslot_info_tag
{
	/* Type of Transmitter, TDM/Packet Bus etc */
	eIPM_TRANSMITTER_TYPE eXmitterType; 
	union
	{
		IPM_TDM_TIMESLOT_INFO tdmInfo;  /* TDM Bus Timeslot Information */
	}timeslotInfo;
} IPM_TIMESLOT_INFO, *PIPM_TIMESLOT_INFO;

typedef struct ipm_param_info_tag
{
	unsigned int	 unParmType;     /* the parameter to set or get */
	union
	{
		unsigned long	ulParmValue;   /* Value of the parameter */
		char			*pcParmValue;   /* Value of the parameter */
	}parmInfo;
} IPM_PARM_INFO, *PIPM_PARM_INFO;

typedef enum
{
	DIGIT_ALPHA_NUMERIC	=	0x00,
	DIGIT_SIGNAL			=	0x01,
	DIGIT_SIGNAL_UPDATE	=	0x02
}eIPM_DIGIT_TYPE;

typedef enum
{
	DIGIT_IP_INBAND	=	0x00,
	DIGIT_TDM			=	0x01
}eIPM_DIGIT_DIRECTION;

typedef struct ipm_digit_info_tag
{
	eIPM_DIGIT_TYPE			eDigitType;				/* Type of digits - DTMF, ALPHA-NUMERIC */
	eIPM_DIGIT_DIRECTION	eDigitDirection;		/* The direction of flow of digits */
	char					cDigits[MAX_IPM_DIGITS];/* the digits */
	unsigned int			unNumberOfDigits;       /* Number of digits */
	unsigned int			unTimeStamp;
	unsigned int			unExpirationTime;
	unsigned int			unDuration;
} IPM_DIGIT_INFO, *PIPM_DIGIT_INFO;

typedef struct ipm_port_info_tag
{
   unsigned int   unPortId;                  /* The Port ID */
   char           cIPAddress[IP_ADDR_SIZE];  /* IP Address  */

} IPM_PORT_INFO, *PIPM_PORT_INFO;

typedef enum
{
	MEDIATYPE_REMOTE_RTP_INFO,
	MEDIATYPE_LOCAL_RTP_INFO,		
	MEDIATYPE_REMOTE_RTCP_INFO,		
	MEDIATYPE_LOCAL_RTCP_INFO,		
	MEDIATYPE_REMOTE_CODER_INFO,	
	MEDIATYPE_LOCAL_CODER_INFO,		
	MEDIATYPE_TDM_INFO				
}eIPM_MEDIA_TYPE;

typedef enum
{
	CODER_TYPE_NONSTANDARD		= 0x0,
	CODER_TYPE_G711ULAW64K,
	CODER_TYPE_G711ULAW56K,
	CODER_TYPE_G711ALAW64K,
	CODER_TYPE_G711ALAW56K,
	CODER_TYPE_G721ADPCM,
	CODER_TYPE_G722_48K,
	CODER_TYPE_G722_56K,
	CODER_TYPE_G722_64K,
	CODER_TYPE_G7231_5_6K,
	CODER_TYPE_G7231_6_3K,
	CODER_TYPE_G7231ANNEXCCAP,
	CODER_TYPE_G728,
	CODER_TYPE_G729,
	CODER_TYPE_G729ANNEXA,
	CODER_TYPE_G729WANNEXB,
	CODER_TYPE_G729ANNEXAWANNEXB,
	CODER_TYPE_GSMFULLRATE,
	CODER_TYPE_GSMHALFRATE,
	CODER_TYPE_GSMEFR,
	CODER_TYPE_IS111172,
	CODER_TYPE_IS13818
}eIPM_CODER_TYPE;

typedef enum
{
	CODER_FRAMESIZE_DONT_CARE	= 0x00,	/* 0 */
	CODER_FRAMESIZE_5			= 0x05,
	CODER_FRAMESIZE_10			= 0xa,
	CODER_FRAMESIZE_20			= 0x14,
	CODER_FRAMESIZE_30			= 0x1e /* 1*/
}eIPM_CODER_FRAMESIZE;

typedef enum
{
	CODER_VAD_DISABLE		= 0x00,
	CODER_VAD_ENABLE		= 0x01
}eIPM_CODER_VAD;

typedef struct ipm_coder_info_tag
{
	   eIPM_CODER_TYPE		eCoderType;		/* The coder Type */
	   eIPM_CODER_FRAMESIZE	eFrameSize;		/* Frame size supported*/
	   unsigned int			unFramesPerPkt;	/* No. of Frames per packet*/
	   eIPM_CODER_VAD		eVadEnable;		/* Flag indicating if VAD is enabled/disabled */
	   unsigned int			unPayloadType;	/* Type of payload supported*/
	   unsigned int			unRedPayloadType;/* Type of Redundancy Payload */

} IPM_CODER_INFO, *PIPM_CODER_INFO;

typedef struct IPM_MEDIA_tag
{
	eIPM_MEDIA_TYPE eMediaType;			
	union
	{
		IPM_PORT_INFO   PortInfo;	/* RTP Port Information*/
		IPM_CODER_INFO  CoderInfo;  /* Coder Information  */
		IPM_TIMESLOT_INFO tsInfo;   /* Timeslot Info */
	}mediaInfo;
} IPM_MEDIA, *PIPM_MEDIA;

typedef struct 
{
	unsigned int unCount;
	IPM_MEDIA MediaData[MAX_MEDIA_INFO];   
} IPM_MEDIA_INFO, *PIPM_MEDIA_INFO;

typedef struct
{
	int nRFU;
}IPM_STOP_INFO, * PIPM_STOP_INFO;

typedef enum
{
	STOP_SEND_DIGITS		=	0x01,
	STOP_RECEIVE_DIGITS		=	0x02,
	STOP_PLAY				=	0x03,
	STOP_RECORD				=	0x04,
	STOP_ALL				=	0x05
}eIPM_STOP_OPERATION;

/* Flow direction of IP packets */
typedef enum
{
	DATA_IP_TO_TDM					=	0x00,
	DATA_TDM_TO_IP					=	0x01,
	DATA_IP_TDM_BIDIRECTIONAL	=	0x02
}eIPM_DATA_DIRECTION;


/****** IPML Functions ******/
#if (defined(__BORLAND__) || defined(__cplusplus) || defined(__STDC__))

#if defined(__cplusplus)
extern "C" {
#endif

IPMEDIALIB_API 
int 
ipm_Open(
	const char *szDevName, 
	IPM_OPEN_INFO *pOpenInfo, 
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_Close(
	int nDeviceHandle, 
	IPM_CLOSE_INFO* pCloseInfo, 
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_SetRemoteMediaInfo(
	int nDeviceHandle, 
	IPM_MEDIA_INFO* pMediaInfo,
	eIPM_DATA_DIRECTION eDirection ,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_Listen(
	int nDeviceHandle, 
	IPM_TIMESLOT_INFO *pTimeslotInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_UnListen(
	int nDeviceHandle, 
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_GetXmitTSInfo(
	int nDeviceHandle, 
	IPM_TIMESLOT_INFO *pTimeslotInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_GetTimeslotInfo(
	int nDeviceHandle, 
	IPM_TIMESLOT_INFO *pTimeslotInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_SetTimeslotInfo(
	int nDeviceHandle, 
	IPM_TIMESLOT_INFO *pTimeslotInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_GetLocalMediaInfo(
	int nDeviceHandle, 
	IPM_MEDIA_INFO *pMediaInfo ,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_GetSessionInfo(
	int nDeviceHandle, 
	IPM_SESSION_INFO *pSessionInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_Stop(
	int nDeviceHandle, 
	eIPM_STOP_OPERATION eOperation, 
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_SendDigits(
	int nDeviceHandle, 
	IPM_DIGIT_INFO *pDigitInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_ReceiveDigits(
	int nDeviceHandle, 
	IPM_DIGIT_INFO *pDigitInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_EnableEvents(
	int nDeviceHandle,
	eIPM_EVENT *pEvents,
	unsigned int unNumofEvents,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_DisableEvents(
	int nDeviceHandle, 
	eIPM_EVENT *pEvents,
	unsigned int unNumofEvents, 
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_GetQoSAlarmStatus(
	int nDeviceHandle, 
	IPM_QOS_ALARM_STATUS  *pQoSAlarmInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_ResetQoSAlarmStatus(
	int nDeviceHandle,
	IPM_QOS_ALARM_STATUS *pQoSAlarmInfo,
	unsigned short usMode
	);

IPMEDIALIB_API
int 
ipm_SetQoSThreshold(
	int nDeviceHandle,
	IPM_QOS_THRESHOLD_INFO *pQoSThresholdInfo,
	unsigned short usMode
	);

IPMEDIALIB_API 
int 
ipm_GetQoSThreshold(
	int nDeviceHandle,
	IPM_QOS_THRESHOLD_INFO *pQoSThresholdInfo,
	unsigned short usMode
	);

#if defined(__cplusplus)
}
#endif

#else /*(defined(__BORLAND__) || defined(__cplusplus) || defined(__STDC__))*/

IPMEDIALIB_API 
int 
ipm_Open();

IPMEDIALIB_API 
int 
ipm_Close();

IPMEDIALIB_API 
int 
ipm_SetRemoteMediaInfo();

IPMEDIALIB_API 
int 
ipm_Listen();

IPMEDIALIB_API 
int 
ipm_UnListen();

IPMEDIALIB_API 
int 
ipm_GetXmitTSInfo();

IPMEDIALIB_API 
int 
ipm_GetTimeslotInfo();

IPMEDIALIB_API 
int 
ipm_SetTimeslotInfo();

IPMEDIALIB_API 
int 
ipm_GetLocalMediaInfo();

IPMEDIALIB_API 
int 
ipm_GetSessionInfo();

IPMEDIALIB_API 
int 
ipm_Stop();

IPMEDIALIB_API 
int 
ipm_SendDigits();

IPMEDIALIB_API 
int 
ipm_ReceiveDigits();

IPMEDIALIB_API 
int 
ipm_EnableEvents();

IPMEDIALIB_API 
int 
ipm_DisableEvents();

IPMEDIALIB_API 
int 
ipm_GetQoSAlarmStatus();

IPMEDIALIB_API 
int 
ipm_ResetQoSAlarmStatus();

IPMEDIALIB_API
int 
ipm_SetQoSThreshold();

IPMEDIALIB_API 
int 
ipm_GetQoSThreshold();

#endif  /*(defined(__BORLAND__) || defined(__cplusplus) || defined(__STDC__))*/
#endif __IPMLIB_H__
