/***************************************************************************
 *		Copyright (c) 1996 Dialogic Corp.
 *		All Rights Reserved.
 *
 *		THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *		The copyright notice above does not evidence any actual or
 *		intended publication of such source code.
 ***************************************************************************/

/****************************************************************************
 * FILE:         qcluster.h
 * ORG:          Dialogic Corporation
 * ORIG-DATE:    28-Aug-96  
 * LAST-MOD:     22-Jul-97  
 * PURPOSE:
 *    This file contains defines and structures relating to clusters.
 *
 ***************************************************************************/
					     
#ifndef __QCLUSTER_H__
#define __QCLUSTER_H__

// Define flag if compiling MNTI host library.
// Undefine flag if compiling applications.
#ifndef __MNTILIB_
#define MNTAPI __declspec(dllimport)
#else
#define MNTAPI __declspec(dllexport)
#endif

// Cluster types
typedef UInt24 QPortDef;

/* Attribute type so apps can allocate SCBus resources */
#define QSCRES_Std_ComponentType   0x100

//  Dec 10, 1999 : ThakkarD : Fix for PT 15569
#define QSCRES_Std_Component_Type  QSCRES_Std_ComponentType

#define QCLUST_Std_ComponentType    0x101

/* TDM Stream types (peripheral attached to stream) */
typedef enum {
	QTS_TYPE_ANY =	0,		/* Wildcard						*/
	QTS_TYPE_NONE=	0,		/* No device					*/
	QTS_TYPE_MMA = 1,		/* MMA TDM Bus					*/
	QTS_TYPE_SCBUS,		/* SCBus							*/
	QTS_TYPE_NET,			/* Network Interface TDM	*/
	QTS_TYPE_RESERVE1,	/* Reserved for future     */
	QTS_TYPE_RESERVE2 	/* Reserved for future     */
} enum_QTStreamType;

/* Stream type aliases */
#define QTS_TYPE_RESOURCE	QTS_TYPE_MMA
#define QTS_TYPE_NETWORK	QTS_TYPE_NET


/* Timeslot data directions */
typedef enum {
	QTS_DIR_ANY	= 0,			/* Direction wildcard				*/
	QTS_DIR_IN	= 1,			/* Data flows into peripheral		*/
	QTS_DIR_OUT	= 2,			/* Data flows out of peripheral	*/
	QTS_DIR_BOTH = 3			/* Creation request - IN and OUT	*/
} enum_QTSlotDir;


/*
 *  Some convenient defines for qClusterTSAssign  to use in
 *  scPortID argument.
 */
#define QSCBUS_PORT_IN   (QClusterDefinePort( 0, SCBUS, IN, 1 ))
#define QSCBUS_PORT_OUT  (QClusterDefinePort( 0, SCBUS, OUT, 1 ))

/* defines for encoding */
#define QSCBUS_ENCODING_ALAW        0x1
#define QSCBUS_ENCODING_MULAW       0x2

/* defines for default idle pattern */
#define QSCBUS_IDLE_ALAW            0xD5
#define QSCBUS_IDLE_MULAW           0x7F

#ifdef __cplusplus
extern "C" {
#endif

// Functional prototypes
MNTAPI BOOL WINAPI mntClusterActivate(HANDLE hDevice, 
                               QTrans nTransID,
                               QCompDesc ClusterDesc,
                               QCompDesc SCDesc,
                               QPortDef SCPortID,
                               QCompDesc ClientDesc,
                               UCHAR Options,
                               USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterByComp(HANDLE	hDevice, 
                             QTrans	nTransID, 
                             QCompDesc CompDesc,
                             PQCompDesc lpClusterDesc,
                             USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterCompByAttr(HANDLE hDevice, 
                                 QTrans nTransID,
                                 QCompDesc ClustDesc,
                                 PQCompAttr lpAttr,
                                 PQCompDesc lpCompDesc,
                                 USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterConnect(HANDLE hDevice, 
                              QTrans nTransID, 
                              QCompDesc ClusterDesc,
                              QCompDesc InstDesc1,
                              QPortDef PortID1,
                              QCompDesc InstDesc2,
                              QPortDef PortID2,
                              USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterCreate(HANDLE hDevice,
                             QTrans nTransID,
                             QCompDesc BrdAddr,
                             PQCompAttr lpAttr,
                             PQCompDesc lpClusterDesc,
                             USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterDeactivate(HANDLE hDevice,
                                 QTrans nTransID,
                                 QCompDesc ClusterDesc,
                                 QCompDesc SCDesc,
                                 QPortDef SCPortId,
                                 USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterDestroy(HANDLE hDevice,
                              QTrans nTransID,
                              QCompDesc ClusterDesc,
                              USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterDisconnect(HANDLE hDevice, 
                                 QTrans nTransID,
                                 QCompDesc ClusterDesc,
                                 QCompDesc InstDesc1,
                                 QPortDef PortID1,
                                 QCompDesc InstDesc2,
                                 QPortDef PortID2,
                                 USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterTSAssign(HANDLE hDevice,
                               QTrans nTransID,
                               QCompDesc ClusterDesc,
                               QCompDesc SCDesc,
                               QPortDef SCPortID,
                               UCHAR Width,
                               UCHAR Encoding,
                               UCHAR Idle,
                               PUSHORT lpSlotId,
                               USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterTSUnassign(HANDLE hDevice,
                                 QTrans nTransID,
                                 QCompDesc ClusterDesc,
                                 QCompDesc SCDesc,
                                 QPortDef SCPortID,
                                 USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterSlotInfo(HANDLE hDevice,
                               QTrans nTransID,
                               QCompDesc	ClusterDesc,
                               QCompDesc SCDesc,
                               QPortDef SCPortID,
                               QClusterSlotInfoResult_t *lpClusterInfo,
                               BYTE nSlots,
                               PUSHORT lpSlots,
                               USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterAllocate(HANDLE hDevice,
                               QTrans nTransID,
                               PQCompDesc lpClusterDesc,
                               PQCompAttr lpAttr,
                               USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterFree(HANDLE hDevice,
                           QTrans nTransID,
                           QCompDesc ClusterDesc,
                           USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterFind(HANDLE hDevice,
                           QTrans nTransID,
                           PQCompDesc lpClusterDesc,
                           PQCompAttr lpAttr,
                           USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterAttrAdd(HANDLE hDevice,
                              QTrans nTransID,
                              QCompDesc ClusterDesc,
                              PQCompAttr lpAttr,
                              USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterAttrRemove(HANDLE hDevice,
                                 QTrans nTransID,
                                 QCompDesc ClusterDesc,
                                 PQCompAttr lpAttr,
                                 USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterConfigLock(HANDLE hDevice,
                                 QTrans nTransID,
                                 QCompDesc ClusterDesc,
                                 USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterConfigUnlock(HANDLE hDevice,
                                   QTrans nTransID,
                                   QCompDesc ClusterDesc,
                                   USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntClusterCompInfo(HANDLE hDevice,
                                   QTrans nTransID,
                                   PQCompDesc lpClusterDesc,
                                   PULONG lpCount,
                                   PQCompDesc lpCompDesc,
                                   USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
#ifdef __cplusplus
}
#endif

#endif // __Q_CLUSTER_H__
