#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <sys\stat.h>

#include "srllib.h"
#include "dxxxlib.h"
#include "extcscu.h"

// prototypes for 'not implemented' functions
static long tascscu_long_notimp  (int);
static int  tascscu_int_notimp   (int);
static void tascscu_void_notimp  (int);
static char *tascscu_null_notimp (int);

#ifdef __cplusplus
extern "C" {				// C Plus Plus function bindings
#define extern
#endif

long ta_libinit (unsigned short);
long tascscu_GetDLLVersion (LPDWORD, LPDWORD);

#ifdef __cplusplus
}				// C Plus Plus function bindings
#undef extern
#endif


// typedefs and #defines
typedef void (*func_ptr)(void);

struct func_table
{
  char *fname;
  func_ptr fptr;
};

typedef enum _FUNCPTR_IDS
{
  dx_verifyCS_ID = 0,
} FUNCPTR_IDS;

typedef int       (*ptr_dx_verifyCS)    (int);

// Global variables
static struct func_table TASCSCUFunc[] =
{  
  {"dx_verifyCS",   (func_ptr)tascscu_int_notimp}
};

char caMTLibName[] = "TASCSCU.DLL";
char caSTLibName[] = "TASCSCU.DLL";
char *pcLibName;

long ta_libinit (unsigned short usFlag)
{
  struct func_table *pTA = TASCSCUFunc;
  func_ptr pProcAddress;
  static HINSTANCE hLibrary = NULL;
  DWORD dwNumberOfFunctions;
  DWORD dwFileVer, dwProdVer;

  // Check if it's already open
  if (hLibrary != (HINSTANCE)NULL)
  {
    return -1;
  }

  // Determine which library to load
  if ((usFlag & DLGC_MT) == DLGC_MT)
  {
    pcLibName = caMTLibName;
  }
  else
  {
    pcLibName = caMTLibName;
    kvscscu_GetDLLVersion (&dwFileVer, &dwProdVer);
    if (dwFileVer <= 0x0004000A)
    {
      pcLibName = caSTLibName;
    }
  }

  dwNumberOfFunctions = sizeof (TASCSCUFunc) / sizeof (TASCSCUFunc[0]);

  // Try to load the library
  hLibrary = LoadLibrary (pcLibName);
  if (hLibrary == (HINSTANCE)NULL)
  {
    return 0;
  }

  // Set up each entry to a default not implemented
  // function and then overwrite the true address if the 
  // function is actually present in the DLL
  while (dwNumberOfFunctions--)
  {
    pProcAddress = (func_ptr)GetProcAddress (hLibrary, pTA->fname);
    if (pProcAddress)
    {
      pTA->fptr = pProcAddress;
    }

    pTA++;
  }

  return 0;
}

long tascscu_GetDLLVersion (LPDWORD pdwFileVer, LPDWORD pdwProdVer)
{
  return ( sr_GetVersion (pcLibName, pdwFileVer, pdwProdVer) );
}

int tascscu_set_lasterr (int iErr, long lError)
{
  dx_set_lasterr (iErr, lError);
  return 0;
}

long tascscu_long_notimp (int iErr)
{
  tascscu_set_lasterr (iErr, EKV_NOTIMP);
  return -1;
}

int tascscu_int_notimp (int iErr)
{
  tascscu_set_lasterr (iErr, EKV_NOTIMP);
  return -1;
}

void tascscu_void_notimp (int iErr)
{
  tascscu_set_lasterr (iErr, EKV_NOTIMP);
  return;
}

char *tascscu_null_notimp (int iErr)
{
  tascscu_set_lasterr (iErr, ED42_NOTIMP);
  return (char *) NULL;
}

int tascscu_dflt_errno (void)
{
  return errno;
}

int dx_verifyCS ( int SrlDevice )
{
  ptr_dx_verifyCS p;

  p = (ptr_dx_verifyCS) *TASCSCUFunc[dx_verifyCS_ID].fptr;
  return ((*p) (SrlDevice));
}

// END OF FILE

