//
//  Copyright (c) 1999-2004 by AudioCodes USA, Inc.
//  All rights reserved.
//
//  NTiErr.h
//      SmartWORKS API error code definitions
//
//  AUDIOCODES USA, INC. PROPRIETARY.
//


//
//  Values are 32 bit values layed out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|C|R|     Facility          |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      C - is the Customer code flag
//
//      R - is a reserved bit
//
//      Facility - is the facility code
//
//      Code - is the facility's status code
//
//

#ifndef _INC_NTIERR_H
#define _INC_NTIERR_H

//
// Define the severity codes
//
#define STATUS_SEVERITY_WARNING          0x2
#define STATUS_SEVERITY_SUCCESS          0x0
#define STATUS_SEVERITY_INFORMATIONAL    0x1
#define STATUS_SEVERITY_ERROR            0x3


#define MT_RET_OK                               0L     // no error

//  Wait for event timed out
#define MT_TIMEOUT                              ((ULONG)0xC0040013L)

//
// MessageId: MT_RET_FORMAT_NOT_SUPPORTED
//
// MessageText:
//
//  Media format not supported%0
//
#define MT_RET_FORMAT_NOT_SUPPORTED             0xE0002001L

//
// MessageId: MT_RET_NO_DIGIT
//
// MessageText:
//
//  No digit available%0
//
#define MT_RET_NO_DIGIT                         0xE0002002L

//
// MessageId: MT_RET_STARTED
//
// MessageText:
//
//  Voice system already started%0
//
#define MT_RET_STARTED                          0xE0002003L

//
// MessageId: MT_RET_NOT_STARTED
//
// MessageText:
//
//  Voice system not started%0
//
#define MT_RET_NOT_STARTED                      0xE0002004L

//
// MessageId: MT_RET_MALFUNCTION
//
// MessageText:
//
//  Hardware failed during initialization%0
//
#define MT_RET_MALFUNCTION                      0xE0002005L

//
// MessageId: MT_RET_INVALID_FUNC
//
// MessageText:
//
//  Function code is not valid%0
//
#define MT_RET_INVALID_FUNC                     0xE0002006L

//
// MessageId: MT_RET_INVALID_HW_INT
//
// MessageText:
//
//  Bad hardware IRQ at initialization%0
//
#define MT_RET_INVALID_HW_INT                   0xE0002007L

//
// MessageId: MT_RET_INVALID_CHANNEL
//
// MessageText:
//
//  Board number is out of range%0
//
#define MT_RET_INVALID_BOARD                    0xE0002008L

//
// MessageId: MT_RET_INVALID_CHANNEL
//
// MessageText:
//
//  Channel number is out of range%0
//
#define MT_RET_INVALID_CHANNEL                  0xE0002009L

//
// MessageId: MT_RET_QUOTA_EXCEEDED
//
// MessageText:
//
//  Request failed due to quota limit%0
//
#define MT_RET_QUOTA_EXCEEDED                   0xE000200AL

//
// MessageId: MT_RET_BUSY
//
// MessageText:
//
//  Channel is busy%0
//
#define MT_RET_BUSY                             0xE0002010L

//
// MessageId: MT_RET_IDLE
//
// MessageText:
//
//  Channel is idle%0
//
#define MT_RET_IDLE                             0xE0002011L

//
// MessageId: MT_RET_INVALID_PARAM
//
// MessageText:
//
//  Invalid argument passed to function%0
//
#define MT_RET_INVALID_PARAM                    0xE0002012L

//
// MessageId: MT_RET_INVALID_device_IO
//
// MessageText:
//
//  Invalid Device IO setting
//
#define MT_RET_INVALID_DEVICE_IO                0xE0002013L

//
// MessageId: MT_RET_NOT_AVAILABLE
//
// MessageText:
//
//  Feature is not available%0
//
#define MT_RET_NOT_AVAILABLE                    0xE0002014L

//
// MessageId: MT_RET_CHANNEL_NOT_OPENED
//
// MessageText:
//
//  Channel device not opened for access
//
#define MT_RET_CHANNEL_NOT_OPENED               0xE0002015L

//
// MessageId: MT_RET_BOARD_NOT_OPENED
//
// MessageText:
//
//  Board device not opened for access
//
#define MT_RET_BOARD_NOT_OPENED                 0xE0002016L

//
// MessageId: MT_RET_CHANNEL_NOT_OPENED
//
// MessageText:
//
//  Channel device not opened for access
//
#define MT_RET_CHANNEL_INUSE                    0xE0002017L

//
// MessageId: MT_RET_BOARD_NOT_OPENED
//
// MessageText:
//
//  Board device not opened for access
//
#define MT_RET_BOARD_INUSE                      0xE0002018L

//
// MessageId: MT_RET_API_THREAD_PROTECTED
//
// MessageText:
//
//  API is been referred by other thread
//
#define MT_RET_API_THREAD_PROTECTED             0xE0002019L

//
// MessageId: MT_RET_MUX_THREAD_PROTECTED
//
// MessageText:
//
//  MUX is been referred by other thread
//
#define MT_RET_MUX_THREAD_PROTECTED             0xE000201AL

//
// MessageId: MT_RET_STREAMIN_THREAD_PROTECTED
//
// MessageText:
//
//  STREAMIN is been referred by other thread
//
#define MT_RET_STREAMIN_THREAD_PROTECTED        0xE000201BL

//
// MessageId: MT_RET_STREAMOUT_THREAD_PROTECTED
//
// MessageText:
//
//  STREAMOUT is been referred by other thread
//
#define MT_RET_STREAMOUT_THREAD_PROTECTED       0xE000201CL

//
// MessageId: MT_RET_STREAMOUT_THREAD_PROTECTED
//
// MessageText:
//
//  STREAMOUT is been referred by other thread
//
#define MT_RET_NOT_APPLICABLE                   0xE000201DL

//
// MessageId: MT_RET_EXTENDED_ERROR
//
// MessageText:
//
//  Extended error%0
//
#define MT_RET_EXTENDED_ERROR                   0xE0002024L

//
// MessageId: MT_RET_NO_MUX
//
// MessageText:
//
//  Multiplexer is not available%0
//
#define MT_RET_NO_MUX                           0xE0002026L

//
// MessageId: MT_RET_MUX_ON
//
// MessageText:
//
//  Multiplexer system was already started%0
//
#define MT_RET_MUX_ON                           0xE0002027L

//
// MessageId: MT_RET_MUX_OFF
//
// MessageText:
//
//  Multiplexer system was not started%0
//
#define MT_RET_MUX_OFF                          0xE0002028L

//
// MessageId: MT_RET_BAD_JOIN
//
// MessageText:
//
//  Invalid crosspoint coordinates%0
//
#define MT_RET_BAD_JOIN                         0xE0002029L

//
// MessageId: MT_RET_INVALID_BUSTYPE
//
// MessageText:
//
//  Invalid crosspoint coordinates%0
//
#define MT_RET_INVALID_BUSTYPE                  0xE0002030L

//
// MessageId: MT_RET_BAD_CURSOR
//
// MessageText:
//
//  Invalid index offset%0
//
#define MT_RET_BAD_CURSOR                       0xE0002031L

//
// MessageId: MT_RET_QUEUE_FULL
//
// MessageText:
//
//  Invalid index offset%0
//
#define MT_RET_QFULL                            0xE0002032L


//
// MessageId: MT_RET_LIB_LOAD_FAILED
//
// MessageText:
//
//  LIB file could't be loaded
//
#define MT_RET_LIB_LOAD_FAILED                   0xE0002040L

//
// MessageId: MT_RET_DRV_LOAD_FAILED
//
// MessageText:
//
//  DRV file could't be loaded
//
#define MT_RET_DRV_LOAD_FAILED                   0xE0002041L

//
// MessageId: MT_RET_INVALID_BINFILE
//
// MessageText:
//
//  Invalid Bin file.
//
#define MT_RET_INVALID_BINFILE                   0xE0002042L

//
// MessageId: MT_RET_BOARD_INCOMPATIBLE_BINFILE
//
// MessageText:
//
//  Board incompatible bin file.
//
#define MT_RET_BOARD_INCOMPATIBLE_BINFILE        0xE0002043L

//
// MessageId: MT_RET_IO_PENDING
//
// MessageText:
//
//  Background I/O operation is in progress or queued.
//
#define MT_RET_IO_PENDING                       0x60002033L

//
// MessageId: MT_RET_NOT_ENOUGH_DATA
//
// MessageText:
//
//  StreamIn does not have enough data for API request
//
#define MT_RET_NOT_ENOUGH_DATA                  0x60002034L

//
// MessageId: MT_RET_NOT_ENOUGH_BUFSPACE
//
// MessageText:
//
//  StreamOut does not have enough buffering space for API request
//
#define MT_RET_NOT_ENOUGH_BUFSPACE              0x60002035L

//
// MessageId: MT_RET_DATA_TRUNCATED
//
// MessageText:
//
//  The provided buffer does not have enough buffering space for API request
//
#define MT_RET_DATA_TRUNCATED                   0x60002036L

//
// MessageId: MT_XRET_BADHANDLE
//
// MessageText:
//
//  Bad handle%0
//
#define MT_XRET_BADHANDLE                       0xE0002114L

//
// MessageId: MT_XRET_QFULL
//
// MessageText:
//
//  Event queue is full (has reached maximum limit)%0
//
#define MT_XRET_QFULL                           0xE0002118L

//
// MessageId: MT_XRET_QEMPTY
//
// MessageText:
//
//  Event queue is empty%0
//
#define MT_XRET_QEMPTY                          0xE0002119L

//
// MessageId: MT_XRET_BADFUNCTION
//
// MessageText:
//
//  Bad function%0
//
#define MT_XRET_BADFUNCTION                     0xE0002120L

//
// MessageId: MT_XRET_BADPARAM
//
// MessageText:
//
//  Invalid argument passed to function%0
//
#define MT_XRET_BADPARAM                        0xE0002121L

//
// MessageId: MT_RET_DEVICE_NOTFOUND
//
// MessageText:
//
//  Voice interface device was not found
//
#define MT_RET_DEVICE_NOTFOUND                  0xE0002200L

//
// MessageId: MT_RET_INTERNAL_ERROR
//
// MessageText:
//
//  Internal error in DLL (%1!s!@%2!s!)%0
//
#define MT_RET_INTERNAL_ERROR                   0xE0002201L

//
// MessageId: MT_RET_UNKNOWN_DEVICE_ERROR
//
// MessageText:
//
//  Unknown device error (%1!s!@%2!s!)%0
//
#define MT_RET_UNKNOWN_DEVICE_ERROR             0xE0002202L

//
// MessageId: MT_RET_SERVICE_NOT_STARTED
//
// MessageText:
//
//  Voice service not started%0
//
#define MT_RET_SERVICE_NOT_STARTED              0xE0002203L

//
// MessageId: MT_RET_CANNOT_CREATE_EVTQMUTEX
//
// MessageText:
//
//  Requested service already started
//
#define MT_RET_SERVICE_ALREADY_STARTED          0xE0002204L

//
// MessageId: MT_RET_CANNOT_CREATE_EVTQEVENT
//
// MessageText:
//
//  Unable to create event for the event queue%0
//
#define MT_RET_CANNOT_CREATE_EVTQEVENT          0xE0002205L

//
// MessageId: MT_RET_NO_MEM
//
// MessageText:
//
//  An operation could not be completed due to insufficient memory or an error occurred while attempting to allocate memory%0
//
#define MT_RET_NO_MEM                           0xE0002206L

//
// MessageId: MT_RET_INVALID_EVENT_QUEUE
//
// MessageText:
//
//  Invalid event queue%0
//
#define MT_RET_INVALID_EVENT_QUEUE              0xE0002207L

//
// MessageId: MT_RET_EVTQMUTEX_ABANDONED
//
// MessageText:
//
//  The event queue synchronization mutex was abandoned by another thread%0
//
#define MT_RET_EVTQMUTEX_ABANDONED              0xE0002208L

//
// MessageId: MT_RET_NO_ASSOC_EVTQ
//
// MessageText:
//
//  No event queue has been associated with channel: %1!s!%0
//
#define MT_RET_NO_ASSOC_EVTQ                    0xE0002209L

//
// MessageId: MT_RET_INVALID_FILEINDEX
//
// MessageText:
//
//  The MT_IO_CONTROL FileIndex field is out of range for the file%0
//
#define MT_RET_INVALID_FILEINDEX                0xE000220AL

//
// MessageId: MT_RET_ZERO_FILELENGTH
//
// MessageText:
//
//  The MT_IO_CONTROL FileLength field is zero%0
//
#define MT_RET_ZERO_FILELENGTH                  0xE000220BL

//
// MessageId: MT_RET_COMPAT_NOTSUPPORTED
//
// MessageText:
//
//  Compatibility mode is not supported in this implementation%0
//
#define MT_RET_COMPAT_NOTSUPPORTED              0xE000220CL

//
// MessageId: MT_RET_UNKNOWN_IOTRANS_HANDLE
//
// MessageText:
//
//  The handle in the IO_TRANS structure was not obtained through the SmartWORKS API file open functions%0
//
#define MT_RET_UNKNOWN_IOTRANS_HANDLE           0xE000220DL

//
// MessageId: MT_RET_CANNOT_CREATE_MUTEX
//
// MessageText:
//
//  Mutex creation failed%0
//
#define MT_RET_CANNOT_CREATE_MUTEX              0xE000220EL

//
// MessageId: MT_EXCEPTION
//
// MessageText:
//
//  An exception occurred during the API call%0
//
#define MT_EXCEPTION                            0xE000220FL

//
// MessageId: MT_RET_INVALID_FILENAME
//
// MessageText:
//
//  Invalid filename%0
//
#define MT_RET_INVALID_FILENAME                 0xE0002210L

//
// MessageId: MT_RET_VERSION_MISMATCH
//
// MessageText:
//
//  Version mismatch%0
//
#define MT_RET_VERSION_MISMATCH                 0xE0002211L

//
// MessageId: MT_RET_MISSING_TERMINATOR
//
// MessageText:
//
//  A terminating entry was not found in an array argument.
//
#define MT_RET_MISSING_TERMINATOR               0xE0002212L

//
// MessageId: MT_RET_SERVICE_STOPPED
//
// MessageText:
//
//  Voice service stopped or terminated
//
#define MT_RET_SERVICE_STOPPED                  0x60002213L



#define MT_ERR_TRUNKPT_SUCCESS                  MT_RET_OK
#define MT_ERR_TRUNKPT_BAD_FRAMER_CNT           0xE0090001L
#define MT_ERR_TRUNKPT_BAD_CHANNEL_CNT          0xE0090002L
#define MT_ERR_TRUNKPT_BAD_FRAMER_NB            0xE0090003L
#define MT_ERR_TRUNKPT_NULL_POINTER             0xE0090004L
#define MT_ERR_TRUNKPT_EMPTY_BUFFER             0xE0090005L
#define MT_ERR_TRUNKPT_FRAME_TOO_LONG           0xE0090006L
#define MT_ERR_TRUNKPT_FRAME_TOO_SHORT          0xE0090007L
#define MT_ERR_TRUNKPT_FRAME_INVALID            0xE0090008L
#define MT_ERR_TRUNKPT_BAD_PLUG                 0xE0090009L
#define MT_ERR_TRUNKPT_BAD_INTERFACE_ID         0xE009000AL
#define MT_ERR_TRUNKPT_CHANNEL_NOT_USED         0xE009000BL
#define MT_ERR_TRUNKPT_CALL_NOT_FOUND           0xE009000CL
#define MT_ERR_TRUNKPT_BAD_TS                   0xE00B000DL
#define MT_ERR_TRUNKPT_BAD_SIC                  0xE00B000EL
#define MT_ERR_TRUNKPT_BAD_SI                   0xE00B000FL

#define MT_ERR_TRUNKPT_L3MSG_TOO_SHORT          0xE0090010L
#define MT_ERR_TRUNKPT_L3MSG_UNVALID            0xE0090011L
#define MT_ERR_TRUNKPT_L3MSG_BAD_PD             0xE0090012L
#define MT_ERR_TRUNKPT_L3MSG_NO_RESOURCE        0xE0090013L
#define MT_ERR_TRUNKPT_L3MSG_BAD_CR             0xE0090014L
#define MT_ERR_TRUNKPT_L3MSG_XFER_MODE          0xE0090015L
#define MT_ERR_TRUNKPT_L3MSG_BAD_L1ID           0xE0090016L
#define MT_ERR_TRUNKPT_L3MSG_BAD_INTTYPE        0xE0090017L
#define MT_ERR_TRUNKPT_L3MSG_BAD_CHANTYPE       0xE0090018L
#define MT_ERR_TRUNKPT_L3MSG_BAD_SLOTMAP        0xE0090019L
#define MT_ERR_TRUNKPT_L3MSG_NO_SLOTMAP         0xE009001AL
#define MT_ERR_TRUNKPT_L3MSG_NO_SEGMENT_IE      0xE009001BL
#define MT_ERR_TRUNKPT_L3MSG_BAD_IE_CODING      0xE009001CL
#define MT_ERR_TRUNKPT_L3MSG_MISSING_IE         0xE009001DL
#define MT_ERR_TRUNKPT_L3MSG_BAD_CON_STATE      0xE009001EL
#define MT_ERR_TRUNKPT_L3MSG_BAD_SETUP_STATE    0xE009001FL

//
// MessageId: MT_RET_ISDNPT_BAD_TRUNK_NUMBER
//
// MessageText:
//
//
#define MT_RET_ISDNPT_BAD_TRUNK_NUMBER          MT_ERR_TRUNKPT_BAD_INTERFACE_ID

//
// MessageId: MT_RET_ISDNPT_CHANNEL_NOT_FOUND
//
// MessageText:
//
//
#define MT_RET_ISDNPT_CHANNEL_NOT_FOUND         MT_ERR_TRUNKPT_CHANNEL_NOT_USED

//
// MessageId: MT_RET_ISDNPT_CALL_NOT_FOUND
//
// MessageText:
//
//
#define MT_RET_ISDNPT_CALL_NOT_FOUND            MT_ERR_TRUNKPT_CALL_NOT_FOUND



#define MT_RET_RBS_SUCCESS                      MT_RET_OK
#define MT_RET_RBS_BAD_CHANNEL_CNT              0xE0070001L
#define MT_RET_RBS_BAD_STATE                    0xE0070002L
#define MT_RET_RBS_BAD_MSG_CODE                 0xE0070003L
#define MT_RET_RBS_NO_DIGITS                    0xE0070004L
#define MT_RET_RBS_BAD_RBS_TYPE                 0xE0070005L
#define MT_RET_RBS_NULL_POINTER                 0xE0070006L
#define MT_RET_RBS_BAD_FRAMING                  0xE0070007L
#define MT_RET_RBS_BAD_PARAM                    0xE0070008L
#define MT_RET_RBS_CHANNEL_INUSE                0xE0070009L
#define MT_RET_RBS_INVALID_STATE                0xE007000AL
#define MT_RET_RBS_INVALID_DIGIT                0xE007000BL

#define RBS_SUCCESS                             MT_RET_RBS_SUCCESS
#define RBS_ERR_BAD_CHANNEL_CNT                 MT_RET_RBS_BAD_CHANNEL_CNT
#define RBS_ERR_BAD_STATE                       MT_RET_RBS_BAD_STATE
#define RBS_ERR_BAD_MSG_CODE                    MT_RET_RBS_BAD_MSG_CODE
#define RBS_ERR_NO_DIGITS                       MT_RET_RBS_NO_DIGITS
#define RBS_ERR_BAD_RBS_TYPE                    MT_RET_RBS_BAD_RBS_TYPE
#define RBS_ERR_NULL_POINTER                    MT_RET_RBS_NULL_POINTER
#define RBS_ERR_BAD_FRAMING                     MT_RET_RBS_BAD_FRAMING
#define RBS_ERR_BAD_PARAM                       MT_RET_RBS_BAD_PARAM
#define RBS_ERR_CHANNEL_INUSE                   MT_RET_RBS_CHANNEL_INUSE
#define RBS_ERR_INVALID_STATE                   MT_RET_RBS_INVALID_STATE
#define RBS_ERR_INVALID_DIGIT                   MT_RET_RBS_INVALID_DIGIT


#define MT_RET_ISDN_NOFREE_CONID                0xE00A0001L
#define MT_RET_ISDN_INVALID_CALLREF             0xE00A0002L
#define MT_RET_ISDN_INVALID_CONID               0xE00A0003L
#define MT_RET_ISDN_INVALID_NAI                 0xE00A0004L
#define MT_RET_ISDN_NULL_POINTER                0xE00A0005L

#define MT_RET_LICENSE_FILE_CORRUPT             0xE00B0001L
#define MT_RET_LICENSE_INVALID_SERIAL_NUMBER    0xE00B0002L
#define MT_RET_LICENSE_INVALID_PRODUCT_ID       0xE00B0003L

#endif

// End of NTiErr.h
