#ifndef _INC_NVAPI_H
#define _INC_NVAPI_H

#include <pshpack1.h>

#ifdef __cplusplus
extern "C" {
#endif
   
//
// I/O data transfer structure
//



#include "defs.h"

typedef struct _NV_IO_CONTROL {
    //
    // general data
    //
    ULONG           reserved1;
	ULONG                   ctrlData;                               // # of noise events to trigger recording for START_VOICE control
    //
    // general termination
    //
    ULONG           MaxTime;                // Timeout in milliseconds
    ULONG           MaxBytes;               // Terminate on size
    //
    // signal termination
    //
    ULONG           MaxSilence;             // Silence time for termination
    ULONG           MaxNoise;               // Noise time for termination
    ULONG           CpmMode;                // Loop Stop / CPM mode
    ULONG           TermMask;               // CPM termination mask
    ULONG           MoniMask;               // Monitor mask
    //
    // digit termination
    //
    ULONG           MaxDigits;              // # of digits for termination
    NV_DIGIT_MASK   DigitMask;              // Bit mask for termination digits
    ULONG           EndDigit;               // Term digit (@ terms any digit)
    ULONG           DigitTime;              // Inter digit timeout
    //
    // miscellaneous control data
    //
    ULONG                       StartControl;           // Start Control Flag Field - Recording Start Prompt
											// The type is changed from USHORT to ULONG on 06-25
	ULONG           ToneDuration;           // Prompts tone duration in milliseconds
    ULONG           TermControl;                        // Termination Control - clear command queue after termination
    //
    // playfile() & playbuffer() specific data
    //
    ULONG           FileIndex;              // File starting offset for playfile()/rec_file()
    ULONG           reserved3;
    ULONG           FileType;               // Type  of voice file
    //
    // Data portion
    //
    ULONG           Length;                 // Length of the following data buffer
    PUCHAR          Buffer;                 // Data buffer
} NV_IO_CONTROL, *PNV_IO_CONTROL;

#define NvIoBuffer(IoControl)  (PUCHAR)((PUCHAR)IoControl + sizeof(NV_IO_CONTROL))

typedef struct _NVFAX_IO_CONTROL {
	ULONG   fhandle;
	ULONG   datatype;       /* data type - TIFF or RAW                      */
	ULONG   length;         /* number of page or length of data */
	ULONG   width;          /* image width for RAW format only      */
	ULONG   resln;          /* vertical resolution for RAW format only*/
	ULONG   encode;         /* encoding for RAW format only         */
	ULONG   reserved[4];
	PUCHAR  ptr;
} NVFAX_IO_CONTROL, *PNVFAX_IO_CONTROL;


//
// used for calls that don't require any data
//
typedef struct _NV_GENERAL {
    ULONG   Status;
    UCHAR   Reserved[512];
} NV_GENERAL, *PNV_GENERAL;

//
// cpm control
//
typedef struct _DEF_CPM_MODE {
    ULONG   Status;
    ULONG   Mode;
} DEF_CPM_MODE, *PDEF_CPM_MODE;



//
// System info structure
//
typedef struct _LINK_INFO {
    ULONG   Status;                 // status value
    ULONG   DigiRate;               // Digitizing rate
    ULONG   NumAudios;               // # of audio channels
    ULONG   NumBoards;              // # of boards in the system
    ULONG   NumChans;               // Number of Channels in the system
    ULONG   Mode;                   // Always 1, Enhanced mode
    ULONG   StartChan;              // Starting internal mux channel
    ULONG   StartBoard;             // Starting internal mux board
    ULONG   MuxBoards;              // Total number of mux boards
    ULONG   MuxSystem;              // Type of mux system
    ULONG   MuxGroups;              // Number of mux groups
    ULONG   MuxChans;               // Total number of mux channels
} LINK_INFO, *PLINK_INFO;

//
// Board info structure
//
typedef struct _BOARD_INFO {
    ULONG   status;                 // status value
    ULONG   BoardNumber;            // Board number
    ULONG   base_version;           // Base board software version
    ULONG   dsp_version;            // DSP version
    ULONG   base_address;           // I/O base address
    ULONG   Interrupt;              // Interrupt number
    ULONG   start_channel;          // Logical number of 1st board chan.
    ULONG   channels;               // Number of channels on the board
    ULONG   option1;                            // Basic Board option
	ULONG   option2;                                // Extended Board option
    ULONG   Digrate;                // Digitizing rate
    ULONG   Bandwidth;              // Channel bandwidth
    ULONG   Mode;         
	USHORT  pciSlot;
	USHORT  pciBusNumber;
} BOARD_INFO, *PBOARD_INFO;


typedef struct {

	USHORT	cadence1_on;
	USHORT	cadence1_off;
	USHORT	cadence2_on;
	USHORT	cadence2_off;
	USHORT	freq1;
	USHORT	freq2;
	USHORT	volume;
	USHORT	cycles;
} MTSM_TONE;


//
// control digits
//
typedef struct _CONTROL_DIGITS {
    ULONG   Status;                 // status value
    ULONG   DtmfEnable;             // 0=disable dtmf detection, 1=enable dtmf detection
    ULONG   QueueControl;           // DQ_FLUSH | DQ_EVENTS
} CONTROL_DIGITS, *PCONTROL_DIGITS;

//
// Call Progress Result structure
//
typedef struct _CALL_RESULT {
    ULONG   Status;                 // status value
    ULONG   CpmTerm;                // Last CPM termination
    ULONG   OffFreq;                // Percent of signal off-frequency
    ULONG   Frequency;              // Detected frequency
    ULONG   HiTime;                 // Longest high period in cadence
    ULONG   LoTimeShort;            // Shortest low period in cadence
    ULONG   LoTimeLong;             // Longest  low period in cadence
    ULONG   AnswerTime;             // Duration of answer for CONNECT
    ULONG   SubTerm;                // Termination sub type
} CALL_RESULT, *PCALL_RESULT;

//
// OEM info structure
//
typedef struct _OEM_INFO {
    ULONG   Status;                 // status value
    CHAR    String[33];             // String with oem info
} OEM_INFO, *POEM_INFO;

//
// physical to logical mapping
//
typedef struct _PHYSICAL_MAP {
    ULONG   Status;                 // status value
    ULONG   Channel;                // channel number
    ULONG   Board;                  // board number
    ULONG   PhysicalLine;           // physical line number
} PHYSICAL_MAP, *PPHYSICAL_MAP;

//
// Channel status information
//
typedef struct _CHAN_STATUS {
    ULONG   Status;                 // status value
    ULONG   EventMask;              // Transition event enable bits
    ULONG   Function;               // Current channel status
    ULONG   CurrentTask;            // Current channel task
    ULONG   PreviousEvent;          // Previous termination event
    ULONG   Bytes;                  // Bytes processed
    ULONG   DigitCount;             // # of digits in DTMF buffer
    ULONG   Error;                  // Last error code
    ULONG   LineStatus;             // Status of phone line
    ULONG   CpmResult;              // Last CPM termination type
} CHAN_STATUS, *PCHAN_STATUS;

//
// version information
//
typedef struct _NV_VERSION {
    ULONG   Status;                 // status value
    ULONG   DriverVersion;          //
    ULONG   BoardVersion;           //
    ULONG   Reserved;               //
} NV_VERSION, *PNV_VERSION;




//
// Conference Group status info
//
typedef struct _GROUP_STATUS {
  ULONG     Status;                 // Group status
  ULONG     TotalChannels;          // Total number of chans in group
  ULONG     TalkChannels;           // Number of talking chans in group
  ULONG     ListenChannels;         // Number of listening chans in grp
} GROUP_STATUS, *PGROUP_STATUS;

//
// Set parameters
//
typedef struct _SET_PARAMS {
    ULONG   Status;                 // status value
    ULONG   WinkDelay;              // wink delay
    ULONG   WinkTime;               // wink time
} SET_PARAMS, *PSET_PARAMS;

//
// set events
//
typedef struct _SET_EVENTS {
    ULONG   Status;                 // status value
    ULONG   EventMask;              // events
    ULONG   Rings;                  // number of rings for evt_rings
} SET_EVENTS, *PSET_EVENTS;

//
// set moni
//
typedef struct _SET_MONI {
    ULONG   Status;                 // status value
    ULONG   MoniMask;               // monitor events
} SET_MONI, *PSET_MONI;

//
// set termination mask
//
typedef struct _SET_TERM {
    ULONG   Status;                 // status value
    ULONG   TermMask;               // termination events
} SET_TERM, *PSET_TERM;

//
// mux functions
//
typedef struct _MUX_INFO {
    ULONG   Status;                 // status value
    ULONG   Board1;                 // board #1
    ULONG   Board2;                 // board #2
    ULONG   Channel1;               // channel #1
    ULONG   Channel2;               // channel #2
    ULONG   Gain;                   // gain adjustment
    ULONG   Channels;               // start mux
    ULONG   Group;                  // group number
    ULONG   Direction;              // group direction
    ULONG   GroupStatus;            // group status
    PUCHAR  Buffer;                 // data buffer
} MUX_INFO, *PMUX_INFO;

// play index data structures - Handle Version
typedef struct _INDEX_PLAY_HANDLE {
	ULONG           Status;
    ULONG       Offset;
    ULONG       Length;
	HANDLE          FileHandle;
} INDEX_PLAY_HANDLE, *PINDEX_PLAY_HANDLE;
//
// play index data structures
//
typedef struct _INDEX_PLAY {
    ULONG       Status;
    ULONG       Offset;
    ULONG       Length;
    ULONG       FileNameLength;
    WCHAR       FileName[1];
} INDEX_PLAY, *PINDEX_PLAY;

typedef struct _NV_INDEX_PLAY {
    ULONG       Count;
    INDEX_PLAY  IndexPlay[1];
} NV_INDEX_PLAY, *PNV_INDEX_PLAY;

//
// the following defines are for use by the application
// writter to ensure that the NV_INDEX_PLAY structure
// is layed out in memory correctly.  the defines ensure
// that the variable length INDEX_PLAY structures are
// aligned properly.  DO NOT attempt to use these
// structures without the use of these defines.  the driver
// uses these defines too, so if all applications use these
// defines and the memory layout changes, then all that
// will be required is a re-compile of all applications.
// YOU ARE ON YOUR OWN if you don't use these defines!
//

//
// calculate a DWORD alignemnt (4 byte)
//
#define DWORD_ALIGN(_n)  ((4 - ((_n) % 4)) & 3)

//
// calculate the size of the current INDEX_PLAY structure
//
#define IndexPlaySize(ip) \
	    (sizeof(INDEX_PLAY) - sizeof(WCHAR) + \
	     ((ip)->FileNameLength * sizeof(WCHAR)))

//
// calculate the pointer for the next INDEX_PLAY based
// on the data in the current INDEX_PLAY structure
//
#define IncrementIndexPlay(ip) \
	    (PINDEX_PLAY)((PUCHAR)(ip) + \
		DWORD_ALIGN(IndexPlaySize((ip))) + IndexPlaySize((ip)))

//
// performance counters
//
typedef struct _NV_PERF_INFO {
    ULONG   Status;
    PVOID   VirtualAddress;
} NV_PERF_INFO, *PNV_PERF_INFO;

typedef struct _NV_PERF_COUNTERS {
    ULONG   Interrupts;
    ULONG   ReadCount;
    ULONG   ReadBytes;
    ULONG   WriteCount;
    ULONG   WriteBytes;
    ULONG   IoCount;
    ULONG   IoBytes;
    ULONG   OffHookCount;
    ULONG   OnHookCount;
    ULONG   RingCount;
    ULONG   FileOpenClose;
    ULONG   PromptsPlayed;
    ULONG   ChannelsInCpmMode;
    ULONG   BkgrndIoCompletions;
    ULONG   PlayBufferCount;
    ULONG   FilePlayCount;
    ULONG   CallCount;
    ULONG   ErrorCount;
} NV_PERF_COUNTERS, *PNV_PERF_COUNTERS;

//
// event structure
//
typedef struct _NVDSP_EVENT {
    LONGLONG        TimeStamp;
    ULONG           Status;
    ULONG           EventCode;
    ULONG           SubReason;
    ULONG           Termination;
    ULONG           Board;
    ULONG           Channel;
    ULONG           UserEvent;
    PVOID           UserData;
} NVDSP_EVENT, *PNVDSP_EVENT;

typedef struct _WAIT_STRUCT {
    ULONG           Status;
    ULONG           Timeout;
    NVDSP_EVENT     Event;
} WAIT_STRUCT, *PWAIT_STRUCT;

//
// Tone Structures

/**** User Tone Pulse structure  ****/
typedef struct {
	USHORT onmin;                 /* Minimum cadence "On" (ms)         */
	USHORT onmax;                 /* Maximum cadence "On" (ms)         */
	USHORT offmin;                /* Minimum cadence "Off" (ms)        */
	USHORT offmax;                /* Maximum cadence "Off" (ms)        */
} NV_USR_PULSE;

//#define USR_PULSE               NVDSP_USR_PULSE

/**** User Tone Structure  ****/
typedef struct {
	USHORT index;                /* Tone table index (0-18)          */
								 /* 0-14 : CPM tone, 15-17: User tone*/
	USHORT signalname;           /* Tone ID                          */
	USHORT signaltype;           /* Tone type (CPM,UsrTone,SIT)      */
	USHORT cycles;               /* Minimum cycles to report event   */
	USHORT freq[3];              /* frequency value                  */
	NV_USR_PULSE    pulse[3];    /* Cadence structure                */
	USHORT threshold;            
} NV_USRTONE, NV_CPMTONE, *PNV_USRTONE, *PNV_CPMTONE;


//
// Register Structures
//


/**** ACT structure  ****/
typedef struct {
	USHORT  act_thresh_lo;          /* Low  signal threshold            */
	USHORT  act_min_silence;        /* Silence deglitch time            */
	USHORT  act_max_silence;
	USHORT  act_thresh_hi;          /* High signal threshold            */
	USHORT  act_min_noise;
	USHORT  act_max_noise;
} NV_ACTPARAMS, *PNV_ACTPARAMS;

/***  DTMF/MF Detection ***:       */
typedef struct {
	USHORT  width;                /* minimum valid duration */
	USHORT  trigger;              /* trailing ir tailing detection */
} NV_DETECTREGS, *PNV_DETECTREGS;

/***  DTMF/MF Generation ***:      */
typedef  struct {
	USHORT  width;                 /* tone duration time   */
	USHORT  gap;                   /* inter-digit duration */
	USHORT  low_gain;              /* low gain             */
	USHORT  high_gain;             /* high gain            */
} NV_GENREGS, *PNV_GENREGS;

/**** PSTN structure   ****/
typedef struct {
	USHORT   flash_char;            /* flash char (default &)         */
	USHORT   flash_time;            /* flash time (ms)                */
	USHORT   wink_delay;            /* wink delay (for reading only)  */
	USHORT   wink_time;             /* wink time  (for reading only)  */
	USHORT   pause_char;            /* pause char (default ',')       */
	USHORT   pause_time;            /* pause time (ms)                */
	USHORT   pulse_on;              /* Pulse dialing onhook length    */
	USHORT   pulse_off;             /* Pulse dialing offhook length   */
	USHORT   pulse_delay_time;      /* Pulse dial inter-digit delay   */
	USHORT   loop_deglitch;         /* minimum glitch on or off       */
	USHORT   loop_delay;            /* loop event delay (in ms)       */
	USHORT   ring_on_time;          /* ring on duration (ms)          */
	USHORT   ring_off_time;         /* ring off duration(ms)          */
	USHORT   ring_reset;
	USHORT   ring_count;            /* ring cycles to report event    */
	USHORT   ring_trigger;          /* ring detection in leading or tailing */
	USHORT   ring_autoanswer;       /* auto-answer (0-disable,1-enable */
	USHORT   cpm_start_delay;       /* CPM analysis delay after dialing */
	USHORT   offhook_delay;         /* offhook event delay             */
	USHORT   min_rings_no_answer;   /* Minimum cycles to report no answer */
	USHORT   max_sig_time;          /* Maximum continuous signal timw  */
	USHORT   max_sil_time;          /* Maximum continuous silence time */
	USHORT   reserved[4];
} NV_PSTNREGS, *PNV_PSTNREGS;

/**** PXD structure  ****/
typedef struct{
	USHORT   format;                /* linear(0),alaw(2),ulaw(1) */
	SHORT    volume;                /* volume for playback*/
	SHORT    gain;                  /* gain for conference */
	USHORT   g165enable;            /* echo cancellation enable */
	USHORT   g165adapt;             /* echo training enable */
} NV_PSTNPXD, *PNV_PSTNPXD;



/*** Play Tone structure ***: */
typedef struct  {
	USHORT  freq1;                /* Frequency for tone 1 (Hz)         */
	SHORT   ampl1;                /* Amplitude for tone 1 (dB)         */
	USHORT  freq2;                /* Frequency for tone 2 (Hz)         */
	SHORT   ampl2;                /* Amplitude for tone 2 (dB)         */
	USHORT  dur;                  /* Duration of generated tone (1ms) */
} NV_PLAYTONE;

typedef struct {
	USHORT	cycles;
	USHORT	nbrtones;
	USHORT  idle[4];
	NV_PLAYTONE tone[4];
	USHORT  state; // reserved
} MT_CADENCE;


/*** Alert Tone  ***: */
typedef struct  {
	USHORT  freq;                /* Frequency for tone  (Hz)         */
	SHORT   ampl;                /* Amplitude for tone  (dB)         */
	USHORT  dur;                  /* Duration of generated tone (ms) */
	ULONG   Interval;             /* Duration between consecutive alert tones (s)*/
} MTALERT_TONE, *PMTALERT_TONE;


//================ FAX Channel Information
typedef struct {
	USHORT  number_of_pages;
	USHORT  channel_number;
	USHORT  phone_validity;
	WCHAR   phone_number[20];
	USHORT  transmitter_receiver;
	USHORT  inbound_outbound;
	USHORT  automatic_manual;
	USHORT  modem_protocol;
	USHORT  vertical_resolution;
	USHORT  coding_2d_capability;
	USHORT  recording_width;
	USHORT  recording_length;
	USHORT  minimum_scan_line_time;
	USHORT  handshake2400;
	USHORT  uncompressed_mode;
	USHORT  operator_alert_request;
}       NVFAX_INFO;



//
// Voice Detection
//
typedef struct _NV_CPMVOICE {
	USHORT mindur;                          // minumum voice duration (80 ms)
	USHORT cdur;                            // current voice duration (100ms)
	USHORT answmchndur;                     // answering machine minimum duration (1000 ms)
	USHORT slmindur;                        // silence minimum duration (80 ms)
	USHORT sldur;                           // silence duration
	USHORT wbthreshold;                     // voice threashold of wideband filter (150)
	USHORT vcthreshold;                     // relative filter level for voice (3)
	USHORT mincpmfltrs;                     // minumum CPM filter to exceed threshold (3)
	USHORT  floor;
	USHORT  arrthreshpct;
	USHORT  depthreshpct;
	USHORT  hsxfreqdet;
	USHORT  hsxnormalfreqs;
	USHORT  hsxspecialfreqs;
} NV_CPMVOICE, *PNV_CPM_VOICE;





/*
 * Indexed-play table structure
 */

typedef struct _NV_INDEX_TABLE {
  ULONG     PhraseOffset;               /* Phrase offset in the index file  */
  ULONG     PhraseLength;               /* Length of the phrase, in bytes   */
  HANDLE	hIndexFileHandle;
} NV_INDEX_TABLE, *PNV_INDEX_TABLE;

typedef struct _NV_INDEX_TABLE const * PCNV_INDEX_TABLE;

/*
 * Indexed-play table structure (includes file name)
 */

typedef struct _NV_FINDEX_TABLEA {
  PSTR      FileName;                   /* File name                        */
  ULONG     PhraseOffset;               /* Phrase offset in the index file  */
  ULONG     PhraseLength;               /* Length of the phrase, in bytes   */
} NV_FINDEX_TABLEA, *PNV_FINDEX_TABLEA;

typedef struct _NV_FINDEX_TABLEA const *PCNV_FINDEX_TABLEA;


typedef struct _NV_FINDEX_TABLEW {
  PWSTR     FileName;                   /* File name (wide)                 */
  ULONG     PhraseOffset;               /* Phrase offset in the index file  */
  ULONG     PhraseLength;               /* Length of the phrase, in bytes   */
} NV_FINDEX_TABLEW, *PNV_FINDEX_TABLEW;

typedef struct _NV_FINDEX_TABLEW const *PCNV_FINDEX_TABLEW;


#ifdef UNICODE
typedef NV_FINDEX_TABLEW   NV_FINDEX_TABLE;
typedef PNV_FINDEX_TABLEW  PNV_FINDEX_TABLE;
typedef PCNV_FINDEX_TABLEW PCNV_FINDEX_TABLE;
#else
typedef NV_FINDEX_TABLEA   NV_FINDEX_TABLE;
typedef PNV_FINDEX_TABLEA  PNV_FINDEX_TABLE;
typedef PCNV_FINDEX_TABLEA PCNV_FINDEX_TABLE;
#endif /* !UNICODE */


#ifdef __cplusplus
} /* end extern "C" */
#endif

// 6-24
#include <poppack.h>
#endif /* _INC_NVAPI_H */
