#ifndef _DEFS_H_
#define _DEFS_H_

#include <pshpack1.h>
#ifdef __cplusplus
extern "C" {
#endif

/*......................................................................................

			General Types Definitions...
		
.......................................................................................*/
typedef ULONG           NV_DIGIT_MASK;
typedef USHORT          BOARD;              /* Board number             */
typedef SHORT           CHANNEL;            /* Logical channel number   */
typedef HANDLE          HEVTQ;              /* Event queue              */
typedef ULONG           HPROMPT;            /* Prompt handle            */
typedef USHORT          PHYSICALLINE;       /* Physical line number     */
typedef ULONG           HGROUP;             /* Group identifier         */

/*
 * Result (return) codes
 */
#ifndef DEVICE_NT_DRIVER
typedef DWORD           NV_RESULT;          /* result values */
typedef NV_RESULT      *PNV_RESULT;
#define NV_RET_OK       0                   /* no error      */
#endif



//  Wait for voice event timed out
#define NVDSP_TIMEOUT                    ((ULONG)0xC0040013L)
#define NV_MAX_ERRMSG_SIZE 256              /* maximum err message size including null  */

/*
 * NV_RESULT return values (backwards compatibility)
 */

/*......................................................................
	Constant Definitions...
........................................................................*/
 
 /*
 * 'id' argument to NVSetParam
 */

#define SP_WINK_DELAY          0x204    /* Delay before starting wink       */
#define SP_WINK_TIME           0x205    /* Duration of outgoing wink        */

/*
 * Event types for NVDSP_EVENT "UserEvent" when NVWaitForVoiceEvent is used
 * with a channel which is associated with an event queue.
 */

#define NV_EVENTSRC_NVDSP   0x0002          /* NVDSP driver event source        */
#define NV_EVENTSRC_USER	0x00f0

#ifndef NO_COMPAT_CODES
#define SRC_NV800           NV_EVENTSRC_NVDSP   /* historical */
#define SRC_USER1           0x000000f0          /* historical */
#endif


/*.................. Bus Type ...............*/
#define  PCI_BUS		0
#define  ISA_BUS		1



static const NV_DIGIT_MASK
    NV_DIGIT_C                              = 0x8000,
    NV_DIGIT_B                              = 0x4000,
    NV_DIGIT_A                              = 0x2000,
    NV_DIGIT_POUND                          = 0x1000,
    NV_DIGIT_CROSSHATCH                     = 0x1000,
    NV_DIGIT_STAR                           = 0x0800,
    NV_DIGIT_ASTERISK                       = 0x0800,
    NV_DIGIT_0                              = 0x0400,
    NV_DIGIT_9                              = 0x0200,
    NV_DIGIT_8                              = 0x0100,
    NV_DIGIT_7                              = 0x0080,
    NV_DIGIT_6                              = 0x0040,
    NV_DIGIT_5                              = 0x0020,
    NV_DIGIT_4                              = 0x0010,
    NV_DIGIT_3                              = 0x0008,
    NV_DIGIT_2                              = 0x0004,
    NV_DIGIT_1                              = 0x0002,
    NV_DIGIT_D                              = 0x0001;


// Constant to signal prompt at start of recording
#define START_TONE                      2               // Plays an alert tone at the begining of recording or playback
#define INTERDIGIT_DELAY        4                       // Inter-digit delay after first digit
#define ALERT_TONE                      0x80            // Plays an alert tone every 10 sec during recording
#define NOAGC                           0x40
#define START_VOICE                     0x10
#define START_LOOP_ON           0x20

//-----------------------------------------------------------------------------
//                Event type codes
//-----------------------------------------------------------------------------

//
// Regular events:
//
#define EVT_MAXDIGITS              1
#define EVT_TERMDIGIT              2
#define EVT_STOP                   3
#define EVT_SYS_ERROR              4
#define EVT_MAXBYTES               5
#define EVT_ERROR                  6
#define EVT_MAXTIME                7
#define EVT_OFFHOOK                8
#define EVT_DIALED                 9
#define EVT_MAX_SILENCE           10
#define EVT_EOF                   11
#define EVT_LOOP_STOP             12
#define EVT_DISK_FULL             13
#define EVT_ONHOOK                14
#define EVT_CPM_STOP              18
#define EVT_LOOP_DROP             20
#define EVT_RINGS                 21
#define EVT_MON_NOISE             22
#define EVT_MON_SILENCE           23
#define EVT_LOOP_ON               26
#define EVT_DIGITMASK             29
#define EVT_INTERDIGIT            30
#define EVT_MAX_NOISE             31
#define EVT_SETPAR_DONE           37
#define EVT_WINKDONE              38
#define EVT_DIGIT                 40
#define EVT_UTONE_ON              41
#define EVT_UTONE_OFF             42
#define EVT_UTONE_TON             44
#define EVT_UTONE_TOFF            45
#define EVT_TONEPERR              47
#define EVT_TONEP_DONE            48
#define EVT_IO_COMPLETED          52
#define EVT_VOICE_RECORD                  53
#define EVT_LOOPON_RECORD             54
#define EVT_STREAM_MAXSIL         59
#define EVT_STREAM_STOP           60
#define EVT_STREAMIN_COMP		  61
#define	EVT_STREAMOUT_COMP	      62
#define EVT_LOOPFLASH			  64
#define EVT_STREAM_EMPTY          65

#define EVT_FAXRECVEOP                    70    //      Receive FAX page done
#define EVT_FAXSEND_DONE                  71    //      Send FAX page complete
#define EVT_FAXRECV_DONE                  72    //      Receive FAX document complete
#define EVT_FAX_ERROR                     74    //      FAX error
#define EVT_FAX_PHASEB_COMP               75    //      Phase B complete
#define EVT_FAX_PHASED_COMP               76    //      Phase D complete                


//
// Extended events:
//

//
// CPM monitor events (if enabled by MONI_... mask):
//
#define EVT_MON_BUSY             129    // Monitor busy cadence cycle
#define EVT_MON_BUSY_TRUNK       130    // Monitor trunk busy cycle
#define EVT_MON_DIAL             136    // Monitor Dial tone
#define EVT_MON_RINGBACK         142    // Monitor Ring back
#define EVT_MON_DRINGBACK        143    // Monitor Double Ring back
#define EVT_MON_SIT              144    // Monitor Intercept tone
#define EVT_MON_FAX              145    // Monitor Fax tone
#define EVT_CALLID_START         150    // 
#define EVT_CALLID_STOP          152    /* CALLID end event                 */
#define EVT_CALLID_MSGLEN                153
#define EVT_RECOFFHOOK           159
#define EVT_REVERSAL             160


//
// CPM Tone signal name:
//
// (CPM tone signalname in CPM profile structure )
#define CPM_DIAL1                       250
#define CPM_DIAL2                       251
#define CPM_DIAL3                       252
#define CPM_BUSY1                       253
#define CPM_RINGBACK1           254
#define CPM_BUSY2                       255
#define CPM_RINGBACK2           256
#define CPM_FAX1                        258
#define CPM_FAX2                        259
#define CPM_SIT1                        284
#define CPM_SIT2                        285
#define CPM_SIT3                        286
#define CPM_SIT4                        287
#define CPM_RECEIVEOFF_EVT  283

// User Tone Types Definitions
#define  NVUSR_RNGBACK           1     /* Ringback Class                         */
#define  NVUSR_BUSY              2     /* Busy Class                             */
#define  NVUSR_FAX               3     /* FAX tone Class                         */
#define  NVUSR_LOOPOFF           10    /* LOOP_OFF                               */                                
#define  NVUSR_DIGTYPE           5
#define	 NVUSR_NONE				 0   /* None, do not know what it is */

// (CPM tone type in CPM profile structure )
#define CPM_TONE                        0
#define SIT_TONE                        1
#define USR_TONE                        2

//
// Call Progress termination types (for event EVT_CPM_STOP):
//

//
// Standard types:
//
#define CPM_BUSY                   7    // Line is busy
#define CPM_NO_ANSWER              8    // No answer after number of rings
#define CPM_NO_RINGBACK            9    // No ringback after timeout
#define CPM_CONNECT               10    // Call is answered
#define CPM_INTERCEPT             11    // Operator intercept tone
#define CPM_NO_DIAL                               17    // No Dial tone after timeout   
//
// Termination event masks
//
#define TERM_LOOP_DROP        0x00000001    // Terminate on Loop current drop
#define TERM_RINGS            0x00000002    // Terminate on incoming Rings
#define TERM_MON_SIL          0x00000004    // Terminate on Silence detection
#define TERM_MON_NOISE        0x00000008    // Terminate on Noise detection
#define TERM_MON_DIAL         0x00000010    // Terminate on Dial Tone detection
#define TERM_MON_BUSY         0x00000020    // Terminate on Monitor busy
#define TERM_BUSY             0x00000100    // Terminate on Busy
#define TERM_CONNECT          0x00000200    // Terminate on Connect
#define TERM_NO_ANSWER        0x00000400    // Terminate on No Answer
#define TERM_INTERCEPT        0x00000800    // Terminate on Intercept
#define TERM_NOR_DIAL         0x00001000    // Terminate on contin. Dial Tone
#define TERM_NOR_SIL          0x00002000    // Terminate on contin. Silence
#define TERM_NOR_NOISE        0x00004000    // Terminate on contin. Noise
#define TERM_FORCE_LOOP       0x00010000    // Force termination on loop drop
#define TERM_DEFAULT          0x0000FF01    // Default termination mask

//
// Monitor Events masks
//
#define MONI_BUSY             0x00000001    // Monitor Busy event
#define MONI_TBUSY            0x00000002    // Monitor Trunk Busy event
#define MONI_DIAL             0x00000004    // Monitor Dial event
#define MONI_RINGBACK         0x00000020    // Monitor Ringback cycle event
#define MONI_DRINGBACK        0x00000040    // Monitor Double ringback event
#define MONI_SIT              0x00000080    // Monitor Intercept tone event
#define MONI_FAX              0x00000100    // Monitor Fax tone event
#define MONI_REVERSAL         0x00000400    // Monitor polarity changes

#define MONI_DEFAULT                      (MONI_BUSY | MONI_DIAL | MONI_RINGBACK)

//
// Enhanced types (if enabled by TERM_... mask):
//
#define CPM_MON_SILENCE          134    // Termination on silence detection
#define CPM_MON_NOISE            135    // Termination on noise detection
#define CPM_MON_DIAL             136    // Termination on monitor dial tone

// MTSM tone type:
#define MTSM_POWER_RING1				0
#define MTSM_POWER_RING2				1
#define MTSM_BUSY1				2
#define MTSM_BUSY2				3
#define MTSM_RINGBACK1			4
#define MTSM_RINGBACK2			5
#define MTSM_DIAL				6
#define MTSM_USER1				7
#define MTSM_USER2				8

// MTSM Parameters:
#define	MTSM_FLASH				1
#define MTSM_DTMF				2
#define	MTSM_LOOP				3
#define MTSM_STATION			4
#define MTSM_QUERY				5

// MTSM return from MTSM_QUERY channel_type
#define TRUNK				0
#define STATION				1
#define VDS					2


//
// Subtypes in CALL_RESULT.sub_term:
//

//
// CPM_BUSY types:
//
#define CPM_BUSY_NORMAL          129    // Normal busy termination
#define CPM_BUSY_TRUNK           130    // Trunk (fast) busy termination
//
// CPM_NO_RINGBACK types:
//
#define CPM_NOR_DIAL             131    // Continuous dial tone, no ringback
#define CPM_NOR_SILENCE          132    // Continuous silence, no ringback
#define CPM_NOR_NOISE            133    // Continuous noise, no ringback

//
// CPM_CONNECT types:
//
#define CPM_CONNECT_FAX          147    // Connect to fax machine
#define CPM_CONNECT_MODEM        148    // Connect to modem
#define CPM_CONNECT_MACHINE      149    // Ansr machine (prompt beep detect)
#define CPM_CONNECT_LOOP         151    // Connect on loop current drop
#define CPM_CONNECT_VOICE		 152	// Connect on voice 
#define CPM_CONNECT_CADBRK		 153	// Connect on cadence break

//
// Other extended events:
//
#define EVT_REVERSAL             160    // Loop current reversal

//
// DTMF_PARAMS
//
#define DTMF_BREAKOFF              1    // Disable DTMF break
#define DTMF_BREAKON               2    // Enable DTMF break

//
// CPM Modes
//
#define CPMMODE_NONE               0    // CPM off
#define CPMMODE_MONI               1    // CPM monitor
#define CPMMODE_COMP               2    // CPM completion

//
// Phone line status masks (use with CHAN_STATUS.line_status)
//
#define LINE_CPM_COMP           0x01    // CPM completion mode bit
#define LINE_POLARITY           0x02    // Tip-Ring polarity, 0 = positive
#define LINE_RINGBACK           0x04    // CPM detected ringback
#define LINE_ON_HOOK            0x08    // Channel is ON hook
#define LINE_NO_RINGS           0x10    // No incoming rings detected
#define LINE_NO_LOOP            0x20    // No loop current present
#define LINE_DTMF               0x40    // DTMF signal currently present
#define LINE_SILENT             0x80    // Silence on the line detected


//
// Digit Queue Control bits
#define DQ_FLUSH              0x00000001    // Flush digit queue
#define DQ_EVENTS             0x00000002    // Generate digit events

//
// SET_EVENTS bits
//
#define SE_LOFF                 0x01    // Loop current OFF
#define SE_RING                 0x02    // Rings received
#define SE_NOISE                0x04    // Noise detected
#define SE_SIL                  0x08    // Silence detected
#define SE_OFFH                 0x10    // Offhook complete
#define SE_ONH                  0x20    // Onhook complete
#define SE_LON                  0x40    // Loop current ON
#define SE_WKRECV               0x80    // Recv incoming wink
#define SE_LREV                 0x100   // Loop Reversal

//
// Equates for cst_mask
//
#define  C_LC        0x01               // Loop signal drop
#define  C_RING      0x02               // Rings received
#define  C_SILOFF    0x04               // Silence off
#define  C_SILON     0x08               // Silence on
#define  C_OFFH      0x10               // Offhook complete
#define  C_ONH       0x20               // Onhook complete
#define  C_LCON      0x40               // Loop signal on
#define  C_WKRECV    0x80               // Recv incoming wink
#define  C_LREV      0x100              // Loop Reversal

//
// Channel status
//
#define MODE_IDLE                  0
#define MODE_RECORD                1
#define MODE_PLAY                  2
#define MODE_GET_DIGITS            3
#define MODE_OFFHOOK               4
#define MODE_DIALING               5
#define MODE_ONHOOK                6
#define MODE_CPM                   7
#define MODE_PARAM                 8
#define MODE_WINK                  9
#define MODE_PLAYTONE              10
#define MODE_PREREC                12
//
//
// Background Function Code

#define FUNC_IDLE                  0
#define FUNC_HOOK                  6
#define FUNC_RECBUF                10
#define FUNC_PLAYBUF               11
#define FUNC_RECFILE               12
#define FUNC_PLAYFILE              13
#define FUNC_GETDIGITS             15
#define FUNC_PLAYTONE              20    
#define FUNC_WINK                  39
#define FUNC_XCPM                  94
#define FUNC_XDIAL                 95


//
// NVDSP modes
//
#define SET_COMPAT                 0x0000
#define SET_ENHANCE                0x0001


//
// MUX type values
//

#define MUX_NONE                   1    // No mux available
#define MUX_MVIP                   4
//
// Gain step values
//

#define STEP_DOWN                 -1    // Step channel gain down
#define STEP_RESET                 0    // Reset channel gain
#define STEP_UP                    1    // Step channel gain up

//
// Board error codes (use with BOARD_INFO.StartStatus)
//

#define BOARD_IDLE                 0    /* Board is idle                   */
#define BOARD_BLOCKED              1    /* Board does not accept commands  */
#define BOARD_NO_RESP              2    /* Board does not respond          */
#define BOARD_ACK                  3    /* Board responds but nothing else */
#define BOARD_OK                   5    /* Board works fine                */
#define BOARD_CHAN_ERROR           6    /* Board generates bad channel nrs */
#define BOARD_HWERROR              7    /* HW failure reported during reset*/

//
// Direction values when entering a group
//
#define DIR_NONE                             0x00    // Direction not specified
#define DIR_TALK                             0x01    // Direction Talk to group
#define DIR_LISTEN                           0x02    // Direction Listen to group
#define DIR_BOTH                             0x03    // Both Talk and Listion to group
#define DIR_SUPER                            0x04    // Supervisor Mode
#define DIR_STALK          (DIR_SUPER | DIR_TALK)    // Supervisor Talk to group
#define DIR_SLISTEN      (DIR_SUPER | DIR_LISTEN)    // Supervisor Listen to group
#define DIR_SBOTH          (DIR_SUPER | DIR_BOTH)    // Supervisor both Talk and Listen
#define DIR_LEAVE                            0x08    // Leave a group

//
// Group status values
//
#define GROUP_NONE                 0    // Group not in use
#define GROUP_ACTIVE               1    // Group is active
#define GROUP_HOLD                 2    // Group is on hold

/****   Stop mode for NVStopCh(ch,mode) : ***/
#define  DEFAULT                                0               // Stop current function
#define  FLUSH_IRPS                             2               // Stop current function and flush all commands in queue
#define  FLUSH_ALL                              8               // Stop all functions in system and clear channel state

/****  Stream actions: ***/                     
#define  NVDSP_STREAM_INPUT                     2
#define  NVDSP_STREAM_OUTPUT                    1


/****  MTDSP signal route control: ***/
// network type:
#define  MTDSP_TDM_SLOT_PORT		2
#define  MTDSP_CHANNEL_PORT			0
#define  MTDSP_LINE_PORT			3

// mode
#define  MTDSP_ROUTE_SET			0
#define  MTDSP_ROUTE_RESET			1

/****  DSPREGS Control : ****/
#define  NVDSP_MEDIA_RATE        0x8800 /* MP Media rate control            */
#define  NVDSP_VOICE_NV_ERROR    0x8803 /* MP Voice not available           */  
#define  NVDSP_MEDIA_MODE        0x8801 /* MP Media mode                    */         
#define  NVDSP_UAI_MODE          0x1    /* DSP UAI interface control        */         

/****  PXD Control  : ****/
#define  ULAWPXD    1
#define  ALAWPXD    2


#define NV_DTMF                 0
#define NV_MF                   1

/* =============== Global Event Queue  ====================== */
#define NVDSP_GEVT_ENABLE               1
#define NVDSP_GEVT_DISABLE              2
#define NVDSP_GEVT_FLUSHQUEUE   3

/************Voice format for record & play ***************/
#define VF_DEFAULT                  0
#define VF_PCM                      0x100/* Mu Law PCM encoded file          */
#define VF_OKI_ADPCM_SR8            0x200/* 8 KHz sampled file               */
#define VF_OKI_ADPCM_SR6            0x800/* 6 KHz sampled file               */
#define VF_OKI_ADPCM                0x800
#define VF_PCM_ALaw                 0x10/* A Law PCM mode                   */
#define VF_Linear                   0x20/* Linear PCM mode(16 bit) */
#define VF_GSM610					0x400
#define VF_WAV                      0x22/* Playback for all    WAV format   */
					
#define VF_WAV_LINEAR                           0x22/* Recording WAV format- 8000 sampled rate and 16bits*/
#define VF_WAV_ULAWPCM                          0x25/* Recording WAV format- 8000 sampled rate and 8 bits*/
#define VF_WAV_PCM                                      0x25
#define VF_WAV_ALAWPCM                          0x26/* Recording WAV format- 8000 sampled rate and 8 bits*/

#define VF_FORMAT_MGSM610                 0x0031  // Recording in microsoft GSM format
#define VF_FORMAT_MGSM610_WITHOUTHEADER   0x1234  // Recording in microsoft GSM format without header

/*** NVFAX format ***/
#define         NVFAX_RAW               0x0
#define         NVFAX_TIFF              0x1

/*** NVFAX length : ***/
#define         NVFAX_297mm             0x1
#define         NVFAX_UNLIMITED         0x2
#define         NVFAX_NONE              0x0

/*** NVFAX width : ***/
#define         NVFAX_1728              0x1
#define         NVFAX_2048              0x2
#define         NVFAX_2432              0x3

/*** NVFAX resln :  ***/
#define         NVFAX_RESLN_LO          0x1
#define         NVFAX_RESLN_HI          0x2


/*** NVFAX parameter names for NVSetParam()  ***/
#define         NVFAX_LOCALID   0x10
#define         NVFAX_REMOTEID  0x11

/*** NVFAX Attribute Request Defines ***/
#define         NVFAX_BAUDRT            0x3               /*  Fax transfer speed          */
#define         NVFAX_WIDTH             0x4               /*      Negotiated width if phase B done */
#define         NVFAX_RESLN             0x5               /*  Negotiated resolution if phase B done */
#define         NVFAX_PGXFER            0x6               /*      number of pages transferred */
#define         NVFAX_STATUS            0x8
#define         NVFAX_TIFBADTAG         0xB
#define         NVFAX_TIFMSTAG          0xC

/*** NVFAX status values  for Attribute NVFAX_STATUS ***/
#define         NVFAX_IDLE              0
#define         NVFAX_SENDING           1
#define         NVFAX_RCVING            2

/*** NVFAX Resolution status values  for Attribute NVFAX_RESLN ***/
#define         NVFAX_HIGH_RESL         0
#define         NVFAX_LOW_RESL          8


/***  NVFAX Phase B Status Values ***/

#define         NVFAX_BSTS_NONE       0x0     /* no status, Phase B not done */
#define         NVFAX_BSTS_REMOTEID   0x1     /* Remote Id available on Phase B */


/*** NVFAX Phase D Commands ***/
#define  NVFAX_DSTS_EOP         0x0    /* end of procedure */
#define  NVFAX_DSTS_MPS         0x1    /* end of page */
#define  NVFAX_DSTS_EOM         0x2    /* end of message */
#define  NVFAX_DSTS_POLL        0x3    /* poll request */
#define  NVFAX_DSTS_PRI_EOP     0x10   /* request for operator intervention (PRI_EOP) sent */
#define  NVFAX_DSTS_PRI_MPS     0x11   /* request for operator intervention (PRI_MPS) sent */
#define  NVFAX_DSTS_PRI_EOM     0x12   /* request for operator intervention (PRI_EOM) sent */

/*** FAX Phase D Replies ***/
#define  NVFAX_DRPLY_MCF         0x1    /* message confirmation - do next page */
#define  NVFAX_DRPLY_RTN         0x2    /* retrain negative */
#define  NVFAX_DRPLY_RTP         0x3    /* retrain positive */
#define  NVFAX_DRPLY_PIP         0x4    /* procedure interrupt positive */
#define  NVFAX_DRPLY_PIN         0x5    /* procedure interrupt negative */

// Board Definitions, added by FLM.

#define BOARD_8XLV3_HS 0x0
#define BOARD_8XLV3PLUS 0x1
#define BOARD_8XLV3MINUS_8XLEZ 0x2
#define BOARD_4XL 0x3
#define BOARD_LT4HS 0x4
#define BOARD_LT4HS_PLUS 0x5
#define BOARD_LT4EZ 0x6
#define BOARD_LT2HS 0x7
#define BOARD_LT2HS_PLUS  0x8
#define BOARD_LT2EZ 0x9
#define BOARD_MT1X1 0x10
#define BOARD_MTP8 0x20
#define BOARD_MTP8PLUS_ASIA 0x21
#define BOARD_MTP8F 0x22
#define BOARD_MTP4 0x23
#define BOARD_MTP4PLUS_ASIA 0x24
#define BOARD_MT8X8 0x25
#define BOARD_MTR16 0x26
#define BOARD_MT160 0x27
#define BOARD_MTP16 0x28
#define BOARD_MT4X12 0x29
#define BOARD_MTP8_PCI 0x30
#define BOARD_MTP4_PCI 0x31
#define BOARD_MTP2_PCI 0x32
#define BOARD_MTP1X1_PCI 0x33
#define BOARD_MTP2X6_PCI 0x34
#define BOARD_MTP4X4_PCI 0x35
#define BOARD_MT80_PCI 0x36
#define BOARD_MTR8_PCI 0x37
#define BOARD_MTR4_PCI 0x38
#define BOARD_MTFX44_PCI 0x39
#define BOARD_MT2X2_PCI 0x40
#define BOARD_MT6X2_PCI 0x41
#define BOARD_MT2HA_PCI 0x42
#define BOARD_PT4PCI 0x46
#define BOARD_PT8PCI 0x47

#ifdef __cplusplus
} /* end extern "C" */
#endif

// 6-24
#include <poppack.h>
#endif
