/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
#include "brd_dialogic_code.h"
#include "..\TFServer\tf_registry.h"
#ifdef TF_CLOSED_SOURCE
#include "..\tts\tts_code.h"
#endif
#include "..\Voice\Voice.h"
#include "..\TFEngine\TFEngine_communication.h"
// --------------------------------------------------------------------------
//  Name:       OpenHardwareDriver
// --------------------------------------------------------------------------
bool OpenHardwareDriver(Log* /*sysLog*/)
{
   return(true);
}
//OpenHardwareDriver

// --------------------------------------------------------------------------
//  Name:       CloseHardwareDriver
// --------------------------------------------------------------------------
bool CloseHardwareDriver(Log* /*sysLog*/)
{
	return(true);
}
//CloseHardwareDriver

/* -------------------------------------------------------------------------
   Class:       DialogicBoardEvent

   Purpose:     Implements the functions defined in "BoardGeneric" class.
   ------------------------------------------------------------------------- */
DialogicBoardEvent::DialogicBoardEvent(void)                // (Constructor)
{
	// Nothing required here
}

DialogicBoardEvent::~DialogicBoardEvent(void)               //  (Destructor)
{
	// Nothing required here
}

bool DialogicBoardEvent::SameEvent(BoardEvent* comparisonEvent)
{
	return(DlgEvent.evtType == ((DialogicBoardEvent*) comparisonEvent)->DlgEvent.evtType);
}

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::BrdDialogic                       (Constructor)
   ------------------------------------------------------------------------- */
BrdDialogic::BrdDialogic(PIVRINFO pIVRInfo) : BoardGeneric(pIVRInfo)
{
   myState = good;   // Assume application will start okay.
   shutdownRequested = false;
   safeToShutdown = true;
   speakAudioExtension = ".vox";
   preferredFormat = TF_VOX;

   if(pInfo->pszProtocol == NULL)
   {
      pInfo->pszProtocol = "lps0";
   }

#if defined IV_TTS
   char *pszTTSPath;

   pszTTSPath = getenv ("TTS_PATH");
   if (pszTTSPath)
      SetTTSDirectory (pszTTSPath);
#endif

   InitializeBoard();
}
/*BrdDialogic::BrdDialogic*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::~BrdDialogic                       (Destructor)
   ------------------------------------------------------------------------- */
BrdDialogic::~BrdDialogic(void)
{
   ReleaseLine();
}
/*BrdDialogic::~BrdDialogic*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::InitializeBoard                       (private)

   Purpose:     Initialize Dialogic boards, ie: link to Dialogic
                DLL and query the boards in the system.

   Notes:		 Modified to provide more output to the log.
                Also, bBrdInit will now be set to false if
                QueryBoard fails. - TF Oct.4/01
   ------------------------------------------------------------------------- */
void BrdDialogic::InitializeBoard(void)
{
   ivLog->Debug ('f', "Opening driver...\n");
   ivLog->Debug ('f', " Hardware: Dialogic\n");
   ivLog->Debug ('f', " Linking to Dialogic library...\n");
   sr_libinit (DLGC_MT);      // link to Standard Runtime Library
   dx_libinit (DLGC_MT);      // link to Voice Software library
   dt_libinit (DLGC_MT);      // link to Digital Network Interface library

	QueryBoard();
   // This does not work! Dialogic's programming model is not
   // compatible with our approach - TF Dec.1/01
	// 	OpenEventQueue();

   // The above functions changes the global variable 'myState' to bad
   // if they could not open at least one Dialogic board.
   if (myState == good)
   {
      ivLog->Debug ('f', "Driver successfully opened.\n");
      bBrdInit = true;
   }
   else
   {
      ivLog->Debug ('f', "Failed to open driver.\n");
      bBrdInit = false;
   }

   // Output contents of Lines array - TF Dec.19/01
   // Display port information one-based - TF Dec.17/02
	ivLog->Print ("Port  bd#       board  chnl         trunk  type  stream  slot\n");
	ivLog->Print ("----  ---  ----------  ----  ------------  ----  ------  ----\n");
   for (int i = 0; i <= lastLine+1; i++)
   {
      ivLog->Print ("%4d  %3d  %10s  %4d  %12s  %4d  %6d  %4d\n",
                    i+1,
                    lines[i].board,
                    lines[i].boardName,
                    lines[i].channel,
                    lines[i].trunkName,
                    lines[i].trunkType,
                    lines[i].stream,
                    lines[i].slot );
   }

   if((bBrdInit) && (pInfo->iPort > 0))
   {
      // for current port, translate dialogic trunk type into TeleFlow type
      switch(lines[pInfo->iPort-1].trunkType)
      {
         case CT_NTNONE:
         {
            trunkType = TF_TRUNK_TYPE_NONE;
            break;
         }
         case CT_NTT1:
         {
            trunkType = TF_TRUNK_TYPE_T1;
            break;
         }
         case CT_NTE1:
         {
            trunkType = TF_TRUNK_TYPE_E1;
            break;
         }
         case CT_NTANALOG:
         {
            trunkType = TF_TRUNK_TYPE_ANALOG;
            break;
         }
         case CT_NTIPT :   // VoIP not supported
            // trunkType = TF_TRUNK_TYPE_VOIP;
            // break;
         case CT_NTMSI:    // MSI/SC station interface boards not supported
         default:
         {
            trunkType = TF_TRUNK_TYPE_INVALID;
         }
      }
   }
}
/*BrdDialogic::InitializeBoard*/

/* ENTIRE METHOD REMOVED - DOES NOT WORK - SEE NOTES - TF Dec.01/01
/o -------------------------------------------------------------------------
   Name:        BrdDialogic::OpenEventQueue                        (private)

   Purpose:     Links the Dialogic event queue(s) with the Engine wait
                objects so all events can be monitored through our Win32
                event handler, 'BoardGeneric::WaitForEvent'.

   Notes:       This function does not work with TeleFlow. Dialogic's
                "Asynchronous with Win32" callback model will only
                work in a single-threaded program.
                (see Dialogic SR 5.01, Voice Software Reference, Standard
                Runtime Library for more information on SRL programming
                models)
                 - TF Dec.1/01
   ------------------------------------------------------------------------- o/
void BrdDialogic::OpenEventQueue()
{
	HANDLE        hEvt;                // Win32 event handle
   SRLWIN32INFO  SrlWin32Info;        // Dialogic structure for Win32 event info
   int           iTemp = SR_STASYNC;  // temporary integer value

   if((curLine == -1) && ((status = RequestLine(pInfo->iPort)) != IVR_SUCCESS))
   {
      return(status);
   }

   ivLog->Debug ('f', " Creating event queue...\n");

   ivLog->Debug ('f', "      CreateEvent\n");
	hEvt = CreateEvent(NULL,
                      FALSE,
                      FALSE,
                      NULL);
   ivLog->Debug ('f', "           Event handle 0x%x\n", hEvt);
	if (hEvt == NULL)
	{
   	ivLog->Debug ('f', "Failed to setup event handler.\n");
		myState = bad;
      return;
   }

	// Create a new wait object array to accomodate the Dialogic wait objects
//   numWaitObjects = numWaitObjects + 1;
//   HANDLE* newWaitObjects = new HANDLE[numWaitObjects];

 	// Copy the existing wait object array
//   newWaitObjects[0] = waitObjects[0];
//   newWaitObjects[1] = waitObjects[1];
//   newWaitObjects[2] = waitObjects[2];
//   newWaitObjects[3] = hEvt;
   waitObjects[3] = hEvt;

	// Destroy the old wait object array
//   delete[] waitObjects;

 	// Assign the new wait object array
//   waitObjects = newWaitObjects;
   ivLog->Debug ('f', "           numWaitObjects = %d\n", numWaitObjects);
   ivLog->Debug ('f', "           waitObjects[0] = 0x%x\n", waitObjects[0]);
   ivLog->Debug ('f', "           waitObjects[1] = 0x%x\n", waitObjects[1]);
   ivLog->Debug ('f', "           waitObjects[2] = 0x%x\n", waitObjects[2]);
   ivLog->Debug ('f', "           waitObjects[3] = 0x%x\n", waitObjects[3]);
// ivLog->Debug ('f', "           waitObjects[4] = 0x%x\n", waitObjects[4]);

   ivLog->Debug ('f', "      Set SrlWin32Info\n");
   SrlWin32Info.dwTotalSize = sizeof(SRLWIN32INFO);
   SrlWin32Info.dwHandleType = SR_RESETEVENT;
   SrlWin32Info.ObjectHandle = waitObjects[3];

   ivLog->Debug ('f', "      sr_setparm, ObjectHandle = 0x%x\n", waitObjects[3]);
   if (sr_setparm(SRL_DEVICE, SR_WIN32INFO, (void *)&SrlWin32Info) == -1)
// DOES NOT WORK:
//   ivLog->Debug ('f', "      sr_setparm, SR_MODELTYPE @ 0x%x = %d\n", &iTemp, iTemp);
//   if (sr_setparm(lines[curLine].chdev, SR_MODELTYPE, &iTemp) == -1)
//   {
//   	ivLog->Debug ('f', "Failed to setup event handler.\n");
//		myState = bad;
//   }
//   ivLog->Debug ('f', "      sr_setparm, ObjectHandle = 0x%x\n", waitObjects[3]);
//   if (sr_setparm(lines[curLine].chdev, SR_WIN32INFO, (void *)&SrlWin32Info) == -1)
//   {
//   	ivLog->Debug ('f', "Failed to setup event handler.\n");
//		myState = bad;
//   }

   ivLog->Debug ('f', " Event Queue Done.\n");
	return;
}
/oBrdDialogic::OpenEventQueue o/
// END OF SECTION REMOVED - TF Dec.01/01
*/


/* -------------------------------------------------------------------------
   Name:        BrdDialogic::ProcessBoardEvent                     (private)

   Purpose:     Handles a board-level event that has been passed through
                our Win32 event handler, 'BoardGeneric::WaitForEvent'.

   Notes:       This function does not work with TeleFlow. Dialogic's
                "Asynchronous with Win32" callback model will only
                work in a single-threaded program.
                (see Dialogic SR 5.01, Voice Software Reference, Standard
                Runtime Library for more information on SRL programming
                models)
                 - TF Dec.1/01
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::ProcessBoardEvent(BoardEvent*, bool&)
{
	return(IVR_SUCCESS);
   /*
   // ENTIRE SECTION REMOVED - DOES NOT WORK - SEE NOTES - TF Dec.01/01
	long  rc;              // result code
   long  devh;            // Dialogic device handle

	ivLog->Print (" Process Board Event...\n");

   DLG_EVENT* eventp = &(((DialogicBoardEvent*) boardEvent)->DlgEvent);

   // When WaitForMultipleEvents signals that a Dialogic event has occurred,
   // call sr_waitevtEx() with a zero timeout to retrieve the event.
   ivLog->Print ("      sr_waitevtEx, lines[curLine].chdev = %d\n", lines[curLine].chdev);
	rc = sr_waitevtEx((long *)&lines[curLine].chdev, 1, 0, &(eventp->evtHandle));
	if (rc != 0)
   {
		ivLog->Print ("ERROR: Unexpected result %d while waiting for event\n", rc);
		return IVR_FATAL;
   }

   // Gather data about the event
   ivLog->Print ("      sr_getevtdev... ");
	devh = sr_getevtdev(eventp->evtHandle);
   ivLog->Print ("0x%x\n", devh);

   ivLog->Print ("      sr_getevttype... ");
   eventp->evtType = sr_getevttype(eventp->evtHandle);
   ivLog->Print ("0x%x\n", eventp->evtType);

   ivLog->Print ("      sr_getevtlen... ");
   eventp->datalen = sr_getevtlen(eventp->evtHandle);
   ivLog->Print ("%d\n", eventp->datalen);
   if (eventp->datalen > 0)
   {
      eventp->datap = sr_getevtdatap(eventp->evtHandle);
   } else {
		eventp->datap = NULL;
   }

   // Display information about the event in the log
	if (ivLog->GetDebug ('S'))
   {
      ivLog->Print ("      ShowEvent\n");
	   ShowEvent (eventp);
   }
   return(IVR_SUCCESS);
	*/
}
/*BrdDialogic::ProcessBoardEvent*/

/*
// ENTIRE METHOD REMOVED - DOES NOT WORK - SEE NOTES - TF Dec.01/01
/o -------------------------------------------------------------------------
   Name:        BrdDialogic::ShowEvent                             (private)

   Purpose:     Displays Dialogic events in the log

   Notes:       This function does not work with TeleFlow. Dialogic's
                "Asynchronous with Win32" callback model will only
                work in a single-threaded program.
                (see Dialogic SR 5.01, Voice Software Reference, Standard
                Runtime Library for more information on SRL programming
                models)
                 - TF Dec.1/01
   ------------------------------------------------------------------------- o/
void BrdDialogic::ShowEvent (DLG_EVENT *eventp)
{
	ivLog->Print ("Dialogic event: ");
	switch (eventp->evtType)
	{
		case TDX_PLAY:
			ivLog->Print ("Play Completed");
			break;
		case TDX_RECORD:
			ivLog->Print ("Record Completed");
			break;
		case TDX_GETDIG:
			ivLog->Print ("Get Digits Completed");
			break;
		case TDX_DIAL:
			ivLog->Print ("Dial Completed");
			break;
		case TDX_CALLP:
			ivLog->Print ("Call Progress Completed");
			break;
		case TDX_CST:
			ivLog->Print ("CST Event Received, ");
         DX_CST* pDxCst;
         pDxCst = (DX_CST*)eventp->datap;
         switch (pDxCst->cst_event)
         {
         	case DE_RINGS:
					ivLog->Print ("Ring");
               break;
				case DE_SILON:
            	ivLog->Print ("Silence on");
               break;
				case DE_SILOF:
					ivLog->Print ("Silence off");
               break;
				default:
            	ivLog->Print ("0x%x",pDxCst->cst_event);
			} // end switch
			break;
		case TDX_SETHOOK:
         ivLog->Print ("SetHook Completed");
         break;
      case TDX_WINK:
         ivLog->Print ("Wink Completed");
         break;
      case TDX_ERROR:
         ivLog->Print ("Error Event");
         break;
      case TDX_PLAYTONE:
         ivLog->Print ("Play Tone Completed");
         break;
      case TDX_GETR2MF:
         ivLog->Print ("Get R2MF completed");
         break;
      case TDX_BARGEIN:
         ivLog->Print ("Barge in completed");
         break;
      case TDX_NOSTOP:
         ivLog->Print ("No Stop needed to be Issued");
         break;
      case TDX_TXDATA:
         ivLog->Print ("Transmit Data Completed");
         break;
      case TDX_RXDATA:
         ivLog->Print ("Transmit Data Completed");
         break;
      case DX_ATOMIC_ERR:
         ivLog->Print ("Error event for Atomic Funcs");
         break;
      default:
         ivLog->Print ("Unknown event type %d\n", eventp->evtType);
			ivLog->Print ("     data: length %d, datap 0x%x", eventp->datalen, eventp->datap);
         break;
	} // end switch
	ivLog->Print ("\n");
	return;
}
/oBrdDialogic::ShowEvento/
// END OF SECTION REMOVED - TF Dec.01/01
*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::QueryBoard                            (private)

   Purpose:     Queries all the boards in the computer one after the other
                to determine the total number of available ports on the
                board.  All ports are added to the array of total lines.

   Modified:    TF Nov.02/01 - Put retry counter around Dialogic API
                     calls because they would occassionally fail

                TF Jan.17/02 - Added sleep to stagger the querying of
                     the boards to avoid unnecessary errors generated
                     by Dialogic API

                TF Oct.10/02 - Modified sleep before querying the boards
                     to 1 second + 200 ms per port number.

                JSH 2003-06-16: increased the base pause value to 45000 ms
                     to compensate for the fact that with starting TFServer as
                     a service, boards *appear* take longer to boot. Also added a
                     bit of sleep code for when the board is being queried.
   ------------------------------------------------------------------------- */
void BrdDialogic::QueryBoard(void)
{
   enum         MODES {ANALOG,DIGITAL}; // mode of network connection
   CT_DEVINFO   devinf;                 // Dialogic device info structure
   MODES        prevMode = ANALOG;      // mode of previous voice board
   int          rc;                     // misc. result code capture
	DWORD        iPause;                 // pausing before thread queries boards
   int          trycount;               // loop control counter for retries
   #define      MAXTRIES 12              // constant for max loop count (used in 3 places)

   int          voxcnt = 0;             // number of voice boards in system
   int          curvoxbd;               // current voice board
   char         szVoxNm[8];             // voice board name
	int          voxChnls;               // number of channels on a voice board device
   char         szChnlNm[12];           // channel name
   int          voxBDev;           // voice board device handle
   int			 chdev;                  // channel device handle

   int          dticnt = 0;             // number of DTI boards in system
   int          curdtibd = 0;           // current DTI board
   int			 tsUnused = 0;           // # of un-used timeslot entries in the
                                        //      total lines array

   LONG  regret;
   HKEY  hkey;
   DWORD dwDisposition;

   // read registry setting
   regret = RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey);
   if (regret != ERROR_SUCCESS)
   {
      // delete[] serviceInfo->pszBoardService;
      // memset (&serviceInfo, 0, sizeof(REG_SERVICE_INFO));
      ivLog->Print("Cannot open reg key for TFServer\n");
      RegCloseKey (hkey);
   }

   // WINSTART
   // BOARDSTART
   // Checking our board-started date/time vs. Windows started date/time
   // has been moved to TFServer level - TF 2006/11/28

   ivLog->Print (" Querying boards...\n");

   // Sleep to stagger the threads querying the boards to avoid Dialogic API errors
   iPause = (pInfo->iPort * 100);
   ivLog->Print ("      Timing pause %d milliseconds...\n", iPause);
   Sleep (iPause);

	// initialize array to zeros
   memset(lines, 0, sizeof(BRD_LINE) * MAX_LINES);

   // query for number of voice boards in system
   // Put retry counter in here because board count would occassionally fail - TF Nov.2/01
   ivLog->Print ("    Voice boards: ");
   rc = -1;
   for (trycount = 0; trycount < MAXTRIES && rc == -1; trycount++)
   {
	   rc = sr_getboardcnt(DEV_CLASS_VOICE, &voxcnt);
      Sleep (100);  // JSH increased this from 10
   }
   if (rc == -1)
   {
   	ivLog->Debug ('f', "error!\n");
		myState = bad;
      return;
   }
   ivLog->Debug ('f', "%d\n", voxcnt);


   // JSH 2003-10-03: assume the board has booted if you can see this....
   // so set the BOARDSTART registry key
   time_t systime;
   struct tm *time_now;

   unsigned char chrSysTime[15];
   time(&systime);
   time_now = localtime(&systime);

   strftime((char*) chrSysTime, 15, "%Y%m%d%H%M%S", time_now);
   chrSysTime[14] = '\0';

   ivLog->Print("Board start at %s\n", chrSysTime);

    regret = RegOpenKeyEx (HKEY_LOCAL_MACHINE, TFSERVER_REG_KEY, 0, KEY_READ, &hkey);
    regret += RegCreateKeyEx(
       HKEY_LOCAL_MACHINE,        // Handle of an open key.
       TFSERVER_REG_KEY,          // Address of subkey name.
       0,                         // Reserved.
       0,                         // Address of class string.
       REG_OPTION_NON_VOLATILE,   // Special options flag.
       KEY_WRITE,                 // Desired security access.
       0,                         // Address of key security structure.
       &hkey,                     // Address of buffer for opened handle.
       &dwDisposition);           // Address of disposition value buffer.

    if (regret != ERROR_SUCCESS)
    {
     // delete[] serviceInfo->pszBoardService;
     // memset (&serviceInfo, 0, sizeof(REG_SERVICE_INFO));
     ivLog->Print("Cannot open reg key for TFServer\n");
     RegCloseKey (hkey);
    }

    regret = RegSetValueEx(hkey,"BOARDSTART", 0, REG_SZ,  chrSysTime, 15);
    if (regret != ERROR_SUCCESS)
    {
    ivLog->Print("Could not set the BOARDSTART key.\n");
    }

    RegCloseKey (hkey);

   // query for number of DTI boards in system
   ivLog->Debug ('f', "    DTI boards:   ");
   rc = -1;
   for (trycount = 0; trycount < MAXTRIES && rc == -1; trycount++)
   {
   	rc = sr_getboardcnt(DEV_CLASS_DTI, &dticnt);
      Sleep (100);  // JSH increased this from 10
   }
   if (rc == -1)
   {
   	ivLog->Debug ('f', "error!\n");
      myState = bad;
      return;
   }
   ivLog->Print ("%d\n", dticnt);

//  && voxBDev == -1

   for (curvoxbd = 1; curvoxbd <= voxcnt; curvoxbd++)
   {
   	// Open voice board device
      szVoxNm[0] = '\0';
      sprintf(szVoxNm,"dxxxB%d",curvoxbd);
      ivLog->Print ("    open %-8s\n", szVoxNm);
      voxBDev = -1;
      for (trycount = 0; trycount < MAXTRIES && voxBDev == -1; trycount++)
      {
   	   voxBDev = dx_open(szVoxNm, 0);
         if (voxBDev == -1)
         {
            ivLog->Print ("Information not yet available for board %d.\n", curvoxbd);
            ivLog->Print ("Sleeping for %d ms\n", 1500);
            Sleep (1500);
         }
         else
         {
            ivLog->Print ("Information available for board %d.\n", curvoxbd);
         }

      }
      if(voxBDev == -1)
      {
         ivLog->Print ("Error getting board information\n");
         myState = bad;
         break;
      }
      else
      {
         ivLog->Print ("Success getting board information\n");
      }



		// Get number of channels
		ivLog->Debug ('f', " channels: ");
      voxChnls = ATDV_SUBDEVS(voxBDev);
      ivLog->Debug ('f', "%-3d", voxChnls);

		// Get voice device information
      memset (szChnlNm, 0, sizeof(szChnlNm));
		sprintf (szChnlNm, "%sC1", szVoxNm);
		ivLog->Debug ('f', " open: %-12s network: ", szChnlNm);
		chdev = dx_open(szChnlNm, 0);
		if (chdev != -1 && dx_getctinfo(chdev, &devinf) != -1)
		{
			// network type
         switch (devinf.ct_nettype)
			{
			case CT_NTNONE:
			case CT_NTT1:
			case CT_NTE1:
				// This is either a T1/E1 device or has no network frontend
				ivLog->Debug ('f', "Digital\n");
				if (prevMode == ANALOG)
            {
            	curdtibd++;
               if (curdtibd <= dticnt)
               {
               	if (QueryDTIBoard(curdtibd) == -1)
                  {
							ivLog->Debug ('f', "\nError: could not determine"
                                        " configuration of DTI %d\n", curdtibd);
                     myState = bad;
                     return;
                  }
						// Fill in analog channels into the digital timeslots entries
						tsUnused = FillAnalogLines(curvoxbd, voxChnls, szVoxNm);
               }
               else
               {
						ivLog->Print ("\nOut of digital timeslots!  Any additional"
												 " voice channels will not be available.\n");
               }
            }
				else
				{
            	// Do we have any un-used timeslots in the array?
               if (tsUnused <= 0)
               {
						// Open next DTI board device
	            	curdtibd++;
	               if (curdtibd <= dticnt)
   	            {
	               	if (QueryDTIBoard(curdtibd) == -1)
	                  {
								ivLog->Debug ('f', "\nError: could not determine"
	                                        " configuration of DTI %d\n", curdtibd);
								myState = bad;
	                     return;
	                  }
      	            /* Should we fill in any unused channels from previous analog device? - TF Oct.25/01
            	      if (tsUnused < 0)
                  		@@@ = FillAnalogLines(curvoxbd, -tsUnused, szVoxNm)
   	               */
						}
                  else
                  {
							ivLog->Debug ('f', "\nOut of digital timeslots!  Any additional"
													 " voice channels will not be available.\n");
                  }
               }
					// Fill in analog channels into the digital timeslots entries
               tsUnused = FillAnalogLines(curvoxbd, voxChnls, szVoxNm);
				}
				prevMode = DIGITAL;
				break;

			case CT_NTANALOG:
         default:
				// This is an analog frontend device
				ivLog->Debug ('f', "Analog\n");
            // Add analog channels to end of lines array
            AddAnalogLines(curvoxbd, voxChnls, szVoxNm, devinf.ct_nettype);
            prevMode = ANALOG;
            break;
			}

			dx_close (chdev);
      }

      dx_close (voxBDev);

   }
	ivLog->Debug ('f', " Total ports: %d\n", lastLine + 1);
	return;
   #undef MAXTRIES
}
/*BrdDialogic::QueryBoard*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::QueryDTIBoard                         (private)

   Purpose:     Open a Dialogic DTI board and add the number of timeslot
                devices to the total lines array.

   Returns:		 number of timeslots
                -1 on error
   ------------------------------------------------------------------------- */
int BrdDialogic::QueryDTIBoard(int bdnum)
{
   CT_DEVINFO  devinf;            // Dialogic device info structure
   char        bdname[8];         // board name
	int	      bdev;				    // board device handle
   int         tscount;           // timeslot count
   char        tsname[12];        // timeslot name
   int         tsdev;             // timeslot device handle
   bool        goodnet = true;    // network type is supported by TeleFlow

	// Open DTI board device
   sprintf(bdname,"dtiB%d",bdnum);
   ivLog->Debug ('f', "    open %-8s", bdname);
   if ((bdev = dt_open(bdname, 0)) == -1) {
   	ivLog->Debug ('f', "- error!\n");
      return -1;
   }

   // Get number of timeslots
   ivLog->Debug ('f', "  timeslots: ");
   tscount = ATDV_SUBDEVS(bdev);
   ivLog->Debug ('f', "%-3d", tscount);

   // close the DTI board (no longer needed)
	dt_close(bdev);

	// Get device information
	sprintf (tsname, "%sT1", bdname);
	tsdev = dt_open(tsname, 0);
	ivLog->Debug ('f', "  network: ");
	if (tsdev != -1 && dt_getctinfo(tsdev, &devinf) != -1)
	{
		// network type
		switch (devinf.ct_nettype)
		{
		case CT_NTT1:
			ivLog->Debug ('f', "T1\n");
			break;
		case CT_NTE1:
			ivLog->Debug ('f', "E1\n");
			break;
		case CT_NTNONE:
			ivLog->Debug ('f', "None\n");
         goodnet = false;
			break;
      case CT_NTANALOG :
			ivLog->Debug ('f', "Analog\n");
         goodnet = false;
			break;
		case CT_NTMSI :
			ivLog->Debug ('f', "MSI\n");
         goodnet = false;
			break;
		case CT_NTIPT :
			ivLog->Debug ('f', "IPT\n");
         goodnet = false;
			break;
		default :
			ivLog->Debug ('f', "Unknown %d\n", devinf.ct_nettype);
         goodnet = false;
		}
		dt_close (tsdev);
      if (!goodnet)
      {
      	ivLog->Debug ('f', "Error: network type not supported.\n");
         return -1;
      }
	}
   else
	{
   	ivLog->Debug ('f', "Error opening timeslot device %s\n", tsname);
      return -1;
	}

   // Add digital timeslots to end of lines array
   AddDigitalLines(bdnum, tscount, bdname, devinf.ct_nettype);
	return (tscount);
}
/*BrdDialogic::QueryDTIBoard*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AddAnalogLines                        (private)

   Purpose:     Adds analog channels to the end of the total lines array.

   Note:        The offset into the total lines array is used as the
					 TeleFlow line ("IVR Port") number.
   ------------------------------------------------------------------------- */
void BrdDialogic::AddAnalogLines (int voxBdNo, int numChnls, char *VoxBdNm, char nettype)
{
	int   index;    // loop counter index

	if (lastLine == MAX_LINES - 1) {
		printf ("Max line (%d) count reached.  Additional lines will not function.\n"
		        "Contact enGenic to request a software update.\n", MAX_LINES);
		// WAS:
		// myState = bad;
      // Allow functionality to continue on the first MAX_LINES - TF Oct.25/01
      return;
	}

   for (index = 1; index <= numChnls && lastLine < MAX_LINES - 1; index++)
   {
      lastLine++;
      lines[lastLine].inUse     = false;           // resource not in use
      strcpy(lines[lastLine].boardName, VoxBdNm);  // voice board name
      lines[lastLine].board     = voxBdNo;         // voice board number
      lines[lastLine].channel   = index;           // voice channel number
      lines[lastLine].chdev     = -1;              // voice channel handle, -1 = not in use
      lines[lastLine].trunkType = nettype;         // trunk type (Analog, T1, E1, etc.)
      strcpy(lines[lastLine].trunkName, "");       // dti board name
      lines[lastLine].stream    = 0;               // dti board number
      lines[lastLine].slot      = 0;               // dti timeslot number
      lines[lastLine].tsdev     = -1;              // timeslot device handle, -1 = not in use
   }
}
/*BrdDialogic::AddAnalogLines*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AddDigitalLines                       (private)

   Purpose:     Adds digital timeslots to the end of the total lines array.

   Note:        The offset into the total lines array is used as the
					 TeleFlow line ("IVR Port") number.
   ------------------------------------------------------------------------- */
void BrdDialogic::AddDigitalLines (int dtiBdNo, int numLines, char *dtiBdNm, char nettype)
{
	int   index;    // loop counter index

	if (lastLine == MAX_LINES - 1) {
		printf ("Max line (%d) count reached.  Additional lines will not function.\n"
		        "Contact enGenic to request a software update.\n", MAX_LINES);
		// WAS:
		// myState = bad;
      // Allow functionality to continue on the first MAX_LINES - TF Oct.25/01
      return;
	}

   for (index = 1; index <= numLines && lastLine < MAX_LINES - 1; index++)
   {
      lastLine++;
      lines[lastLine].inUse     = false;           // resource not in use
      strcpy(lines[lastLine].boardName, "");		   // voice board name
      lines[lastLine].board     = 0;			      // voice board number
      lines[lastLine].channel   = 0;		         // voice channel number
      lines[lastLine].chdev     = -1;              // voice channel handle, -1 = not in use
      lines[lastLine].trunkType = nettype;         // trunk type (T1/E1)
      strcpy(lines[lastLine].trunkName, dtiBdNm);  // dti board name
      lines[lastLine].stream    = (char) dtiBdNo;  // dti board number
      lines[lastLine].slot      = (char) index;    // dti timeslot number
      lines[lastLine].tsdev     = -1;              // timeslot device handle, -1 = not in use
   }
   return;
}
/*BrdDialogic::AddDigitalLines*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::FillAnalogLines                       (private)

   Purpose:     Fill in analog channels into the digital timeslots entries
                of the total lines array.

   Returns:		 Number of digital entries left un-filled at the end of
                the array.  A negative number means there were more analog
                channels than available timeslots.

   Note:        The offset into the total lines array is used as the
					 TeleFlow line ("IVR Port") number.
   ------------------------------------------------------------------------- */
int BrdDialogic::FillAnalogLines (int voxBdNo, int numChnls, char *VoxBdNm)
{
	int   index;    // loop counter index
   int	pos;      // position offset to change within the array

	pos = lastLine;
   while (pos >= 0 && lines[pos].board == 0)
   	pos--;

   for (index = 1; index <= numChnls && pos < lastLine; index++)
   {
      pos++;
      lines[pos].board = voxBdNo;               // voice board number
      lines[pos].channel = index;               // voice channel number
      strcpy(lines[pos].boardName, VoxBdNm);    // voice board name
   }

   if (pos == lastLine)
	   return (index - 1 - numChnls);  // return zero, or number of channels
                                      // not used as a negative number
   else
   	return (lastLine - pos);        // return number of un-used timeslots in the
                                      // array as a positive number
}
/*BrdDialogic::FillAnalogLines*/

// ****************************************************************************
// ************************** LINE FUNCTIONS **********************************
// ****************************************************************************

IV_STATUS BrdDialogic::StartReorderToneDetection(void)
{
   // Only activate reorder tone detection with an active line
   // Return IVR_SUCCESS otherwise 
   if(curLine != -1)
   {
      if(dx_deltones(lines[curLine].chdev) != 0)
      {
         ivLog->Print("   ERROR: StartReorderToneDetection::dx_deltones failed\n");
         ivLog->Print("      '%s'\n", ATDV_ERRMSGP(lines[curLine].chdev));
         return(IVR_FAIL);
      }

      // Set up reorder tone
      if(dx_blddtcad(
         101,                                                           // First available tone ID
         reorderTone.frequency1,                                        // First tone
         25,                                                            // First tone deviation
         reorderTone.frequency2,                                        // Second tone
         25,                                                            // Second tone deviation
         (reorderTone.maximumOnTime + reorderTone.minimumOnTime)/20,    // Nominal on-time      (10 ms intervals)
         (reorderTone.maximumOnTime - reorderTone.minimumOnTime)/20,    // On-time deviation    (10 ms intervals)
         (reorderTone.maximumOffTime + reorderTone.minimumOffTime)/20,  // Nominal off-time     (10 ms intervals)
         (reorderTone.maximumOffTime - reorderTone.minimumOffTime)/20,  // Off-time deviation   (10 ms intervals)
         2) != 0)    // Repeat twice before detection
      {
         ivLog->Print("   ERROR: StartReorderToneDetection::dx_blddtcad failed\n");
         ivLog->Print("      '%s'\n", ATDV_ERRMSGP(lines[curLine].chdev));
         return(IVR_FAIL);
      }

      if(dx_addtone(lines[curLine].chdev, NULL, 0) != 0)
      {
         ivLog->Print("   ERROR: StartReorderToneDetection::dx_addtone failed\n");
         ivLog->Print("      '%s'\n", ATDV_ERRMSGP(lines[curLine].chdev));
         return(IVR_FAIL);
      }

      if(dx_enbtone(lines[curLine].chdev, 101, DM_TONEON | DM_TONEOFF ) != 0)
      {
         ivLog->Print("   ERROR: StartReorderToneDetection::dx_enbtone failed\n");
         ivLog->Print("      '%s'\n", ATDV_ERRMSGP(lines[curLine].chdev));
         return(IVR_FAIL);
      }
   }

   return(IVR_SUCCESS);
}

IV_STATUS BrdDialogic::StopReorderToneDetection(void)
{
   if((curLine != -1) && (dx_deltones(lines[curLine].chdev) != 0))
   {
      ivLog->Print("   ERROR: StopReorderToneDetection::dx_deltones failed\n");
      ivLog->Print("      '%s'\n", ATDV_ERRMSGP(lines[curLine].chdev));
      return(IVR_FAIL);
   }

   return(IVR_SUCCESS);
}

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::Wait                                   (public)

   Purpose:     Wait for increments of 1/1000 seconds.

   Notes:       - Actual waiting code copied from Wait.cpp
                - Dialogic does not support WaitForEvent, as the other
                  boards do, so cannot have TT interruptable wait.
                - We can't interrupt wait by touch tone using synchronous
                  model, but we can interrupt wait by system events
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::Wait(int iTimeout, bool interruptable)
{
   char           ttDig;
   HANDLE         hWaitTimer;
   HANDLE         waitObjs[2];
   LARGE_INTEGER  waitTimeout;
   IV_STATUS      status = IVR_SUCCESS;

	if (interruptable)
   {
	   if(lineOffHook)
	   {
	   	QueryDigit(ttDig);
	      if(ttDig != '\0')
	      {
	         ivLog->Print("      Interrupted!\n");
	         return(IVR_SUCCESS);
	      } else {
				ivLog->Print("      Warning: Dialogic driver does not support interruptable wait\n");
         }
      }
   }

   if(iTimeout == 0)
   {
      return(IVR_SUCCESS);
   }

	// waiting code copied from Wait.cpp
   // ---------------------------------

   hWaitTimer = CreateWaitableTimer (0, false, 0);

   // waitTimeout is measured in 100 nanosecond intervals (10^-7 = .0000001).
   // To convert to milliseconds times by 10,000 (10^-3 = 10^-7 x 10^4).

	// Sean change to prevent overflow and allow waits longer than 214748 ms (2^31 / 10000)
   // was: waitTimeout.QuadPart = -(LONGLONG)(iTimeout * 10000);
   waitTimeout.QuadPart = -(LONGLONG) iTimeout * 10000;
   SetWaitableTimer (hWaitTimer, &waitTimeout, 0, 0, 0, false);
   waitObjs[0] = pInfo->messages->GetMessageSignal();
   waitObjs[1] = hWaitTimer;
   while(status == IVR_SUCCESS)
   {
      switch(WaitForMultipleObjects(2, waitObjs, false, INFINITE))
      {
         case WAIT_FAILED:
         case WAIT_TIMEOUT:
         {
            status = IVR_FATAL;
            break;
         }

         case WAIT_OBJECT_0:
         {
            status = pInfo->ivBoard->ProcessSystemEvent();
            break;
         }

         case WAIT_OBJECT_0 + 1:
         {
            status = IVR_TIMEOUT;
            break;
         }
      }
   }

   CloseHandle(hWaitTimer);

   return(status == IVR_TIMEOUT ? IVR_SUCCESS : status);
}
/*BrdDialogic::Wait*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::CallStatus                             (public)

   Modified:    Dec.1, 2001 - TF: Created.

   Purpose:     Queries the status of the current telephony channel.
                Required to support hangup supervision asynchronously in
                an environments other than basic loop-start.

   Returns:     appropriate IV_STATUS value corresponding to current
                call state:
                  IVR_SUCCESS = call is currently connected
                  IVR_HANGUP  = call has been disconnected
                  IVR_FAIL    = call in an invalid state - should not have
                                been calling this function at this time.
                  IVR_FATAL   = error attempting to detect call status

   Notes:       - Handles only loop-start.
                - All other protocols deemed invalid and this function will
                  return IVR_NOTSUPPORTED.
                - Use appropriate child-class to deal with other protocols.

   Questions:   - May want to change IVR_FAIL results to IVR_SUCCESS.
                  If call is in an un-handled state, it is likely because the
                  channel is in the process of setting up a call.  Shouldn't
                  we leave the call alone until its state becomes either
                  connected or disconnected? - TF Dec.1/01
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::CallStatus(void)
{
   long        state;                 // Dialogic call state

   ivLog->Print ("      Call Status (%s)\n", pInfo->pszProtocol);

   if (curLine >= 0)
   {
		if (strcmp (pInfo->pszProtocol, "lps0") != 0)
		{
	   	// Loop-start protocol is valid.
   	   // All others will display this message, and return
			ivLog->Print ("      Protocol '%s' not supported.\n", pInfo->pszProtocol);
         return IVR_NOTSUPPORTED;
	   }
   	// end of protocol check

     	// Loop-start protocol:
      // May wish to use ATDX_LINEST() or ATDX_STATE() if ATDX_HOOKST proves unreliable. - TF Dec.1/01
		ivLog->Print ("            ATDX_HOOKST (%d)\n", lines[curLine].chdev);
      state = ATDX_HOOKST(lines[curLine].chdev);
      if (state == AT_FAILURE)
      {
			PrintErrorMsg("Failed to query call status", lines[curLine].chdev);
         return IVR_FATAL;
      }
      else if (state == DX_OFFHOOK)
      {
      	return IVR_SUCCESS;
		}
      else if (state == DX_ONHOOK)
      {
			ivLog->Print("            Call disconnected\n");
      	return(Hangup(true));
      }
      else
      {
			ivLog->Print("            Call in an invalid state\n");
      	return IVR_FAIL;
      }
   }
   else
   {
   	ivLog->Print ("            No line in use\n");
     	return IVR_FAIL;
   }
}
/*BrdDialogic::CallStatus*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::FlashHook                              (public)

   Purpose:     Used to perform a flash-hook during an active call.

   Modified:    TF Mar.26/03 - implemented proper soft coding of
                   flash-time.  Prior to this change, the flashTime
                   parameter was being ignored.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::FlashHook(long flashTime)
{
   unsigned short prevSetting = 0;
   int            newSetting = (int)(flashTime/10);	// convert to 1/100 seconds

	if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
   	return(IVR_FATAL);
   }

	ivLog->Print("      Flashing hook for %d ms\n", flashTime);

   if(flashTime == 0)
   {
      return(IVR_SUCCESS);
   }

	// Retrieving previous flash setting
   if(dx_getparm(lines[curLine].chdev, DXBD_FLASHTM, (void*)&prevSetting) == -1)
   {
		ivLog->Print("      ERROR: Failed to retrieve previous flash setting\n");
      return(IVR_FAIL);
   }

	// Flashing the hook
   if(dx_dial(lines[curLine].chdev, "&", 0, EV_SYNC) != 0)
   {
      return(IVR_FAIL);
   }

	// Reset to previous flash setting
   newSetting = (int)prevSetting;
   if(dx_setparm(lines[curLine].chdev, DXBD_FLASHTM, (void*)&newSetting) == -1)
   {
		ivLog->Print("      ERROR: Failed to reset previous flash setting\n");
      return(IVR_FAIL);
   }

   return(IVR_SUCCESS);
}
/*FlashHook*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::Hangup                                 (public)

   Purpose:     To disconnect the call currently in progress.
                Hang up the call, waiting for the Call Released event.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::Hangup(bool returnHangup, char * /*pszCause*/)
{
   IV_STATUS status;

   if(lineOffHook)
   {
   	if(strcmp(pInfo->pszProtocol, "lps0") != 0)
      {
		   // Only Loop Start protocol is valid.
   		// All others display this message, and default to loop start
	   	ivLog->Print("      Protocol '%s' not supported.\n", pInfo->pszProtocol);
		   return(IVR_NOTSUPPORTED);
   	}

      if(dx_sethook(lines[curLine].chdev, DX_ONHOOK, EV_SYNC) == -1)
   	{
	   	PrintErrorMsg("Hangup failed", lines[curLine].chdev);
         ReleaseLine();
         return(IVR_FAIL);
   	}

      lineOffHook = false;
      callEnd.Update();
   }

   if((status = ReleaseLine()) != IVR_SUCCESS)
   {
      return(status);
   }

   return(returnHangup ? IVR_HANGUP : IVR_SUCCESS);
}
/*BrdDialogic::Hangup*/

IV_STATUS BrdDialogic::Pickup(void)
{
   IV_STATUS status = IVR_SUCCESS;

   if(lineOffHook)
   {
      ivLog->Print("      ERROR: Port already off hook\n");
      return(IVR_FATAL);
   }

   if((curLine == -1) && ((status = RequestLine(pInfo->iPort)) != IVR_SUCCESS))
   {
      return(status);
   }

	// Now take the port off-hook - TF Feb.21/02
	ivLog->Print("      dx_sethook, lines[curLine].chdev = %d, OFFHOOK\n", lines[curLine].chdev);

	if(dx_sethook(lines[curLine].chdev, DX_OFFHOOK, EV_SYNC) != 0)
	{
		PrintErrorMsg("Could not assert off-hook state", lines[curLine].chdev);
		return(IVR_FAIL);
	}

   if(disableReorderToneDetection)
   {
      ivLog->Print("      Reorder tone detection disabled!\n");
   }

  	callStart.Update();
  	callEnd.Invalidate();
	lineOffHook = true;

   if(pInfo->portLicense == UNLICENSED)
   {
      ivLog->Print("      THIS PORT IS UNLICENSED, PLAYING EVALUATION MESSAGE\n");
      status = pInfo->ivBoard->AudioPlayEvalMessage();
   }
   return(status);
}

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::PlaceCall                              (public)

   Dial Chars:  0 - 9
                *, #, a, b, c, d
                , (comma)         : pause
                &                 : flash
                L                 : ?
                I                 : ?
                X                 : ?
                P                 : Chg dial mode to Pulse mode.
                T                 : Chg dial mode to DTMF  mode (default).
                M                 : Chg dial mode to MF    mode.

   Modified:    TF Oct.9/01 - Flash hook added when using existing line

                TF Mar.17/03 - Removed additional flash hooks in failure
                     handling.  Presumably these were to bring the original
                     caller back on the line.  This is not desired and is
                     not a part of our board level API.

                TF Mar.21/03 - Made flash hook duration soft-coded

                TF Apr.21/04 - Allow PlaceCall to proceed if port already
                   opened, on call mode of "initiate connection", in order
                   to support placing a call when port already opened by a
                   Switch step that bridged in another port; ie: allows
                   blind-transfers.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::PlaceCall(const char* digits, const char*, int numrings,
   callMode mode, int flashdur, bool connectOnCED, bool connectOnSIT, int*, int silenceTimeout,
   int, ISDNNumberPlan, ISDNNumberType, ISDNNumberPlan, ISDNNumberType)
{
   DX_CAP cap;
   int car;
   IV_STATUS status;

   if(strcmp(pInfo->pszProtocol, "lps0") != 0)
   {
		// Only Loop Start protocol is valid.
		// All others display this message, and return
		ivLog->Print("      Protocol '%s' not supported.\n", pInfo->pszProtocol);
      return(IVR_NOTSUPPORTED);
   }

   if(silenceTimeout > 0)
   {
		ivLog->Print("      Initial voice length detection is not supported on this hardware\n");
      return(IVR_NOTSUPPORTED);
   }

   if(mode == newLine)
   {
      if(lineOffHook)
      {
         ivLog->Print("      ERROR: Port already in use\n");
      	return(IVR_FATAL);
      }

      // WAS:  if((status = RequestLine(pInfo->iPort)) != IVR_SUCCESS)
	   if((curLine == -1) && ((status = RequestLine(pInfo->iPort)) != IVR_SUCCESS))
      {
         return(status);
      }
   }
   else
   {
    	if(!lineOffHook)
      {
         ivLog->Print("      ERROR: Port not off hook\n");
      	return(IVR_FATAL);
      }
   }

   dx_clrcap(&cap);

   // Notes on Basic vs. Enhanced Call Analysis
   // - Basic supports number of rings detection (ca_nbrdna) but Enhanced requires
   // a timeout (ca_noanswer)
   // - Basic detects connections, busy signals, no answer, no ringback, and SIT
   // - Enhanced detects all the signals that Basic does, plus it can detect CED
   // and no dial tone
   // - Both Basic and Enhanced can detect a voice connection (PVD)
   // - Initial voice length detection (ATDX_ANSRSIZ()) never yielded an answer
   // length other than 0, even with PVD disabled, on the D41/H

   // Wait 5 s for connection, 6 s per ring, and 10 s of buffer before timing out
   cap.ca_cnosig = (unsigned short) (1500 + 600*numrings);

   // Wait 6 s per ring before timing out because ring counting is disabled in
   // enhanced call analysis
   cap.ca_noanswer = (unsigned short) (600*numrings);

   // Enable SIT detection and PVD
   cap.ca_intflg = DX_PVDOPTNOCON;

   // Wait 3 seconds for no voice using PVD
   cap.ca_pamd_failtime = 300;

   if(StopReorderToneDetection() != IVR_SUCCESS)
   {
		ivLog->Print("      ERROR: Failed to stop reorder tone detection\n");
      if(mode == newLine)
      {
         ReleaseLine();
      }
      return(IVR_FAIL);
   }

   // Enhanced call analysis functionality
   if(dx_initcallp(lines[curLine].chdev) == -1)
   {
      PrintErrorMsg("Failed to initialize call progress", lines[curLine].chdev);
      if(mode == newLine)
      {
         ReleaseLine();
      }
      else if((status = FlashHook(500)) != IVR_SUCCESS)
      {
         return(status);
      }
      return(IVR_FAIL);
   }

   if(!lineOffHook && (dx_sethook(lines[curLine].chdev, DX_OFFHOOK, EV_SYNC) == -1))
   {
 		PrintErrorMsg("Failed to set port off hook", lines[curLine].chdev);
      if(mode == newLine)
      {
         ReleaseLine();
      }
      return(IVR_FAIL);
   }

   // Flash hook when using existing line
   if(mode != newLine)
   {
   	if((status = FlashHook(flashdur)) != IVR_SUCCESS)
      {
       	return(status);
      }

    	Sleep(500);
   }

   if((car = dx_dial(lines[curLine].chdev, digits, &cap, DX_CALLP | EV_SYNC)) == -1)
   {
      ivLog->Print("      ERROR: Failed to dial out\n");
   }

	// Reset reorder tone detection after dial complete
   if(disableReorderToneDetection)
   {
      ivLog->Print("      Reorder tone detection disabled!\n");
   }
   else if(StartReorderToneDetection() != IVR_SUCCESS)
	{
		ivLog->Print("      WARNING: Failed to restart reorder tone detection\n");
	}

   switch(car)
   {
      case CR_BUSY:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", "BUSY");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "BUSY");
         status = IVR_BUSY;
         break;
      }

      case CR_NOANS:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", "NOANSWER");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "MAX_RINGS");
         status = IVR_TIMEOUT;
         break;
      }

      case CR_NORB:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", "NOANSWER");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "TIMEOUT");
         status = IVR_TIMEOUT;
         break;
      }

      case CR_CNCT:
      {
         car = ATDX_CPTERM(lines[curLine].chdev);

         switch(car)
         {
            case CR_CEPT:
            {
               pInfo->variables->SetSystem("@CALLOUT_RESULT", connectOnSIT ? "CONNECT" : "FAILURE");
               pInfo->variables->SetSystem("@CALLOUT_REASON", "SIT");
               status = connectOnSIT ? IVR_SUCCESS : IVR_FAIL;
               break;
            }

            case CR_FAXTONE:
            {
               pInfo->variables->SetSystem("@CALLOUT_RESULT", connectOnCED ? "CONNECT" : "FAILURE");
               pInfo->variables->SetSystem("@CALLOUT_REASON", "CED");
               status = connectOnCED ? IVR_SUCCESS : IVR_FAIL;
               break;
            }

            default:
            {
               pInfo->variables->SetSystem("@CALLOUT_RESULT", "CONNECT");
               pInfo->variables->SetSystem("@CALLOUT_REASON",
                  ATDX_CONNTYPE(lines[curLine].chdev) == CON_PVD ? "VOICE" : "CONNECT");

               status = IVR_SUCCESS;
            }
         }

         break;
      }

      case CR_CEPT:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", connectOnSIT ? "CONNECT" : "FAILURE");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "SIT");
         status = connectOnSIT ? IVR_SUCCESS : IVR_FAIL;
         break;
      }

      case CR_STOPD:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", "FAILURE");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "UNKNOWN");
         ivLog->Print("      Call Analysis stopped\n");
         status = IVR_FAIL;
         break;
      }

      // Enhanced call analysis only
      case CR_NODIALTONE:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", "FAILURE");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "NO_DIALTONE");
         status = IVR_FAIL;
         break;
      }

      // Enhanced call analysis only
      case CR_FAXTONE:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", connectOnCED ? "CONNECT" : "FAILURE");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "CED");
         status = connectOnCED ? IVR_SUCCESS : IVR_FAIL;
         break;
      }

      case CR_ERROR:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", "FAILURE");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "UNKNOWN");
         ivLog->Print("      Call analysis error\n");
         ivLog->Print("         Error: %d\n", ATDX_CPERROR(lines[curLine].chdev));
         status = IVR_FAIL;
         break;
      }

      default:
      {
         pInfo->variables->SetSystem("@CALLOUT_RESULT", "FAILURE");
         pInfo->variables->SetSystem("@CALLOUT_REASON", "UNKNOWN");
         ivLog->Print("      Unknown dx_dial result %d\n", car);
         status = IVR_FAIL;
      }
   }

   if(status != IVR_SUCCESS)
   {
      if(mode == newLine)
      {
         dx_sethook(lines[curLine].chdev, DX_ONHOOK, EV_SYNC);
         ReleaseLine();
      }
      else
      {
			if((status = FlashHook(flashdur)) != IVR_SUCCESS)
         {
         	return(status);
         }

       	Sleep (500);
      }
   }
   else if(mode == newLine)
   {
      lineOffHook = true;
      callStart.Update();
      callEnd.Invalidate();
   }

   return(status);
}
/*BrdDialogic::PlaceCall*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::ReleaseLine                            (public)

   Notes:       - Don't raise an error if a line is currently not in use.
                - Only concerned with voice devices - TF Mar.12/02
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::ReleaseLine(void)
{
   if(curLine == -1)
   {
      return(IVR_SUCCESS);
   }

   ivLog->Print ("      Attempting ag_unlisten, lines[curLine].chdev = 0x%x\n", lines[curLine].chdev);
   if(lines[curLine].chdev != -1)
   {
	   ivLog->Print ("            ag_unlisten");
      if(ag_unlisten(lines[curLine].chdev) >= 0)
      {
		   ivLog->Print (" - success\n");
      }
      else
      {
		   ivLog->Print (" - failure\n");
      }
   }

	// Leave port on-hook; ignore any errors
	dx_sethook(lines[curLine].chdev, DX_ONHOOK, EV_SYNC);

   ivLog->Print ("            dx_close\n");
	dx_close(lines[curLine].chdev);

	lines[curLine].inUse = false;
	lines[curLine].chdev = -1;
   curLine = -1;

   ivLog->Print ("      ReleaseLine done.\n");

   return (IVR_SUCCESS);
}
/*BrdDialogic::ReleaseLine*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::RequestLine                            (public)

   Notes:       - Since the line data structure is only available to this
                  application, there isn't much point maintaining an In Use
                  flag.  Left in since it may become useful if the structure
                  is moved to shared memory. JR

                - Dialogic board numbers are logical not physical.  A dual
                  or quad T1 board is considered 2 or 4 logical boards.
                  Timeslots are in the range of 1..24 for T-1, and 1..30
                  for E-1.  Also, analog boards are considered to be a
                  collection of 2 or 4 port logical devices. - TF Oct.26/01

                - The lines[] array reflects all the information that is
                  required about the Dialogic boards. - TF Oct.26/01
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::RequestLine(int lineNo)
{
   char	szChannel[12];

   if(myState != good)
   {
      ivLog->Print("      RequestLine: Board in an invalid state, cannot continue.\n");
      ivLog->Print("      Is the Dialogic service running?\n");
      return(IVR_FAIL);
   }

   if(curLine != -1)
   {
      ivLog->Print("      ERROR: A port is already in use\n");
      return(IVR_FATAL);
   }

   if(lineNo < 1)
   {
      ivLog->Print("      ERROR: Port %d cannot be less than 1\n", lineNo);
      return(IVR_FATAL);
   }

   if(lineNo - 1 > lastLine)
   {
      ivLog->Print("      ERROR: Port %d specified exceeds maximum ports (%d)\n", lineNo, lastLine + 1);
      return(IVR_FATAL);
   }

   if(lines[lineNo - 1].inUse)
   {
      ivLog->Print("      ERROR: Port %d is already in use\n", lineNo);
      return(IVR_FATAL);
   }

	// Open telephone interface (timeslot) first
   // -----------------------------------------
   if(strcmp(pInfo->pszProtocol, "lps0") != 0)
   {
   	// Loop Start protocol is valid.
      // All others will display this message, and return
		ivLog->Print("      Protocol '%s' not supported\n", pInfo->pszProtocol);
      return(IVR_NOTSUPPORTED);
   }
   // end of protocol check

	// Adjust human one-based counter to computer zero-based counting
   curLine = lineNo - 1;

	// Open voice interface (channel)
   // -----------------------------------
	sprintf(szChannel, "%sC%d", lines[curLine].boardName, lines[curLine].channel);
	if((lines[curLine].chdev = dx_open(szChannel, 0)) != -1)
   {
		lines[curLine].inUse = true;

		// Ensure port is on-hook - TF Aug.25/04
		if(dx_sethook(lines[curLine].chdev, DX_ONHOOK, EV_SYNC) == -1)
		{
			PrintErrorMsg("Set-hook failed", lines[curLine].chdev);
			ReleaseLine();
			return(IVR_FATAL);
		}

      // Ensure telephony channel and voice channel are connected - TF Jan.3/02
      if(SwitchReset() != IVR_SUCCESS)
      {
      	ivLog->Print("      Warning: port's switch status unchanged.\n");
      }

      if(!disableReorderToneDetection)
      {
	      return(StartReorderToneDetection());
      }

      return(IVR_SUCCESS);
   }
   else
	{
		PrintErrorMsg("Unable to open channel", lines[curLine].chdev);
		curLine = -1;
		return(IVR_FAIL);
   }
}
/*BrdDialogic::RequestLine*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::Switch                                 (public)

   Purpose:     Switch two SCbus timeslots together

   Modified:    TF Dec.31/01 - Created.
                TF Apr.19/04 - Allow switch connect when port not off-hook

   Notes:       - Parameters 'curPort' and 'newPort' are one-based port
                  numbers.  Must be translated to zero-based to reference
                  their position in the lines array.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::Switch(int newPort, bool bListen)
{
	SC_TSINFO tsinfo;       // Dialogic SCbus timeslot info structure
   int       result;       // result returned from Dialogic function calls
   char      chname[12];   // Dialogic channel name
   long      curxmit;      // current port's SCbus transmit timeslot
   int       newLine = newPort - 1;       // new port's index in the lines array
   long      newxmit;      // new port's SCbus transmit timeslot
   IV_STATUS status;

	// Need to open the port, if we haven't done so already
   if((curLine == -1) && ((status = RequestLine(pInfo->iPort)) != IVR_SUCCESS))
   {
      return(status);
   }

   if(bListen)
   {
		ivLog->Print ("      Switch current port (%d) to port %d (listen only)\n", pInfo->iPort, newPort);
		ivLog->Print ("      ERROR: Listen-only switch connections not supported on Dialogic hardware\n");
      return(IVR_NOTSUPPORTED);
   }
   else
   {
		ivLog->Print ("      Switch current port (%d) to port %d\n", pInfo->iPort, newPort);
   }

   // Gather information about the ports:
   // -----------------------------------
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &curxmit;
   result = ag_getxmitslot(lines[curLine].chdev, &tsinfo);
   if (result == -1)
   {
		PrintErrorMsg("Port not available on SCbus", lines[curLine].chdev);
		return(IVR_FATAL);
   }

   if(lines[newLine].chdev <= 0)
   {
   	// need to open the new port first
      sprintf(chname, "%sC%d", lines[newLine].boardName, lines[newLine].channel);
		ivLog->Print("      dx_open, '%s'\n", chname);
		lines[newLine].chdev = dx_open(chname, 0);
      if (lines[newLine].chdev <= 0)
      {
      	ivLog->Print("      Could not open device '%s'.\n", chname);
			ivLog->Print("      ERROR: Failed to initialize port number %d\n", newPort);
			return IVR_FATAL;
      }
   }
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &newxmit;
   result = ag_getxmitslot(lines[newLine].chdev, &tsinfo);
   if (result == -1)
   {
		PrintErrorMsg("Port not available on SCbus", lines[newLine].chdev);
		return(IVR_FATAL);
   }

   // Make iPort listen to port:
   // -------------------------------
   ivLog->Print("      %d, %d -> %d, %d\n", pInfo->iPort, curxmit, newPort, newxmit);
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &newxmit;
   ivLog->Print("      ag_listen, 0x%x, %d\n", lines[curLine].chdev, *tsinfo.sc_tsarrayp);
	result = ag_listen(lines[curLine].chdev, &tsinfo);
   ivLog->Print("            result = %d \n", result);
	if (result < 0)
	{
		PrintErrorMsg("Unable to complete switch request", lines[curLine].chdev);
		return IVR_FAIL;
	}

   // Make port listen to iPort:
   // -------------------------------
   ivLog->Print("      %d, %d <- %d, %d\n", pInfo->iPort, curxmit, newPort, newxmit);
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &curxmit;
   ivLog->Print("      ag_listen, 0x%x, %d\n", lines[newLine].chdev, *tsinfo.sc_tsarrayp);
	result = ag_listen(lines[newLine].chdev, &tsinfo);
	if (result < 0)
	{
		PrintErrorMsg("Unable to complete switch request", lines[newLine].chdev);
		return IVR_FAIL;
	}

   // Done.
   // -----
	ivLog->Print("      Connected.\n");
   return(IVR_SUCCESS);
}
/*BrdDialogic::Switch*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::UnSwitch                               (public)

   Purpose:     Implements "Switch Disconnect" step.  Calls SwitchReset()
                to reset a port so that previous switch connections are
                broken and the voice and network resources are connected
                together.

   Modified:    TF Oct.28/03 - Created.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::UnSwitch(void)
{
	ivLog->Print("      Disconnecting switch state(s)\n");
	return(SwitchReset());
}
/*BrdDialogic::UnSwitch*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::SwitchReset                           (private)

   Purpose:     Reset a port so that previous switch connections are
                broken and the voice and network resources are connected
                together.

   Modified:    TF Jan.3/02 - Created.

   Notes:       - None.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::SwitchReset(void)
{
	SC_TSINFO tsinfo;       // Dialogic SCbus timeslot info structure
   int       result;       // result returned from Dialogic function calls
   long      ag_xmit;      // current port's network SCbus transmit timeslot
   long      dx_xmit;      // current port's voice SCbus transmit timeslot

   // Gather information about the current port:
   // ------------------------------------------
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &ag_xmit;
   result = ag_getxmitslot(lines[curLine].chdev, &tsinfo);
   if (result == -1)
   {
      ivLog->Print("      Port not available on SCbus\n");
      return(IVR_SUCCESS);
   }
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &dx_xmit;
   result = dx_getxmitslot(lines[curLine].chdev, &tsinfo);
   if (result == -1)
   {
		PrintErrorMsg("Port not available on SCbus", lines[curLine].chdev);
		return(IVR_FATAL);
   }

   // Make network channel listen to voice channel:
   // ---------------------------------------------
   // ivLog->Print("      %d: %d -> %d \n", curLine, ag_xmit, dx_xmit);
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &dx_xmit;
	result = ag_listen(lines[curLine].chdev, &tsinfo);
	if (result < 0)
	{
		PrintErrorMsg("Unable to complete switch request", lines[curLine].chdev);
		return IVR_FAIL;
	}

   // Make voice channel listen to network channel:
   // ---------------------------------------------
   // ivLog->Print("      %d: %d <- %d \n", curLine, ag_xmit, dx_xmit);
   tsinfo.sc_numts = 1;
   tsinfo.sc_tsarrayp = &ag_xmit;
	result = dx_listen(lines[curLine].chdev, &tsinfo);
	if (result < 0)
	{
		PrintErrorMsg("Unable to complete switch request", lines[curLine].chdev);
		return IVR_FAIL;
	}

   // Done.
   // -----
   return(IVR_SUCCESS);
}
/*BrdDialogic::SwitchReset*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::WaitForCall                            (public)

   Purpose:     Waits for the phone to ring.

   Notes:       - May want to leave the phone onhook after rings detected.
                - Need to figure out how to collect caller ID.
                * Dialogic has a bug that if the application crashes with
                  the phone offhook, it stays offhook.  Need to initialize
                  the phone to the onhook state. - JR
                - May want to adopt code similar to NMS so that other events,
                	such as a shutdown request, can be processed while waiting for
                  a call.
                - Added error checking on the dx_sethook call.  Will return
                  IVR_FAIL if dx_sethook does not succeed. - TF Oct.4/01
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::WaitForCall(long timeout, int numrings, bool pickup, char* calledDigits,
   char* callingDigits, char* handsetDisplay, int)
{
   IV_STATUS status;
   int result;
   unsigned short parmval;

	if(lineOffHook)
   {
      ivLog->Print("      ERROR: Port already in use\n");
   	return(IVR_FATAL);
   }

   if(calledDigits != NULL)
   {
      calledDigits[0] = '\0';
   }

   if(handsetDisplay != NULL)
   {
      handsetDisplay[0] = '\0';
   }

   if((status = RequestLine(pInfo->iPort)) != IVR_SUCCESS)
   {
      return(status);
   }

   if(numrings == 0)
   {
      numrings = 1;
   }

	// Original method: SYNCHRONOUS. At least it works! - TF Dec.1/01
   // -----------------------------
	// WAS:
	//  // Wait for the phone to ring
	//   if (dx_wtring (lines[curLine].chdev, numrings, DX_OFFHOOK, timeout) == -1) {
	// Wait for the phone to ring, but leave on-hook until we get Caller ID - TF Feb.21/02
   if(callingDigits == NULL)
   {
      result = dx_wtring(lines[curLine].chdev, numrings, DX_ONHOOK, timeout);
   }
   else
   {
      // Enable Caller ID
      parmval = DX_CALLIDENABLE;
      if(dx_setparm(lines[curLine].chdev, DXCH_CALLID, (void*) &parmval) == -1)
      {
         ivLog->Print("      ERROR: Failed to set caller ID parameter\n");
         ReleaseLine();
         return(IVR_FAIL);
      }

      result = dx_wtcallid(lines[curLine].chdev, numrings, timeout, (unsigned char*) callingDigits);
   }

   if(result == -1)
   {
      switch(ATDV_LASTERR(lines[curLine].chdev))
      {
         case EDX_TIMEOUT:
   		{
            ReleaseLine();
            return(IVR_TIMEOUT);
         }

         case EDX_CLIDBLK:
         {
            strcpy(callingDigits, "Caller ID Blocked");
            break;
         }

         case EDX_CLIDINFO:
         {
            break;
         }

         case EDX_CLIDOOA:
         {
            strcpy(callingDigits, "Out of Area");
            break;
         }

         default:
         {
   	   	ivLog->Print("      dx_wtring failed for device 0x%x\n", lines[curLine].chdev);
            return(IVR_FAIL);
         }
      }
   }

	ivLog->Print ("         Incoming call...\n");
	return(pickup ? Pickup() : IVR_SUCCESS);
/*
	// Attempted method: ASYNCHRONOUS. Does not work! - TF Dec.1/01
   // ------------------------------
	IV_STATUS status;
   DialogicBoardEvent boardEvent;

	// Wait for the phone to ring
   ivLog->Print ("      dx_setevtmsk, lines[curLine].chdev = %d\n", lines[curLine].chdev);
   if (dx_setevtmsk(lines[curLine].chdev,
		DM_LCOFF|DM_LCON|DM_RINGS|DM_WINK|DM_SILON|DM_SILOFF) == -1)
	{
      PrintErrorMsg("Failed to set environment mask", lines[curLine].chdev);
      return(IVR_FAIL);
   }
	// Wait for board to signal an in-coming ring
   for (;;)
   {
		ivLog->Print ("      WaitForEvent.\n");
      status = WaitForEvent(&boardEvent);
   	if (status == IVR_SUCCESS)
      {
	      switch(boardEvent.DlgEvent.evtType)
	      {
				//  case // ignored events:
	         //  	break;
         	case TDX_CST:
            {
               // TO DO:  - TF Nov.29/01
               // - How to get DX_CST information
               // - Still need to count the rings
					ivLog->Print("      Call State Transition...");
					return(IVR_SUCCESS);
            }
	         default:                      // unhandled event
	         {
	         	ivLog->Print("WaitForCall: Unexpected Dialogic event type 0x%x\n",
	            	boardEvent.DlgEvent.evtType);
	            // Eventually fail, but for now want to see all events that come through - TF Nov.28/01
	            // return(IVR_FAIL);
	         }
	      } // end switch
      }
      else
      {
      	return (status);
		} // end if
   } // end for
*/
}
/*BrdDialogic::WaitForCall*/

// ****************************************************************************
// *********************** TOUCH TONE FUNCTIONS *******************************
// ****************************************************************************

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::FlushDigits                            (public)

   Purpose:     Removes any Touch-Tones that the user has entered.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::FlushDigits(void)
{
   if(curLine != -1)
   {
      dx_clrdigbuf(lines[curLine].chdev);
   }

   return(IVR_SUCCESS);
}
/*BrdDialogic::FlushDigits*/

/* -------------------------------------------------------------------------
   Name:        TermDigit

   Purpose:     Checks to see if a digit is a termination digit.

                Dialogic adds the termination digit to the list of pressed
                keys.  We don't want the termination digit included in the
                keys pressed so we need to strip it off.

   Notes:       - I'm making this a local function (static) function and not
                  part of the class since it is only needed by Dialogic.
   ------------------------------------------------------------------------- */
bool BrdDialogic::TermDigit(char dig, const char* ttDone)
{
   if (!dig)
      return true;

   for (int i = 0; ttDone[i]; ++i)
      if (dig == ttDone[i])
         return true;
   return false;
}
/*TermDigit*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::GetDigits                              (public)

   Purpose:     Attempts to collect the specified number of DTMFs
                (Touch-Tones).  Collection will end when the maximum number
                of TT are entered, timeout occurs, or the user presses one
                of the termination TT.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::GetDigits(
   unsigned count,         // Number of digits to collect.
   char*& digits,          // Character array to store digits collected.
   int newTime,            // Timeout between key presses.
   const char* ttOkay,		      // Digits that are valid.
   const char* ttDone)           // Digits that terminate TT collection.
{
   unsigned        index;
   unsigned short  iTimeout;
   int             term;
   DV_TPT          tpt[6];
   DV_DIGIT        digit;
   unsigned short  termMask;
   IV_STATUS       cState;	// To query status of channel before function begins
   string          digitString;

   digits = new char[1];
   digits[0] = '\0';

   if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      return(IVR_FATAL);
   }

   // Dialogic Synchronous hangup supervision - TF Dec.4/01
   cState = CallStatus();
   if (cState != IVR_SUCCESS)
   	return (cState);

   BuildDTMF(ttDone, &termMask);

   // Adjust the timeout value to Dialogic resolution
   iTimeout = (unsigned short)(newTime / 100);
   if(iTimeout < 1)
   {
      iTimeout = 1;
   }

   // Clear the DV_TPT structure.
   // Note: must setup links before using DV_TPT structure, even to clear it - TF Dec.09/02
   for(index = 0; index < 5; index++)
   {
      tpt[index].tp_type = IO_CONT;
   }
   tpt[index++].tp_type = IO_EOT;
   dx_clrtpt(tpt, index);

   // Set-up the DV_TPT structure
   index = 0;

   if(count != INFINITE)
   {
      tpt[index].tp_type   = IO_CONT;
      tpt[index].tp_termno = DX_MAXDTMF;             // Max number of digits
      tpt[index].tp_length = (unsigned short)count;  // terminate on 'count' digits
      tpt[index].tp_flags  = TF_MAXDTMF;             // terminate if already in buf.
      index++;
   }

   if(!disableReorderToneDetection)
   {
      tpt[index].tp_type   = IO_CONT;
      tpt[index].tp_termno = DX_TONE;          // Global Tone
      tpt[index].tp_length = 101;              // Reorder tone
      tpt[index].tp_flags  = TF_TONE;          // Terminate if tone occurs
      tpt[index].tp_data = DX_TONEON;
      index++;

      tpt[index].tp_type   = IO_CONT;
      tpt[index].tp_termno = DX_TONE;          // Global Tone
      tpt[index].tp_length = 101;              // Reorder tone
      tpt[index].tp_flags  = TF_TONE;          // Terminate if tone occurs
      tpt[index].tp_data = DX_TONEOFF;
      index++;
   }

   tpt[index].tp_type   = IO_CONT;
   tpt[index].tp_termno = DX_LCOFF;           // LC off termination
   tpt[index].tp_length = 3;                  // Use 30ms (10ms resolution timer)
   tpt[index].tp_flags  = TF_LCOFF|TF_10MS;   // level triggered, clear history,
   index++;

   tpt[index].tp_type   = IO_CONT;
   tpt[index].tp_termno = DX_IDDTIME;         // Maximum delay between digits
   tpt[index].tp_length = iTimeout;           // 100ms resolution timer
   tpt[index].tp_flags  = TF_MAXTIME;         // Edge-triggered
   tpt[index].tp_type   = (unsigned short) ((termMask != 0) ? IO_CONT : IO_EOT);
   index++;

   tpt[index].tp_type   = IO_EOT;
   tpt[index].tp_termno = DX_DIGMASK;         // Specific digit received
   tpt[index].tp_length = termMask;           // (100 ms resolution timer)
   tpt[index].tp_flags  = TF_DIGMASK;         // Level- and Edge-triggered

   if (dx_getdig (lines[curLine].chdev, tpt, &digit, EV_SYNC) == -1)
   {
      PrintErrorMsg("Failed to request digits", lines[curLine].chdev);
      return IVR_FAIL;
   }

   if ((term = ATDX_TERMMSK (lines[curLine].chdev)) == AT_FAILURE)
   {
      return IVR_FATAL;
   }

   if((term & TM_LCOFF) || (term & TM_TONE))
   {
    	return(Hangup(true));
   }

   // Check for inter-digit timeout
   if (term & TM_IDDTIME && !digit.dg_value[0])
      return IVR_TIMEOUT;

   for(index = 0; ((index < count) || (count == INFINITE)) && !TermDigit(digit.dg_value[index], ttDone); index++)
   {
      digitString += digit.dg_value[index];
   }

	// Include termination characters if acceptable
   if(((index < count) || (count == INFINITE)) && TermDigit(digit.dg_value[index], ttOkay))
   {
		digitString += digit.dg_value[index];
   }

   delete[] digits;
   digits = new char[digitString.length() + 1];
   strcpy(digits, digitString.c_str());
   strupr(digits);

   return(IVR_SUCCESS);
}
/*BrdDialogic::GetDigits*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::GenerateDigits                         (public)

   Purpose:     Generates Touch Tones.

   Modified:    TF Feb.20/02 - Created.

   Notes:       - Replaces the sequence "<esc>" with a single escape
                  character (ASCII 27), in order to support call transfer
                  on D/42-xx PBX integration cards.

                - All other digits passed directly to dx_dial()

                - Will want to remove the "<esc>" handling in the future.
                  The plan is to create a D/42-xx transfer step that has
                  the translation of the <esc> sequences to use for each
                  type of KSU/PBX.
   ----------------------------------------------------------------------- */
IV_STATUS BrdDialogic::GenerateDigits(const char* ttStr)
{
	char       *dialstr;
	int        numchars;
	int        pos = 0;
	IV_STATUS  cState;              // Capture call status for h/u supervision

   if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      return(IVR_FATAL);
   }

	// Dialogic Synchronous hangup supervision
	cState = CallStatus();
	if (CallStatus() != IVR_SUCCESS)
	return (cState);

	// Parse string to replace special characters
	numchars = strlen(ttStr) + 1;
	dialstr = new char[numchars];
	memset (dialstr, NULL, numchars);
	for (int count=0; count<numchars; count++)
	{
		switch (ttStr[count])
		{
			case '<':
				if (!strnicmp(ttStr+count, "<esc>", 5))
				{
					dialstr[pos] = 27;  // ASCII escape character
					pos++;
					count = count + 4;
					break;
				}
				// else, fall through and let the char be added to the dial string
			default:
				dialstr[pos] = ttStr[count];
				pos++;
		} // end switch
	} // end for
   /*
	ivLog->Print ("            Original string: '%s'\n", ttStr);
	ivLog->Print ("            Translated string: '%s'\n", dialstr);
	ivLog->Print ("            hex: 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
    				  dialstr[0],
                 dialstr[1],
                 dialstr[2],
                 dialstr[3],
                 dialstr[4],
                 dialstr[5],
					  dialstr[6],
					  dialstr[7],
                 dialstr[8],
                 dialstr[9] );
   */
	if (dx_dial(lines[curLine].chdev, dialstr, NULL, EV_SYNC) == -1)
	{
		PrintErrorMsg("Dial error\n", lines[curLine].chdev);
		delete dialstr;
		return (IVR_FAIL);
	}
	delete dialstr;
	return(IVR_SUCCESS);
}
/*BrdDialogic::GenerateDigits*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::QueryDigit                             (public)

   Purpose:     Checks the ADI Touch-Tone queue to see if any tones are
                pending, and puts the first one into the 'ttStr'
                parameter.  The tones ARE NOT removed from the queue.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::QueryDigit(char& digit)
{
   if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      return(IVR_FATAL);
   }

   if((digit = (char) ATDX_BUFDIGS(lines[curLine].chdev)) == AT_FAILURE)
   {
		PrintErrorMsg("      ERROR: ATDX_BUFDIGS failed\n", lines[curLine].chdev);
   	return(IVR_FAIL);
   }

 	return(IVR_SUCCESS);
}
/*BrdDialogic::QueryDigit*/

/* -------------------------------------------------------------------------
   Function:    BrdDialogic::BuildDTMF                             (private)

   Purpose:     Builds a list of TT keys that will be collected and TT keys
                that will terminate TT collection.  These two list are
                ** mutually exclusive **.

                The termination TT key list is build first to allow
                programming to include all keys except those that terminate
                input.
   ------------------------------------------------------------------------- */
void BrdDialogic::BuildDTMF(const char* sTerm, unsigned short* dTerm)
{
	int i;

   for(i = 0, *dTerm = 0; sTerm[i]; ++i)
   {
      switch (sTerm[i])
      {
         case '0': *dTerm |= DM_0;     break;
         case '1': *dTerm |= DM_1;     break;
         case '2': *dTerm |= DM_2;     break;
         case '3': *dTerm |= DM_3;     break;
         case '4': *dTerm |= DM_4;     break;
         case '5': *dTerm |= DM_5;     break;
         case '6': *dTerm |= DM_6;     break;
         case '7': *dTerm |= DM_7;     break;
         case '8': *dTerm |= DM_8;     break;
         case '9': *dTerm |= DM_9;     break;
         case 'A': *dTerm |= DM_A;     break;
         case 'B': *dTerm |= DM_B;     break;
         case 'C': *dTerm |= DM_C;     break;
         case 'D': *dTerm |= DM_D;     break;
         case '*': *dTerm |= DM_S;     break;
         case '#': *dTerm |= DM_P;     break;
      }
   }
}
/*BrdDialogic::BuildDTMF*/

// ****************************************************************************
// *************************** AUDIO FUNCTIONS ********************************
// ****************************************************************************

// Note that currently this function always returns the same value but is used
// for consistancy and future expansion
DX_XPB BrdDialogic::GetEncoding(SOUND_FORMAT format)
{
   DX_XPB xpb = { FILE_FORMAT_VOX, DATA_FORMAT_DIALOGIC_ADPCM, DRT_8KHZ, 4 };

	switch(format)
   {
   	case TF_VOX:
      {
         return(xpb);
      }

      case TF_WAV_8KHZ_8BIT_MULAW:
      {
         xpb.wDataFormat = DATA_FORMAT_MULAW;
         xpb.wBitsPerSample = 8;
         return(xpb);
      }

		default:	return(xpb);
   }
}

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AudioBeep                              (public)

   Purpose:     Generate a beep tone.

   Notes:       - See "dx_blddt", "dx_addtone", "dx_enbtone".
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::AudioBeep(bool interruptable)
{
   int       term;
   DV_TPT    tpt[2];
   DX_XPB    xpb = GetEncoding(TF_VOX);
   DX_IOTT   iott;
   IV_STATUS cState;   // captures current call status

   char         ttDig;

   if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      return(IVR_FATAL);
   }

   // Dialogic Synchronous hangup supervision - TF Dec.4/01
   cState = CallStatus();
   if (cState != IVR_SUCCESS)
   	return (cState);

	QueryDigit(ttDig);
   if(interruptable && (ttDig != '\0'))
   {
      ivLog->Print("      Interrupted!\n");
      return(IVR_SUCCESS);
   }

   HINSTANCE hLibrary = LoadLibrary("brd_dialogic.dll");
   HRSRC audioBufferHandle = FindResource(hLibrary, "BEEP", "VOX");
   char* audioBuffer = (char*) LoadResource(hLibrary, audioBufferHandle);

   if(audioBuffer == NULL)
   {
      ivLog->Print("      Could not open audio buffer\n");
      return(IVR_FAIL);
   }

   ivLog->Print("      Playing audio buffer...\n");

   // Must setup links before using DV_TPT structure, even to clear it - TF Dec.9/02
   tpt[0].tp_type = IO_CONT;
   tpt[1].tp_type = IO_EOT;
   dx_clrtpt (tpt, 2);

   // The "tpt" array contains the reasons to stop playback.
   // If playback is interruptable, both array entries are used.
   // If playback is uninterruptable, only the first array entry is used,
   // the second entry is ignored.
   if (interruptable) {
      tpt[0].tp_type = IO_CONT;
      tpt[1].tp_type = IO_EOT;
   } else
      tpt[0].tp_type = IO_EOT;

   // First entry in array: If the caller hangs up, stop playback.
   tpt[0].tp_termno = DX_LCOFF;         // LC off termination
   tpt[0].tp_length = 3;                // Use 30 ms (10 ms resolution timer)
   tpt[0].tp_flags  = TF_LCOFF|TF_10MS; // level triggered, clear history,

   // Second entry in array: If the caller presses a key, stop playback.
   if (interruptable) {
                                           // 10 ms resolution
      tpt[1].tp_termno = DX_MAXDTMF;       // Max number of digits
      tpt[1].tp_length = 1;                // terminate on first digit
      tpt[1].tp_flags  = TF_MAXDTMF;       // terminate if already in buf.
   }

   // Populate IOTT structure
   iott.io_fhandle = NULL;
   iott.io_bufp    = audioBuffer;
   iott.io_offset  = 0;
   iott.io_length  = SizeofResource(hLibrary, audioBufferHandle);
   iott.io_type    = IO_MEM | IO_EOT;

   // Play the beep buffer
   if(dx_playiottdata(lines[curLine].chdev, &iott, tpt, &xpb, EV_SYNC) == -1)
   {
		PrintErrorMsg("Play buffer error\n", lines[curLine].chdev);
      return(IVR_FAIL);
   }

   if((term = ATDX_TERMMSK(lines[curLine].chdev)) == AT_FAILURE)
   {
      return(IVR_FATAL);
   }

   if(term & TM_LCOFF)
   {
     	return(Hangup(true));
   }

	return(IVR_SUCCESS);
}
/*BrdDialogic::AudioBeep*/
// -------------------------------------------------------------------------


/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AudioRecord                            (public)

   Purpose:     Record a voice message.

   Modified:    08-Aug-1998 Jamie Royer: Converted to support Dialogic.

   Notes:       - Currently only supports Dialogic VOX files.
                - Need to write code to check file extension (when present)
                  matches file type.

   Questions:   - Do we want to make Beep optional?
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::AudioRecord(
      const char* filename,   	// Filename to record into.
      bool interruptable,  // Indicates if key press stops recording.
      SOUND_FORMAT format, // Type of audio file.
      int,						// volLevel,
      int maxRecTime,      // Maximum number of milliseconds to record.
      int maxSilence,
      bool disableBeep,
      bool automaticGainControl)
{
   int     i;
   char    ttDig;
   int     term;
   DV_TPT  tpt[6];
   DX_XPB  xpb = (format == TF_BY_EXTENSION) ? GetEncoding(GetExtensionFormat(filename)) : GetEncoding(format);
   IV_STATUS cState;			// captures call status for synchronous hangup supervision
   ESCTime recordStartTime;
   ESCTime recordEndTime;
   unsigned short mode = (unsigned short) (EV_SYNC | (disableBeep ? 0 : PM_TONE) | (automaticGainControl ? MD_GAIN : MD_NOGAIN));

   if((curLine == -1) && ((cState = RequestLine(pInfo->iPort)) != IVR_SUCCESS))
   {
      return(cState);
   }

   // Dialogic synchronous hangup supervision - TF Dec.4/01
   cState = CallStatus();
   if (cState != IVR_SUCCESS)
   	return (cState);

	QueryDigit(ttDig);
   if(interruptable && (ttDig != '\0'))
   {
      ivLog->Print("      Interrupted!\n");
      return(IVR_SUCCESS);
   }

   // Clear the DV_TPT structure
   // Note: must setup links before using DV_TPT structure, even to clear it - TF Dec.9/02
   for(i = 0; i < 5; i++)
   {
      tpt[i].tp_type = IO_CONT;
   }
   tpt[i++].tp_type = IO_EOT;
   dx_clrtpt(tpt, i);

   // Set-up the DV_TPT structure
   i = 0;
   if(maxRecTime != FOREVER)
   {
      tpt[i].tp_type   = IO_CONT;
      tpt[i].tp_termno = DX_MAXTIME;       // Maximum length of recording.
      tpt[i].tp_length = (unsigned short)(maxRecTime / 100);
                                           // Number of seconds to record.
      tpt[i].tp_flags  = TF_MAXTIME;       // Terminate after max rec time.
      ++i;
   }

   if(maxSilence != FOREVER)
   {
      tpt[i].tp_type   = IO_CONT;
      tpt[i].tp_termno = DX_MAXSIL;        // Maximum time without voice.
      tpt[i].tp_length = (unsigned short) ((maxSilence > 0) ?  maxSilence / 100 : 30); // Default to terminate after 3 sec of silence.
      tpt[i].tp_flags  = TF_MAXSIL;        // Terminate if no voice.
      ++i;
   }

   if(!disableReorderToneDetection)
   {
      tpt[i].tp_type   = IO_CONT;
      tpt[i].tp_termno = DX_TONE;          // Global Tone
      tpt[i].tp_length = 101;              // Reorder tone
      tpt[i].tp_flags  = TF_TONE;          // Terminate if tone occurs
      tpt[i].tp_data = DX_TONEON;
      i++;

      tpt[i].tp_type   = IO_CONT;
      tpt[i].tp_termno = DX_TONE;          // Global Tone
      tpt[i].tp_length = 101;              // Reorder tone
      tpt[i].tp_flags  = TF_TONE;          // Terminate if tone occurs
      tpt[i].tp_data = DX_TONEOFF;
      ++i;
   }

   tpt[i].tp_type   = (unsigned short) (interruptable ? IO_CONT : IO_EOT);
   tpt[i].tp_termno = DX_LCOFF;         // LC off termination
   tpt[i].tp_length = 3;                // Use 30 ms (10 ms resolution timer)
   tpt[i].tp_flags  = TF_LCOFF|TF_10MS; // level triggered, clear history,
   ++i;
                                        // 10 ms resolution
   tpt[i].tp_type   = IO_EOT;
   tpt[i].tp_termno = DX_MAXDTMF;       // Max number of digits
   tpt[i].tp_length = 1;                // terminate on first digit
   tpt[i].tp_flags  = TF_MAXDTMF;       // terminate if already in buf.

   recordStartTime.Update();

   if(dx_recvox(lines[curLine].chdev, filename, tpt, &xpb, mode) == -1)
   {
		PrintErrorMsg("Record failed\n", lines[curLine].chdev);
      return(IVR_FAIL);
   }

   recordEndTime.Update();

   if((term = ATDX_TERMMSK(lines[curLine].chdev)) == AT_FAILURE)
   {
		PrintErrorMsg("Failed to obtain termination\n", lines[curLine].chdev);
      return(IVR_FAIL);
   }

   if((term & TM_LCOFF) || (term & TM_TONE))
   {
      return(Hangup(true));
   }

   if(term & TM_MAXSIL)
   {
      // Is the recording longer than the timeout + 100 ms?
      // Note: ATDX_TRCOUNT() could be used but would require a conversion from bytes to ms
      if(1000*recordEndTime.GetSecondsSince(recordStartTime) > maxSilence + 100)
      {
// Temporary system variable until TFDesigner can warn users about changes between versions
         pInfo->variables->SetSystem("@RECORD_RESULT", "SPEECH STOP");
         return(IVR_SUCCESS);
      }
      else
      {
         ivLog->Print("      No voice detected\n");
// Removed until TFDesigner can warn users about changes between versions
//         remove(filename);
//         return(IVR_NO_VOICE);
         pInfo->variables->SetSystem("@RECORD_RESULT", "NO VOICE");
         return(IVR_SUCCESS);
      }
   }

   if(term & TM_MAXTIME)
   {
      ivLog->Print("      Maximum record time reached\n");
// Temporary system variable until TFDesigner can warn users about changes between versions
      pInfo->variables->SetSystem("@RECORD_RESULT", "MAX LENGTH");
      return(IVR_TIMEOUT);
   }

   if(term & TM_MAXDTMF)
   {
      ivLog->Print("      Interrupted\n");
// Temporary system variable until TFDesigner can warn users about changes between versions
      pInfo->variables->SetSystem("@RECORD_RESULT", "INTERRUPTED");
      return(IVR_SUCCESS);
   }

// Temporary system variable until TFDesigner can warn users about changes between versions
   pInfo->variables->SetSystem("@RECORD_RESULT", "UNKNOWN");
   return(IVR_SUCCESS);
}
/*BrdDialogic::AudioRecord*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AudioPlay                              (public)

   Purpose:     Play an audio (voice) file or a message segment of the file.

   Modified:    01-Aug-1998 Jamie Royer: Created.
                16-Mar-2002 TF: Added volume support
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::AudioPlay(
      const char* filename,      // Name of audio file to play from.
      bool interruptable, 			// Flag indicating if play can be interrupted.
      SOUND_FORMAT format,			// Type of audio file.
      int volLevel,					// Volume level.
      bool fancy,
      unsigned int playFromTime,
      unsigned int playForTime)
{
   char           ttDig;
	IV_STATUS      cState;			// Capture call status for h/u supervision
   unsigned short voladj;			// Adjusted volume suitable for Dialogic API

   if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      return(IVR_FATAL);
   }

   if(access(filename, 00) != 0)
   {
		ivLog->Print("      File does not exist\n");
   	return(IVR_FAIL);
   }

   // Dialogic Synchronous hangup supervision - TF Feb.6/02
   cState = CallStatus();
   if (CallStatus() != IVR_SUCCESS)
   	return (cState);

	QueryDigit(ttDig);
   if((ttDig != '\0') || ((pInfo->speechRecognition != NULL)
      && (pInfo->speechRecognition->SRIsReceivingAudio() || pInfo->speechRecognition->SRIsPendingAudio())))
   {
      // Reset play information, essentially record a zero-length play (for Speech Trainer to use)
      audioState->StartPlaying(audioState->PlayFormat());
      audioState->StopPlaying();
      ivLog->Print("      Interrupted!\n");
      return(IVR_SUCCESS);
   }

   // Adjust volume
   voladj = (unsigned short)(volLevel * 2);
   if (dx_adjsv(lines[curLine].chdev, SV_VOLUMETBL, SV_ABSPOS, voladj) < 0)
   {
      PrintErrorMsg("Could not adjust volume", lines[curLine].chdev);
      return(IVR_FAIL);
   }

   if(playForTime == 0)
   {
      return(IVR_SUCCESS);
   }

   if (fancy)
      return AudioPlayFileFancy(filename, format);
   else
      return AudioPlayFile(filename, format, interruptable, playFromTime, playForTime);
}
/*BrdDialogic::AudioPlay*/


/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AudioPlayBuffer                        (public)

   Purpose:     Play audio from a memory buffer.

   Modified:    2003/11/17 TF: Created.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::AudioPlayBuffer(char* audioBuffer, unsigned bufferSize,
         bool interruptable, SOUND_FORMAT format, int volLevel)
{
   int            term;
   DV_TPT         tpt[2];
   DX_XPB         xpb = GetEncoding(format);
   int            chdev = lines[curLine].chdev;
   IV_STATUS      cState;        // captures current call status
   unsigned short voladj;			// Adjusted volume suitable for Dialogic API

   if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      return(IVR_FATAL);
   }

   // Dialogic synchronous hangup supervision - TF Dec.4/01
   cState = CallStatus();
   if(cState != IVR_SUCCESS)
   {
   	return (cState);
   }

   if(audioBuffer == NULL)
   {
      ivLog->Print("      Could not open audio buffer\n");
      return(IVR_FAIL);
   }

   // Adjust volume
   voladj = (unsigned short)(volLevel * 2);
   if (dx_adjsv(lines[curLine].chdev, SV_VOLUMETBL, SV_ABSPOS, voladj) < 0)
   {
      PrintErrorMsg("Could not adjust volume", lines[curLine].chdev);
      return(IVR_FAIL);
   }

   // Must setup links before using DV_TPT structure, even to clear it - TF Dec.9/02
   tpt[0].tp_type = IO_CONT;
   tpt[1].tp_type = IO_EOT;
   dx_clrtpt (tpt, 2);

   // The "tpt" array contains the reasons to stop playback.
   // If playback is interruptable, both array entries are used.
   // If playback is uninterruptable, only the first array entry is used,
   // the second entry is ignored.
   if(interruptable)
   {
      tpt[0].tp_type = IO_CONT;
      tpt[1].tp_type = IO_EOT;
   }
   else
   {
      tpt[0].tp_type = IO_EOT;
   }

   // First entry in array: If the caller hangs up, stop playback.
   tpt[0].tp_termno = DX_LCOFF;         // LC off termination
   tpt[0].tp_length = 3;                // Use 30 ms (10 ms resolution timer)
   tpt[0].tp_flags  = TF_LCOFF|TF_10MS; // level triggered, clear history,

   // Second entry in array: If the caller presses a key, stop playback.
   if(interruptable)
   {
      tpt[1].tp_termno = DX_MAXDTMF;       // Max number of digits
      tpt[1].tp_length = 1;                // terminate on first digit
      tpt[1].tp_flags  = TF_MAXDTMF;       // terminate if already in buf.
   }

   DX_IOTT iott;

   iott.io_fhandle = NULL;
   iott.io_bufp    = audioBuffer;
   iott.io_offset  = 0;
   iott.io_length  = bufferSize;
   iott.io_type    = IO_MEM | IO_EOT;

   if(dx_playiottdata(chdev, &iott, tpt, &xpb, EV_SYNC) == -1)
   {
		PrintErrorMsg("Play buffer error\n", chdev);
      return(IVR_FAIL);
   }

   if((term = ATDX_TERMMSK(chdev)) == AT_FAILURE)
   {
      return(IVR_FATAL);
   }

   if(term & TM_LCOFF)
   {
     	return(Hangup(true));
   }

	return IVR_SUCCESS;
}
/*BrdDialogic::AudioPlayBuffer*/


/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AudioPlayFile                         (private)

   Purpose:     Play an audio (voice) file or a message segment of the file.

   Modified:    01-Aug-1998 Jamie Royer: Written for Dialogic boards.

   Testing:     [x] Plays VOX file.                           07-Aug-1998 JR
                [ ] Plays WAV file.
                [x] Interruptable.                            07-Aug-1998 JR
                [ ] Non-interruptable.
                [x] Loop current lost.                        07-Aug-1998 JR
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::AudioPlayFile(const char* filename, SOUND_FORMAT format,
   bool interruptable, unsigned int playFromTime, unsigned int playForTime)
{
   /*
    * See section 4.1.5 DX_IOTT - I/O transer table.
    * Use dx_play() to play a list of files (from section 2.1.4).
    * Use dx_playiottdata() to ???.
    */
   int     i;
   int     term;
   DV_TPT  tpt[4];
   DX_XPB  xpb = (format == TF_BY_EXTENSION) ? GetEncoding(GetExtensionFormat(filename)) : GetEncoding(format);
   IV_STATUS cState;   // captures current call status

   // Dialogic synchronous hangup supervision - TF Dec.4/01
   cState = CallStatus();
   if(cState != IVR_SUCCESS)
   {
   	return (cState);
   }

   // Must setup links before using DV_TPT structure, even to clear it - TF Dec.9/02
   // Clear the DV_TPT structure
   // Note: must setup links before using DV_TPT structure, even to clear it - TF Dec.9/02
   for(i = 0; i < 3; i++)
   {
      tpt[i].tp_type = IO_CONT;
   }
   tpt[i++].tp_type = IO_EOT;
	
   dx_clrtpt(tpt, i);

   // Set-up the DV_TPT structure
   i = 0;

   if(!disableReorderToneDetection)
   {
      tpt[i].tp_type   = IO_CONT;
      tpt[i].tp_termno = DX_TONE;          // Global Tone
      tpt[i].tp_length = 101;              // Reorder tone
      tpt[i].tp_flags  = TF_TONE;          // Terminate if tone occurs
      tpt[i].tp_data = DX_TONEON;
      i++;

      tpt[i].tp_type   = IO_CONT;
      tpt[i].tp_termno = DX_TONE;          // Global Tone
      tpt[i].tp_length = 101;              // Reorder tone
      tpt[i].tp_flags  = TF_TONE;          // Terminate if tone occurs
      tpt[i].tp_data = DX_TONEOFF;
      ++i;
   }

   tpt[i].tp_type   = (unsigned short) (interruptable ? IO_CONT : IO_EOT);
   tpt[i].tp_termno = DX_LCOFF;         // LC off termination
   tpt[i].tp_length = 3;                // Use 30 ms (10 ms resolution timer)
   tpt[i].tp_flags  = TF_LCOFF|TF_10MS; // level triggered, clear history,
   ++i;

   tpt[i].tp_type   = IO_EOT;
   tpt[i].tp_termno = DX_MAXDTMF;       // Max number of digits
   tpt[i].tp_length = 1;                // terminate on first digit
   tpt[i].tp_flags  = TF_MAXDTMF;       // terminate if already in buf.

   int      fd;
   DX_IOTT  iott;

   if((fd = dx_fileopen (filename, O_RDONLY | O_BINARY, S_IREAD)) == -1)
   {
      ivLog->Print("      ERROR: dx_fileopen failed\n");
      return(IVR_FAIL);
   }

   iott.io_fhandle = fd;
   iott.io_bufp    = 0;
   // io_offset and io_length are in bytes and so the calculation below relies
   // on a sample rate of 8 kHz.  If we start supporting audio formats with other
   // sample rates this calculation will have to be modified
   iott.io_offset  = playFromTime * xpb.wBitsPerSample;
   iott.io_length  = playForTime == FOREVER ? -1 : playForTime * xpb.wBitsPerSample;
   iott.io_type    = IO_DEV |IO_EOT;

   if(dx_playiottdata(lines[curLine].chdev, &iott, tpt, &xpb, EV_SYNC) == -1)
   {
      PrintErrorMsg("dx_playiottdata failed", lines[curLine].chdev);
      dx_fileclose(fd);
      return(IVR_FAIL);
   }

   dx_fileclose(fd);

   if((term = ATDX_TERMMSK(lines[curLine].chdev)) == AT_FAILURE)
   {
      return(IVR_FATAL);
   }

   if((term & TM_LCOFF) || (term & TM_TONE))
   {
     	return(Hangup(true));
   }

   if(term & TM_MAXDTMF)
   {
      ivLog->Print("      Interrupted\n");
   }

   audioState->SetPlayedEntireAudio();

	return(IVR_SUCCESS);
}
/*BrdDialogic::AudioPlayFile*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AudioPlayFileFancy                    (private)

   Purpose:     Synchronously play a voice file message, swallowing events
                up to and including VCEEVN_PLAY_DONE.  (All events are
                displayed, however.)

   Notes:       - This *must* be interruptable.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::AudioPlayFileFancy(
      const char*, // filename,      // name of voice file
      SOUND_FORMAT)	 // format
{
   ivLog->Print("      ERROR: Dialogic hardware does not support Message Review Mode\n");
   return(IVR_NOTSUPPORTED);
}
/*BrdDialogic::AudioPlayFileFancy*/

// ---------------------------------------------------------------------------
// Name:        BrdDialogic::AudioRecognise                          (private)
// ---------------------------------------------------------------------------
IV_STATUS BrdDialogic::AudioRecognise(int noVoiceTimeout, int)
{
   int term;
   DV_TPT tpt[4];
   DX_XPB xpb = GetEncoding(TF_WAV_8KHZ_8BIT_MULAW);
   DX_IOTT iott;            // I/O transfer table
   IV_STATUS status;
   char ttDig;

   if(!lineOffHook)
   {
      ivLog->Print("      ERROR: Port not off hook\n");
      return(IVR_FATAL);
   }

   if((status = CallStatus()) != IVR_SUCCESS)
   {
   	return(status);
   }

   if(pInfo->speechRecognition == NULL)
   {
      ivLog->Print("      ERROR: No speech recognition module loaded\n");
      ivLog->Print("         Make sure to use SR Acquire before SR steps\n");
      return(IVR_FATAL);
   }

   if((status = QueryDigit(ttDig)) != IVR_SUCCESS)
   {
      return(status);
   }

   if((ttDig != '\0') || pInfo->speechRecognition->SRIsPendingAudio())
   {
      ivLog->Print("      Interrupted!\n");
      return(ttDig != '\0' ? IVR_NO_VOICE : IVR_SUCCESS);
   }

   // Clear the DV_TPT structure
   // Note: must setup links before using DV_TPT structure, even to clear it - TF Dec.9/02
   for(int i=0; i<3; i++)
   {
      tpt[i].tp_type = IO_CONT;
   }
   tpt[3].tp_type = IO_EOT;
   dx_clrtpt(tpt, 4);

   // Set-up the DV_TPT structure
   tpt[0].tp_type = IO_CONT;
   tpt[0].tp_termno = DX_MAXTIME;       // Maximum length of recording.
   tpt[0].tp_length = (unsigned short) 100; // Tenths of seconds to record (10 s)
   tpt[0].tp_flags  = TF_MAXTIME;       // Terminate after max rec time.

   tpt[1].tp_type   = IO_CONT;
   tpt[1].tp_termno = DX_MAXSIL;        // Maximum time without voice.
   tpt[1].tp_length = (unsigned short) (noVoiceTimeout/100); // Tenths of seconds for no voice timeout
   tpt[1].tp_flags  = TF_MAXSIL;        // Terminate if no voice.

   tpt[2].tp_type   = IO_CONT;
   tpt[2].tp_termno = DX_LCOFF;         // LC off termination
   tpt[2].tp_length = 3;                // Use 30 ms (10 ms resolution timer)
   tpt[2].tp_flags  = TF_LCOFF|TF_10MS; // level triggered, clear history, 10 ms resolution

   tpt[3].tp_type   = IO_EOT;
   tpt[3].tp_termno = DX_MAXDTMF;       // Max number of digits
   tpt[3].tp_length = 1;                // terminate on first digit
   tpt[3].tp_flags  = TF_MAXDTMF;       // terminate if already in buf.

   memset(&iott, 0, sizeof(DX_IOTT));
   iott.io_type = IO_MEM | IO_EOT;      // Record to memory, last DX_IOTT block
   iott.io_bufp = pInfo->speechRecognition->SRGetAudioBuffer();         // Audio buffer
   iott.io_length = pInfo->speechRecognition->SRGetAudioBufferLength();   // Length of buffer

   audioState->SetInterruptable(true);
   pInfo->speechRecognition->SRStartAudio();
   pInfo->speechRecognition->SRSetAudioStart(iott.io_bufp);

   if(dx_reciottdata(lines[curLine].chdev, &iott, tpt, &xpb, EV_SYNC) == -1)
   {
      ivLog->Print("      ERROR: Failed to record to buffer\n");
      ivLog->Print("         %d: %s\n", ATDV_LASTERR(lines[curLine].chdev), ATDV_ERRMSGP(lines[curLine].chdev));
      return(IVR_FAIL);
   }

   pInfo->speechRecognition->SRSetAudioLength(ATDX_TRCOUNT(lines[curLine].chdev));
   pInfo->speechRecognition->SRStopAudio();

   term = ATDX_TERMMSK(lines[curLine].chdev);

   if(term == AT_FAILURE)
   {
      ivLog->Print("      ERROR: Failed to record to buffer\n");
      return(IVR_FAIL);
   }
   else if(term & TM_LCOFF)
   {
      ivLog->Print("      Caller hung up\n");
     	return(Hangup(true));
   }
   else if((term & TM_MAXSIL) || (term & TM_MAXTIME))
   {
      return(IVR_SUCCESS);
   }
   else if(term & TM_MAXDTMF)
   {
      ivLog->Print("      Interrupted by touch tone\n");
      return(IVR_NO_VOICE);
   }
   else
   {
   	ivLog->Print("      ERROR: Unhandled termination condition 0x%x\n", term);
      return(IVR_FAIL);
   }
}
//BrdDialogic::AudioRecognise

SOUND_FORMAT BrdDialogic::GetSRAudioFormat(void)
{
   return(TF_WAV_8KHZ_8BIT_MULAW);
}

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::AudioPlayEvalMessage                   (public)

   Purpose:     Plays a special TeleFlow advertisement for the evaluation
                version of TeleFlow Server.

   Modified:    03-Dec-2001 Sean Brothers: Created.
   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogic::AudioPlayEvalMessage(void)
{
   IV_STATUS status;

   HINSTANCE hLibrary = LoadLibrary("brd_dialogic.dll");
   HRSRC audioBufferHandle = FindResource(hLibrary, "EVAL_MESSAGE", "VOX");
   char* audioBuffer = (char*) LoadResource(hLibrary, audioBufferHandle);

   if(audioBuffer == NULL)
   {
      ivLog->Print("      Could not open audio buffer\n");
      return(IVR_FAIL);
   }

   status = AudioPlayBuffer(audioBuffer,
	                         SizeofResource(hLibrary, audioBufferHandle),
	                         FALSE,		// not interruptable
                            TF_VOX,		// Dialogic VOX format
                            0);			// normal volume

   return(status);
}
/*BrdDialogic::AudioPlayEvalMessage*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogic::PrintErrorMsg                         (private)

   Purpose:     Calls ATDX_LASTERR and then updates channel status text with
                error string from ATDX_ERRMSP.
   ------------------------------------------------------------------------- */
void BrdDialogic::PrintErrorMsg(const char* pszComment, int iChDev)
{
   ivLog->Print("      ERROR: %s\n", pszComment);

   if(ATDV_LASTERR(iChDev) == EDX_SYSTEM)
   {
      ivLog->Print("         System error: %d\n", errno);
   }
   else
   {
      ivLog->Print("         (%d): %s\n", ATDV_LASTERR(iChDev), ATDV_ERRMSGP(iChDev));
   }
}
/*BrdDialogic::PrintErrorMsg*/

