#ifndef XmlVoiceDBF_H
#define XmlVoiceDBF_H

#include <iostream>
using std::cout;
using std::cin;
using std::endl;

#include <string>
using std::string;

#include <vector>
using std::vector;

#include <sstream> //for string conversions

#define TIXML_USE_STL

// Warnings disabled
#pragma warn -8027
#include "..\3rdParty\tinyxml\tinyxml.h"
// Warnings enabled
#pragma warn +8027

class XmlVoiceDBF{

   public:

      //loads the Voice DBF file into memory.
      XmlVoiceDBF(string fileName);

      virtual ~XmlVoiceDBF(void);

      //Retrieves the name of the file.
      // 1 or 0 is returned on success or failure.
      int GetName(int entry, string& name);

      //Retrieves the Vox file path
      // 1 or 0 is returned on success or failure.
      int GetVoxFile(int entry, string& file);

      //Retrieves the Wav File path
      // 1 or 0 is returned on success or failure.
      int GetWavFile(int entry, string& file);

      //Retrieves the Vox Conv number
      // 1 or 0 is returned on success or failure.
      int GetVoxConv(int entry, int* name);

      //Returns the size of the Voice File list
      int Size(void);

      //Returns 1 or 0 when the file is a valid or invalid xml file. 
      int IsXmlFile(void);
     
   private:

      //Retrieves an Numeric attribute from the DOM Tree. 
      // 1 or 0 is returned on success or failure.
      int ParseRecordNumeric(int entry, string input, int* data);

      //Retrieves an String Attribute from the DOM Tree
      // 1 or 0 is returned on success or failure.
      int ParseRecordString(int entry, string input, string& data);

      //Loads all the <TeleflowVoiceScript> nodes into a vector
      // 1 or 0 is returned on success or failure.
      int LoadEntries(TiXmlElement* pElement);

      //Stores the nodes of the <TeleflowVoicescript> tags.
      vector<TiXmlNode*> XmlVoiceEntries;

      bool isXml;
      
      //Stores the xml file in memory.
      TiXmlDocument doc;
};
#endif
