#include "..\inet\Socket.h"
#include "..\escutil\iv_str.h"        // HexStr2CharStr
#include "step_UDPIPSend.h"

StepUDPIPSend::StepUDPIPSend(STEPREC& stepRecord) : Step(stepRecord)
{
   peer = stepRecord.s3.arr;
   port = stepRecord.s4.arr;
   socketHandle = stepRecord.s1.arr;
   message = stepRecord.s2.arr;
   hexString = (bool) stepRecord.u2;
}

StepUDPIPSend::~StepUDPIPSend(void)
{
}

const char* StepUDPIPSend::GetStepName(void) const
{
	return("UDP Send");
}

IV_STATUS StepUDPIPSend::Execute(PIVRINFO pInfo)
{
   IV_STATUS status = IVR_SUCCESS;
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string parsedPeer;
   unsigned short uPort;
   UDPSocket socket;
   ACE_INET_Addr peerAddress;
   unsigned int stringLength;
   int ret;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Peer address", peer.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedPeer = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Port", port.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      uPort = (unsigned short) atoi(value);
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Socket handle", socketHandle.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      socket.set_handle((ACE_HANDLE) atoi(value));
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Message", message.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else if(((stringLength = strlen(value)) > 0) && hexString)
   {
      stringLength /= 2;
      HexStr2CharStr(value, (unsigned char*) value, stringLength);
   }

   if(!bParmsValid)
   {
      delete[] value;
      return(IVR_FATAL);
   }

   if(peerAddress.set(uPort, parsedPeer.c_str()) != 0)
   {
      pInfo->ivLog->Print("      ERROR: Failed to set peer address\n");
      pInfo->ivLog->Print("         '%s'\n", Socket::WSAErrorString(errno));
      delete[] value;
      return(IVR_FAIL);
   }

   if(stringLength == 0)
   {
      pInfo->ivLog->Print("      WARNING: Empty message will not be sent\n");
   }
   else
   {
      pInfo->ivLog->Print("      Sending message: '%s'\n", value);

      if((ret = socket.SendBytes(value, stringLength, peerAddress)) != Socket::SUCCESS)
      {
         pInfo->ivLog->Print("      ERROR: Failed to send message\n");
         pInfo->ivLog->Print("         %s\n", Socket::WSAErrorString(ret));
         status = IVR_FAIL;
      }
   }

   delete[] value;

   return(status);
 }
