#include "..\escutil\odbc_db.h"
#include "step_sqlbegintrans.h"

StepSQLBeginTrans::StepSQLBeginTrans(STEPREC& stepRecord) : Step(stepRecord)
{
   odbcConnection = stepRecord.s2.arr;
}

StepSQLBeginTrans::~StepSQLBeginTrans(void)
{
}

const char* StepSQLBeginTrans::GetStepName(void) const
{
	return("SQL Begin Transaction");
}

IV_STATUS StepSQLBeginTrans::Execute(PIVRINFO pInfo)
{
   char* valueConnectHandle = NULL;
   SQLList* connection;
   IV_STATUS status;
   bool bParmsValid = true;      // Evaluate all parameters before exiting

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if (pInfo->odbcManager == NULL)
   {
      pInfo->ivLog->Print("      ERROR: SQL not initialized.\n");
      return(IVR_FATAL);
   }

   // Parse ODBC connection handle
   if((valueConnectHandle = pInfo->variables->ParseParameter("ODBC Connection", odbcConnection.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      if(strcmp(valueConnectHandle,"") == 0)
      {
         pInfo->ivLog->Print("      Using default ODBC handle\n");
      }

      // Find ODBC connection by its handle
      if((connection = pInfo->odbcManager->FindConnection(valueConnectHandle)) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: No current connection matching handle '%s'\n", valueConnectHandle);
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      delete[] valueConnectHandle;  // safe to delete, pointer was initialized to NULL
      return(IVR_FATAL);
   }

   status = connection->BeginTrans();

   return(status);
}
