#ifndef TFAPPLICATIONS_MANAGER_H
#define TFAPPLICATIONS_MANAGER_H

#include <map>
using std::map;

#include "TFEngine_communication_manager.h"
#include "TCPClientManager.h"
#include "file_log.h"
#include "..\escutil\CriticalSection.h"
#include "TFApplication.h"

// Helper class to facilitate management of the applications, including loading
// applications from a TAL file, or running TFSimulator applications.
// This class is designed to be thread-safe
class TFApplicationsManager : public TFEngineCommunicationManager
{
   public:

      TFApplicationsManager(CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub,
         TCPClientManager* tcpClientManager, FileLog& sysLog, HINSTANCE boardLibrary);
      virtual ~TFApplicationsManager(void);

      void AddApplication(int appID, TFApplication* application);
      bool Empty(void);
   	bool LoadTAL(void);
      void PrintRunningApplications(void);
      void RemoveApplication(int appID);
      void RemoveApplications(void);
      bool StartApplication(int appID);
      bool StartApplications(void);
      bool StartSimulation(int appID);
      void StopApplication(int appID);
      void StopApplications(void);

   protected:

      Mutex mutex;
      map<int, TFApplication*> applications;
      CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID>* commHub;
      TCPClientManager* tcpClientManager;
      FileLog& sysLog;
      HINSTANCE boardLibrary;
};

#endif
