#ifndef TFTCPCLIENT_COMMUNICATION_H
#define TFTCPCLIENT_COMMUNICATION_H

#include "..\inet\Socket.h"
#include "..\TFServer\ivr_defs.h"
#include "..\TFServer\communication_hub.h"

class TFTCPClientCommunicationPacket : public CommunicationPacket
{
   public:

      enum TF_TCPCLIENT_EVENT
      {
         // Sent from ServerMonClient to ServerMonitorMain to indicate that ServerMonClient
         // has shut down, or sent from ServerMonitorMain to ServerMonClient to indicate
         // that ServerMonClient should shut down
         TCPCLIENT_EVT_SHUTDOWN,

         // Sent from engine to indicate that streaming has started
         TCPCLIENT_EVT_START_STREAM,

         // Sent from engine to indicate that streaming has stopped
         TCPCLIENT_EVT_STOP_STREAM,

         // Sent from engine to indicate that streaming has paused
         TCPCLIENT_EVT_PAUSE_STREAM
      };

      TFTCPClientCommunicationPacket(TCPSocket* client, TF_TCPCLIENT_EVENT event);

      // Pointer to identify TFTCPClient
      TCPSocket* client;
      TF_TCPCLIENT_EVENT event;

      // Thread ID of the TFTCPClient sending a TCPCLIENT_EVT_SHUTDOWN event
      int shutDownThread;

      // Format being streamed in a TCPCLIENT_EVT_START_STREAM event
      SOUND_FORMAT format;
};

// Message destination for ServerMonClient
// Note that messages must be TFTCPClientCommunicationPackets
// Messages for ServerMonClients must use TF_TCPCLIENT_MANAGER as their destination
// Messages with a threadID of 0 indicate a message for ServerMonitorMain
class TFTCPClientCommunicationDestination : public CommunicationDestination<int>
{
   public:

      TFTCPClientCommunicationDestination(int threadID);

      // Identified by ServerMonClient thread ID
      int ID(void);

   protected:
   
      int threadID;
};

#endif