/* -------------------------------------------------------------------------
   File:        LOG.H

   Copyright (c) 1997 Engage! Software Corporation.  All rights reserved.
   ------------------------------------------------------------------------- */
#ifndef IV_LOG_H
#define IV_LOG_H

#include <stdio.h>
#include <string>
using std::string;

#include "..\escutil\ESCTime.h"

/* -------------------------------------------------------------------------
   Class:       Log

   Purpose:     This abstract base class implements a generic logging class.

                This logging class allows logging to a console/GUI window
                and optionally logging simultaneously to a log file.

                Since file logging and debugging options are common to all
                logging classes, these functions are implemented here.
   ------------------------------------------------------------------------- */
// class LOGLIBAPI Log
class Log
{
   public:

      Log(int maxLogFileSize = 500);
      virtual ~Log(void);

      virtual void Open(const char* s, bool bAppend = false, bool bUnbuffered = false);
      virtual void Close(void);

      virtual void EnableScreenOutput(bool bEnable);

      virtual void PrintLine(const std::string& asLine, bool bEOL) = 0;

      virtual void SetDebug(int pos);
      virtual void ClearDebug(int pos);
      virtual bool GetDebug(int pos) const;

      virtual void EnableTimeStamp(bool bEnable);

      virtual int Print(const char* format, ...);
      virtual int Debug(int pos, const char* format, ...);

      virtual void Disable(bool disable);

   protected:

      FILE* fp;
      string logName;
      bool bFileAppend;
      bool bFileUnbuffered;
   	int iMaxFileSize;
      bool disabled;
      ESCTime currentFileTime;

   private:

      bool debug[256];
      bool toScreen;
      bool bTimeStamp;
      bool bAtLeftMargin;

      void Stamp(char* pszDate);

      // Prevent the compile from generating a copy constructor or assignment
      // operator.
      Log(const Log&);
      Log& operator=(const Log&);

      void SeparateLines(char* pszLine);
};

// #undef LOGLIBAPI

#endif
