#include "InternetMessageTCPTransaction.h"

InternetMessageTCPTransaction::InternetMessageTCPTransaction(bool persistant) : TCPSocket(persistant)
{
   request = NULL;
   response = NULL;
   peerPort = Socket::ANY_PORT;
}

InternetMessageTCPTransaction::InternetMessageTCPTransaction(ACE_HANDLE handle) : TCPSocket(handle)
{
   request = NULL;
   response = NULL;
   peerPort = Socket::ANY_PORT;
}

InternetMessageTCPTransaction::~InternetMessageTCPTransaction(void)
{
}

int InternetMessageTCPTransaction::Connect(void)
{
   ACE_INET_Addr peerAddress;
   int ret;

   if(peerAddress.set(peerPort, peerName.c_str()) != 0)
   {
      Response()->SetText(Socket::WSAErrorString(errno));
      return(errno);
   }

   if((ret = TCPSocket::Connect(peerAddress)) != Socket::SUCCESS)
   {
      Response()->SetText(Socket::WSAErrorString(ret));
   }

   return(ret);
}

int InternetMessageTCPTransaction::Connect(const ACE_Addr& peer, const ACE_Addr& host)
{
   return(TCPSocket::Connect(peer, host));
}

int InternetMessageTCPTransaction::Send(void)
{
   int ret;

   Response()->ClearText();

   if((ret = SendBytes(Request()->GetText(), strlen(Request()->GetText()))) != Socket::SUCCESS)
   {
      Response()->SetText(Socket::WSAErrorString(ret));
   }

   return(ret);
}

int InternetMessageTCPTransaction::Receive(void)
{
   const char* responseText;
   int ret;

   if((ret = ReceiveString(responseText)) == Socket::SUCCESS)
   {
      Response()->AddText(responseText);
   }

   return(ret);
}

const char* InternetMessageTCPTransaction::GetPeerName(void)
{
   return(peerName.c_str());
}

unsigned short InternetMessageTCPTransaction::GetPeerPort(void)
{
   return(peerPort);
}

InternetMessage* InternetMessageTCPTransaction::Request(void)
{
   return(request);
}

InternetMessage* InternetMessageTCPTransaction::Response(void)
{
   return(response);
}

void InternetMessageTCPTransaction::SetPeerName(const char* name)
{
   peerName = name;
}

void InternetMessageTCPTransaction::SetPeerPort(unsigned short port)
{
   peerPort = port;
}


