#include <string>
using std::string;

// Warning en/disables are for Borland C++Builder, ignore in MS VC++
#ifndef _MSC_VER
// Warning disabled because the ACE code is not to our standards
#pragma warn -8061
#endif
#include "ace\ssl\SSL_SOCK_Connector.h"
#include "ace\Time_Value.h"
// Warning re-enabled because our code is
#ifndef _MSC_VER
#pragma warn +8061
#endif

#include "Socket.h"

Socket::Socket(bool persistant) : persistant(persistant)
{
   eventObject = CreateEvent(0, false, false, 0);
}

Socket::~Socket(void)
{
   if(eventObject != NULL)
   {
      CloseHandle(eventObject);
   }
}

int Socket::Close(void)
{
   int ret = SelectEvents(0);
   return(close() != 0 ? GenerateError() : ret);
}

int Socket::GenerateError(void) const
{
   int sslError = ERR_get_error();
   return(sslError != 0 ? sslError : errno);
}

int Socket::GetEvents(int& events) const
{
   WSANETWORKEVENTS networkEvents;

   if(get_handle() == ACE_INVALID_HANDLE)
   {
      errno = WSAENOTCONN;
      return(errno);
   }

   if(WSAEnumNetworkEvents((SOCKET) get_handle(), eventObject, &networkEvents) == SOCKET_ERROR)
   {
      return(WSAGetLastError());
   }

   events = networkEvents.lNetworkEvents;

   return(Socket::SUCCESS);
}

HANDLE Socket::GetEventHandle(void) const
{
   return(eventObject);
}

const char* Socket::GetHostAddress(void)
{
   ACE_INET_Addr hostAddress;
   // This hack is unnecessary after ACE 5.4.2
   hostAddress.set((unsigned short) ANY_PORT, hostAddress.get_host_name());
   return(hostAddress.get_host_addr());
}

int Socket::GetHostAddress(ACE_INET_Addr& host) const
{
   if(get_local_addr(host) == 0)
   {
      // This hack should be unnecessary after ACE 5.4.2
      host.set(host.get_port_number(), host.get_host_name());
      return(Socket::SUCCESS);
   }
   else
   {
      return(GenerateError());
   }
}

const char* Socket::GetHostName(void)
{
   ACE_INET_Addr hostAddress;
   return(hostAddress.get_host_name());
}

int Socket::GetStatus(void) const
{
   int status;
   int length = sizeof(int);
   get_option(SOL_SOCKET, SO_ERROR, &status, &length);
   return(status);
}

int Socket::NetShutdown(void)
{
   return(WSACleanup() == SOCKET_ERROR ? WSAGetLastError() : Socket::SUCCESS);
}

int Socket::NetStartup(void)
{
   // Will use Winsock DLL versions 0 to 2
   WORD WINSOCK_LOW = 0;
   WORD WINSOCK_HIGH = 2;
   WORD wVersionRequested = (WORD) (WINSOCK_HIGH | (WINSOCK_LOW << 8));
   WSADATA wsaData;
   int ret = WSAStartup(wVersionRequested, &wsaData);
   ERR_load_SSL_strings();
   return(ret != 0 ? ret : Socket::SUCCESS);
}

int Socket::SelectEvents(long lNetworkEvents) const
{
   if(enable(ACE_NONBLOCK) != 0)
   {
      return(GenerateError());
   }

   return(WSAEventSelect((SOCKET) get_handle(), eventObject, lNetworkEvents) == SOCKET_ERROR ? WSAGetLastError() : Socket::SUCCESS);
}

static string SSLErrorString;

const char* Socket::WSAErrorString(int errorCode)
{
   switch(errorCode)
   {
      case Socket::SUCCESS:        return("SUCCESS");
      // Windows Sockets definitions of regular Microsoft C error constants
      case WSAEINTR:               return("WSAEINTR");
      case WSAEBADF:               return("WSAEBADF");
      case WSAEACCES:              return("WSAEACCES");
      case WSAEFAULT:              return("WSAEFAULT");
      case WSAEINVAL:              return("WSAEINVAL");
      case WSAEMFILE:              return("WSAEMFILE");
      // Windows Sockets definitions of regular Berkeley error constants
      case WSAEWOULDBLOCK:         return("WSAEWOULDBLOCK");
      case WSAEINPROGRESS:         return("WSAEINPROGRESS");
      case WSAEALREADY:            return("WSAEALREADY");
      case WSAENOTSOCK:            return("WSAENOTSOCK");
      case WSAEDESTADDRREQ:        return("WSAEDESTADDRREQ");
      case WSAEMSGSIZE:            return("WSAEMSGSIZE");
      case WSAEPROTOTYPE:          return("WSAEPROTOTYPE");
      case WSAENOPROTOOPT:         return("WSAENOPROTOOPT");
      case WSAEPROTONOSUPPORT:     return("WSAEPROTONOSUPPORT");
      case WSAESOCKTNOSUPPORT:     return("WSAESOCKTNOSUPPORT");
      case WSAEOPNOTSUPP:          return("WSAEOPNOTSUPP");
      case WSAEPFNOSUPPORT:        return("WSAEPFNOSUPPORT");
      case WSAEAFNOSUPPORT:        return("WSAEAFNOSUPPORT");
      case WSAEADDRINUSE:          return("WSAEADDRINUSE");
      case WSAEADDRNOTAVAIL:       return("WSAEADDRNOTAVAIL");
      case WSAENETDOWN:            return("WSAENETDOWN");
      case WSAENETUNREACH:         return("WSAENETUNREACH");
      case WSAENETRESET:           return("WSAENETRESET");
      case WSAECONNABORTED:        return("WSAECONNABORTED");
      case WSAECONNRESET:          return("WSAECONNRESET");
      case WSAENOBUFS:             return("WSAENOBUFS");
      case WSAEISCONN:             return("WSAEISCONN");
      case WSAENOTCONN:            return("WSAENOTCONN");
      case WSAESHUTDOWN:           return("WSAESHUTDOWN");
      case WSAETOOMANYREFS:        return("WSAETOOMANYREFS");
      case WSAETIMEDOUT:           return("WSAETIMEDOUT");
      case WSAECONNREFUSED:        return("WSAECONNREFUSED");
      case WSAELOOP:               return("WSAELOOP");
      case WSAENAMETOOLONG:        return("WSAENAMETOOLONG");
      case WSAEHOSTDOWN:           return("WSAEHOSTDOWN");
      case WSAEHOSTUNREACH:        return("WSAEHOSTUNREACH");
      case WSAENOTEMPTY:           return("WSAENOTEMPTY");
      case WSAEPROCLIM:            return("WSAEPROCLIM");
      case WSAEUSERS:              return("WSAEUSERS");
      case WSAEDQUOT:              return("WSAEDQUOT");
      case WSAESTALE:              return("WSAESTALE");
      case WSAEREMOTE:             return("WSAEREMOTE");
      // Extended Windows Sockets error constant definitions
      case WSASYSNOTREADY:         return("WSASYSNOTREADY");
      case WSAVERNOTSUPPORTED:     return("WSAVERNOTSUPPORTED");
      case WSANOTINITIALISED:      return("WSANOTINITIALISED");
      case WSAEDISCON:             return("WSAEDISCON");
      case WSAENOMORE:             return("WSAENOMORE");
      case WSAECANCELLED:          return("WSAECANCELLED");
      case WSAEINVALIDPROCTABLE:   return("WSAEINVALIDPROCTABLE");
      case WSAEINVALIDPROVIDER:    return("WSAEINVALIDPROVIDER");
      case WSAEPROVIDERFAILEDINIT: return("WSAEPROVIDERFAILEDINIT");
      case WSASYSCALLFAILURE:      return("WSASYSCALLFAILURE");
      case WSASERVICE_NOT_FOUND:   return("WSASERVICE_NOT_FOUND");
      case WSATYPE_NOT_FOUND:      return("WSATYPE_NOT_FOUND");
      case WSA_E_NO_MORE:          return("WSA_E_NO_MORE");
      case WSA_E_CANCELLED:        return("WSA_E_CANCELLED");
      case WSAEREFUSED:            return("WSAEREFUSED");

      // Error return codes from gethostbyname and gethostbyaddr.
      case WSAHOST_NOT_FOUND:      return("WSAHOST_NOT_FOUND");
      case WSATRY_AGAIN:           return("WSATRY_AGAIN");
      case WSANO_RECOVERY:         return("WSANO_RECOVERY");
      case WSANO_DATA:             return("WSANO_DATA");
   // case WSANO_ADDRESS:  Maps to WSANO_DATA.
      default:
      {
         if((errorCode >= 0x01000000) && ((unsigned int) errorCode < 0xFFFFFFFF))
         {
            SSLErrorString = ERR_lib_error_string(errorCode) == NULL ? "none" : ERR_lib_error_string(errorCode);
            SSLErrorString += (string) ", " + (string) (ERR_func_error_string(errorCode) == NULL ? "none" : ERR_func_error_string(errorCode));
            SSLErrorString += (string) ", " + (string) (ERR_reason_error_string(errorCode) == NULL ? "none" : ERR_reason_error_string(errorCode));

            if(errorCode == ((ERR_LIB_SSL << 24) | (SSL_F_SSL23_GET_SERVER_HELLO << 12) | SSL_R_UNKNOWN_PROTOCOL))
            {
               SSLErrorString += ", Possibly non-SSL port";
            }

            return(SSLErrorString.c_str());
         }
         else
         {
            return("Unknown");
         }
      }
   }
}

TCPSocket::TCPSocket(bool persistant) : Socket(persistant), ACE_SSL_SOCK_Stream()
{
   isSSLEnabled = false;
   buffer = NULL;
}

TCPSocket::TCPSocket(ACE_HANDLE handle) : Socket(true), ACE_SSL_SOCK_Stream()
{
   isSSLEnabled = false;
   buffer = NULL;
   set_handle(handle);
   if(handle != ACE_INVALID_HANDLE)
   {
      SelectEvents(FD_READ | FD_CLOSE);
   }
}

TCPSocket::~TCPSocket(void)
{
   RemoveSocket(get_handle());

   delete[] buffer;

   if(!persistant)
   {
      Close();
   }
}

static map<ACE_HANDLE, TCPSocket*> TCPSockets;
static Mutex TCPSocketsMutex;

void TCPSocket::AddSocket(ACE_HANDLE handle, TCPSocket* socket)
{
   TCPSocketsMutex.Get();
   TCPSockets[handle] = socket;
   TCPSocketsMutex.Release();
}

int TCPSocket::close(void)
{
   RemoveSocket(get_handle());
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::close() : stream_.close());
}

int TCPSocket::Connect(const ACE_Addr& peer, const ACE_Addr& host)
{
   ACE_Time_Value timeout(0, 0);
   int ret;

   if(isSSLEnabled)
   {
      ACE_SSL_SOCK_Connector socketConnector;

      // Allow connections that already exist, are already in process to be considered
      // successes
      ret = socketConnector.connect(*this, peer, &timeout, host);
   }
   else
   {
      ACE_SOCK_Connector socketConnector;

      // Allow connections that already exist, are already in process to be considered
      // successes
      ret = socketConnector.connect(stream_, peer, &timeout, host);
   }

   if(ret == -1)
   {
      ret = GenerateError();

      switch(ret)
      {
         case WSAEISCONN:
         {
            ret = Socket::SUCCESS;
            break;
         }

         case WSAEALREADY:
         case WSAEWOULDBLOCK:
         {
            ret = SelectEvents(FD_CONNECT);
            break;
         }

         default:
         {
            return(ret);
         }
      }
   }

   AddSocket(get_handle(), this);

   return(ret);
}

int TCPSocket::DisableSSL(void)
{
   if(get_handle() != ACE_INVALID_HANDLE)
   {
      errno = WSAEISCONN;
      return(errno);
   }

   isSSLEnabled = false;

   return(Socket::SUCCESS);
}

int TCPSocket::enable(int value) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::enable(value) : stream_.enable(value));
}

int TCPSocket::EnableSSL(void)
{
   if(get_handle() != ACE_INVALID_HANDLE)
   {
      errno = WSAEISCONN;
      return(errno);
   }

   isSSLEnabled = true;

   return(Socket::SUCCESS);
}

int TCPSocket::GetBytesWaiting(int& bytesWaiting) const
{
   if(isSSLEnabled)
   {
      int SSLBytesWaiting = SSL_pending(ssl());

      if(SSLBytesWaiting == -1)
      {
         return(GenerateError());
      }
      else if(ACE_SSL_SOCK_Stream::control(FIONREAD, &bytesWaiting) == -1)
      {
         return(GenerateError());
      }
      else
      {
         bytesWaiting += SSLBytesWaiting;
         return(Socket::SUCCESS);
      }
   }
   else
   {
      return(stream_.control(FIONREAD, &bytesWaiting) != 0 ? GenerateError() : Socket::SUCCESS);
   }
}

ACE_HANDLE TCPSocket::get_handle(void) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::get_handle() : stream_.get_handle());
}

int TCPSocket::get_local_addr(ACE_Addr& addr) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::get_local_addr(addr) : stream_.get_local_addr(addr));
}

int TCPSocket::get_option(int level, int option, void* optval, int* optlen) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::get_option(level, option, optval, optlen) : stream_.get_option(level, option, optval, optlen));
}

int TCPSocket::get_remote_addr(ACE_Addr& addr) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::get_remote_addr(addr) : stream_.get_remote_addr(addr));
}

int TCPSocket::GetPeerAddress(ACE_Addr& peer) const
{
   return(get_remote_addr(peer) == -1 ? GenerateError() : Socket::SUCCESS);
}

TCPSocket* TCPSocket::GetSocket(ACE_HANDLE handle)
{
   TCPSocket* socket;
   TCPSocketsMutex.Get();
   socket = TCPSockets.count(handle) == 0 ? NULL : TCPSockets[handle];
   TCPSocketsMutex.Release();
   return(socket);
}

bool TCPSocket::IsSSLEnabled(void) const
{
   return(isSSLEnabled);
}

int TCPSocket::ReceiveBytes(const char*& message, int& length)
{
	int bytesWaiting;
   int ret;

   delete[] buffer;
   buffer = NULL;

   // Check for valid length
   if(length <= 0)
   {
      message = NULL;
      length = -1;
      errno = WSAEINVAL;
      return(errno);
   }

   // Check if data is waiting
	if((ret = GetBytesWaiting(bytesWaiting)) != Socket::SUCCESS)
	{
      // Return NULL on error
      message = NULL;
      length = -1;
		return(ret);
	}

   // Return NULL if no data is waiting
   if(bytesWaiting == 0)
	{
      int events = 0;
      message = NULL;
      GetEvents(events);
      length = (events & FD_CLOSE) == 0 ? 0 : -1;
		return((events & FD_CLOSE) == 0 ? Socket::SUCCESS : WSAECONNRESET);
	}

   // Take the minimum of the maximum length and bytesWaiting
   if(bytesWaiting < length)
   {
      length = bytesWaiting;
   }

   // Allocate new storage for receiving data
   buffer = new char[length + 1];

   // Receive data
   length = recv(buffer, length);

   // Validate message and guarantee it is null terminated
   if (length > 0)
   {
      buffer[length] = '\0';
      message = buffer;
   }
   else
   {
      message = NULL;
   }

   if(isSSLEnabled && (GetBytesWaiting(bytesWaiting) == Socket::SUCCESS) && (bytesWaiting > 0))
   {
      SetEvent(eventObject);
   }

   return(length == -1 ? GenerateError() : Socket::SUCCESS);
}

int TCPSocket::ReceiveLine(const char*& message)
{
	int bytesWaiting;
   int length;
   int ret;

   delete[] buffer;
   buffer = NULL;

   // Check if data is waiting
	if((ret = GetBytesWaiting(bytesWaiting)) != Socket::SUCCESS)
	{
      // Return NULL on error
      message = NULL;
		return(ret);
	}

   // Return "\n" if no data is waiting
   if(bytesWaiting == 0)
	{
      int events = 0;
      message = "\n";
      GetEvents(events);
		return((events & FD_CLOSE) == 0 ? Socket::SUCCESS : WSAECONNRESET);
	}

   // Allocate new storage for receiving data with room to pad '\n' and '\0' terminators
   buffer = new char[bytesWaiting + 2];

   // Copy data (MSG_PEEK flag means data is copied rather than moved from O/S socket)
   if(recv(buffer, bytesWaiting, MSG_PEEK) == -1)
   {
      // Return NULL on error
      message = NULL;
		return(GenerateError());
   }

   // Check data for terminators and calculate length
   for(length = 0; (length < bytesWaiting) && (buffer[length] != '\0') && (buffer[length] != '\n'); length++);

   // Adjust length for terminators
   if((length < bytesWaiting) && (buffer[length] == '\n'))
   {
      length++;
   }

   if((length < bytesWaiting) && (buffer[length] == '\0'))
   {
      length++;
   }

   // Receive data
   if(recv(buffer, length) == -1)
   {
      int ret = GenerateError();

      if(ret != WSAEMSGSIZE)
      {
         // Return NULL on error
         message = NULL;
   		return(ret);
      }
   }

   // Add terminators if needed
   if(buffer[length - 1] == '\n')
   {
      buffer[length] = '\0';
   }
   else if(buffer[length - 2] != '\n')
   {
      buffer[length] = '\n';
      buffer[length + 1] = '\0';
   }

   message = buffer;

   if(isSSLEnabled && (GetBytesWaiting(bytesWaiting) == Socket::SUCCESS) && (bytesWaiting > 0))
   {
      SetEvent(eventObject);
   }

   return(Socket::SUCCESS);
}

int TCPSocket::ReceiveString(const char*& message)
{
	int bytesWaiting;
   int length;
   int ret;

   delete[] buffer;
   buffer = NULL;

   // Check if data is waiting
	if((ret = GetBytesWaiting(bytesWaiting)) != Socket::SUCCESS)
	{
      // Return NULL on error
      message = NULL;
		return(ret);
	}

   // Return empty string if no data is waiting
   if(bytesWaiting == 0)
	{
      int events = 0;
      message = "";
      GetEvents(events);
		return((events & FD_CLOSE) == 0 ? Socket::SUCCESS : WSAECONNRESET);
	}

   // Allocate new storage for receiving data with room to pad '\n' and '\0' terminators
   buffer = new char[bytesWaiting + 1];

   // Copy data (MSG_PEEK flag means data is copied rather than moved from O/S socket)
   if(recv(buffer, bytesWaiting, MSG_PEEK) == -1)
   {
      // Return NULL on error
      message = NULL;
		return(GenerateError());
   }

   // Check data for terminator and calculate length
   for(length = 0; (length < bytesWaiting) && (buffer[length] != '\0'); length++);

   if((length < bytesWaiting) && (buffer[length] == '\0'))
   {
      length++;
   }

   // Receive data
   if(recv(buffer, length) == -1)
   {
      int ret = GenerateError();

      if(ret != WSAEMSGSIZE)
      {
         // Return NULL on error
         message = NULL;
   		return(ret);
      }
   }

   // Add terminator
   buffer[length] = '\0';

   message = buffer;

   if(isSSLEnabled && (GetBytesWaiting(bytesWaiting) == Socket::SUCCESS) && (bytesWaiting > 0))
   {
      SetEvent(eventObject);
   }

   return(Socket::SUCCESS);
}

ssize_t TCPSocket::recv(void* buf, size_t n) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::recv(buf, n) : stream_.recv(buf, n));
}

ssize_t TCPSocket::recv(void* buf, size_t n, int flags) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::recv(buf, n, flags) : stream_.recv(buf, n, flags));
}

TCPSocket* TCPSocket::RemoveSocket(ACE_HANDLE handle)
{
   TCPSocket* socket = NULL;
   TCPSocketsMutex.Get();
   if(TCPSockets.count(handle) != 0)
   {
      socket = TCPSockets[handle];
      TCPSockets.erase(handle);
   }
   TCPSocketsMutex.Release();
   return(socket);
}

ssize_t TCPSocket::send(const void* buf, size_t n) const
{
   return(isSSLEnabled ? ACE_SSL_SOCK_Stream::send(buf, n) : stream_.send(buf, n));
}

int TCPSocket::SendBytes(const char* message, int length) const
{
   return(send(message, length) == -1 ? GenerateError() : Socket::SUCCESS);
}

int TCPSocket::SendLine(const char* message) const
{
   int length = strlen(message) + 1;
   string newLine = message;

   // Pad with '\n' if needed
   if(message[length - 2] != '\n')
   {
      newLine += '\n';
      length++;
   }

   // Send line
   return(send(newLine.c_str(), length) == -1 ? GenerateError() : Socket::SUCCESS);
}

int TCPSocket::SendString(const char* message) const
{
   // Send string including '\0' terminator
   return(send(message, strlen(message) + 1) == -1 ? GenerateError() : Socket::SUCCESS);
}

void TCPSocket::set_handle(ACE_HANDLE handle)
{
   ACE_HANDLE oldHandle = get_handle();

   if(oldHandle != ACE_INVALID_HANDLE)
   {
      RemoveSocket(oldHandle);
   }

   if(isSSLEnabled)
   {
      ACE_SSL_SOCK_Stream::set_handle(handle);
   }
   else
   {
      stream_.set_handle(handle);
   }

   if(handle != ACE_INVALID_HANDLE)
   {
      AddSocket(handle, this);
   }
}

int TCPSocket::SSLHandshake(void)
{
   ACE_SSL_SOCK_Connector socketConnector;
   int ret = socketConnector.complete(*this, NULL, NULL) == -1 ? GenerateError() : Socket::SUCCESS;
   SelectEvents(FD_READ | FD_CLOSE);
   return(ret);
}

UDPSocket::UDPSocket(unsigned short port, bool persistant) : Socket(persistant), ACE_SOCK_Dgram()
{
   ACE_INET_Addr hostAddress;
   buffer = NULL;
   hostAddress.set(port);
   open(hostAddress);
   SelectEvents(FD_READ);
}

UDPSocket::UDPSocket(ACE_HANDLE handle) : Socket(true), ACE_SOCK_Dgram()
{
   buffer = NULL;
   if(handle != ACE_INVALID_HANDLE)
   {
      set_handle(handle);
      SelectEvents(FD_READ);
   }
}

UDPSocket::~UDPSocket(void)
{
   delete[] buffer;

   if(!persistant)
   {
      Close();
   }
}

int UDPSocket::Bind(unsigned short port)
{
   ACE_INET_Addr hostAddress;
   Close();
   hostAddress.set(port);
   if(open(hostAddress) != 0)
   {
      return(errno);
   }
   return(SelectEvents(FD_READ));
}

int UDPSocket::close(void)
{
   return(ACE_SOCK_Dgram::close());
}

int UDPSocket::enable(int value) const
{
   return(ACE_SOCK_Dgram::enable(value));
}

ACE_HANDLE UDPSocket::get_handle(void) const
{
   return(ACE_SOCK_Dgram::get_handle());
}

int UDPSocket::get_local_addr(ACE_Addr& addr) const
{
   return(ACE_SOCK_Dgram::get_local_addr(addr));
}

int UDPSocket::get_option(int level, int option, void* optval, int* optlen) const
{
   return(ACE_SOCK_Dgram::get_option(level, option, optval, optlen));
}

int UDPSocket::GetBytesWaiting(int& bytesWaiting) const
{
   return(control(FIONREAD, &bytesWaiting) != 0 ? errno : Socket::SUCCESS);
}

int UDPSocket::ReceiveBytes(const char*& message, int& length, ACE_Addr& peer)
{
	int bytesWaiting;
   int ret;

   delete[] buffer;
   buffer = NULL;

   // Check if data is waiting
   if((ret = GetBytesWaiting(bytesWaiting)) != Socket::SUCCESS)
	{
      // Return NULL on error
      message = NULL;
      length = -1;
		return(ret);
	}

   // Return NULL if no data is waiting
   if(bytesWaiting == 0)
	{
      message = NULL;
      length = 0;
		return(Socket::SUCCESS);
	}

   // Allocate new storage for receiving data
   buffer = new char[bytesWaiting];

   // Receive data
   length = recv(buffer, bytesWaiting, peer);

   message = length == -1 ? NULL : buffer;
   return(length == -1 ? errno : Socket::SUCCESS);
}

int UDPSocket::ReceiveLine(const char*& message, ACE_Addr& peer)
{
	int bytesWaiting;
   int length;
   int ret;

   delete[] buffer;
   buffer = NULL;

   // Check if data is waiting
   if((ret = GetBytesWaiting(bytesWaiting)) != Socket::SUCCESS)
	{
      // Return NULL on error
      message = NULL;
		return(ret);
	}

   // Return "\n" if no data is waiting
   if(bytesWaiting == 0)
	{
      message = "\n";
		return(Socket::SUCCESS);
	}

   // Allocate new storage for receiving data with room to pad '\n' and '\0' terminators
   buffer = new char[bytesWaiting + 2];

   // Receive data
   if((recv(buffer, bytesWaiting, peer) == -1) && (errno != WSAEMSGSIZE))
   {
      // Return NULL on error
      message = NULL;
		return(errno);
   }

   // Check data for terminators and calculate length
   for(length = 0; (length < bytesWaiting) && (buffer[length] != '\0') && (buffer[length] != '\n'); length++);

   // Add terminators
   buffer[length] = '\n';
   buffer[length + 1] = '\0';

   message = buffer;
   return(Socket::SUCCESS);
}

int UDPSocket::ReceiveString(const char*& message, ACE_Addr& peer)
{
	int bytesWaiting;
   int length;
   int ret;

   delete[] buffer;
   buffer = NULL;

   // Check if data is waiting
   if((ret = GetBytesWaiting(bytesWaiting)) != Socket::SUCCESS)
	{
      // Return NULL on error
      message = NULL;
		return(ret);
	}

   // Return "" if no data is waiting
   if(bytesWaiting == 0)
	{
      message = "";
		return(Socket::SUCCESS);
	}

   // Allocate new storage for receiving data with room to pad '\0' terminator
   buffer = new char[bytesWaiting + 1];

   // Receive data
   if((recv(buffer, bytesWaiting, peer) == -1) && (errno != WSAEMSGSIZE))
   {
      // Return NULL on error
      message = NULL;
		return(errno);
   }

   // Check data for terminator and calculate length
   for(length = 0; (length < bytesWaiting) && (buffer[length] != '\0'); length++);

   // Add terminator
   buffer[length] = '\0';

   message = buffer;
   return(Socket::SUCCESS);
}

int UDPSocket::SendBytes(const char* message, int length, const ACE_Addr& peer) const
{
   return(send(message, length, peer) == -1 ? errno : Socket::SUCCESS);
}

int UDPSocket::SendLine(const char* message, const ACE_Addr& peer) const
{
   int length = strlen(message) + 1;
   string newLine = message;

   // Pad with '\n' if needed
   if(message[length - 2] != '\n')
   {
      newLine += '\n';
      length++;
   }

   // Send line
   return(send(newLine.c_str(), length, peer) == -1 ? errno : Socket::SUCCESS);
}

int UDPSocket::SendString(const char* message, const ACE_Addr& peer) const
{
   // Send string including '\0' terminator
   return(send(message, strlen(message) + 1, peer) == -1 ? errno : Socket::SUCCESS);
}

SocketServer::SocketServer(unsigned short port, bool persistant, int maxPendingConnections)
   : Socket(persistant), ACE_SOCK_Acceptor()
{
   ACE_INET_Addr hostAddress;
   hostAddress.set(port);
   open(hostAddress, 0, PF_UNSPEC, maxPendingConnections, 0);
   SelectEvents(FD_ACCEPT);
}

SocketServer::SocketServer(ACE_HANDLE handle) : Socket(true), ACE_SOCK_Acceptor()
{
   set_handle(handle);
   if(handle != ACE_INVALID_HANDLE)
   {
      SelectEvents(FD_ACCEPT);
   }
}

SocketServer::~SocketServer(void)
{
   if(!persistant)
   {
      Close();
   }
}

int SocketServer::Accept(TCPSocket& newConnection) const
{
   ACE_Time_Value timeout(0, 0);

   if(accept(newConnection.peer(), NULL, &timeout) == -1)
   {
      return(GenerateError());
   }

   return(newConnection.SelectEvents(FD_READ | FD_CLOSE));
}

int SocketServer::Get_Port(void) const
{
   ACE_INET_Addr Address;
   int port = Socket::ANY_PORT;
   int result = get_local_addr(Address);
   
   if(result == Socket::SUCCESS)
   {
      port = Address.get_port_number();
   }

   return(port);
}

int SocketServer::close(void)
{
   return(ACE_SOCK_Acceptor::close());
}

int SocketServer::enable(int value) const
{
   return(ACE_SOCK_Acceptor::enable(value));
}

ACE_HANDLE SocketServer::get_handle(void) const
{
   return(ACE_SOCK_Acceptor::get_handle());
}

int SocketServer::get_local_addr(ACE_Addr& addr) const
{
   return(ACE_SOCK_Acceptor::get_local_addr(addr));
}

int SocketServer::get_option(int level, int option, void* optval, int* optlen) const
{
   return(ACE_SOCK_Acceptor::get_option(level, option, optval, optlen));
}
